@@ -20,7 +20,9 @@
 
 #include "libavutil/opt.h"
 #include "libavutil/channel_layout.h"
+#include "libavutil/thread.h"
 
+#include "codec_internal.h"
 #include "dcadec.h"
 #include "dcahuff.h"
 #include "dca_syncwords.h"
@@ -41,13 +43,17 @@ int ff_dca_set_channel_layout(AVCodecContext *avctx, int *ch_remap, int dca_mask
         13, 14, 3, 9, 10, 11, 12, 14, 16, 15, 17, 8, 4,  5,
     };
 
+    DCAContext *s = avctx->priv_data;
+
     int dca_ch, wav_ch, nchannels = 0;
 
-    if (avctx->request_channel_layout & AV_CH_LAYOUT_NATIVE) {
+    av_channel_layout_uninit(&avctx->ch_layout);
+    if (s->output_channel_order == CHANNEL_ORDER_CODED) {
         for (dca_ch = 0; dca_ch < DCA_SPEAKER_COUNT; dca_ch++)
             if (dca_mask & (1U << dca_ch))
                 ch_remap[nchannels++] = dca_ch;
-        avctx->channel_layout = dca_mask;
+        avctx->ch_layout.order       = AV_CHANNEL_ORDER_UNSPEC;
+        avctx->ch_layout.nb_channels = nchannels;
     } else {
         int wav_mask = 0;
         int wav_map[18];
@@ -69,10 +75,10 @@ int ff_dca_set_channel_layout(AVCodecContext *avctx, int *ch_remap, int dca_mask
         for (wav_ch = 0; wav_ch < 18; wav_ch++)
             if (wav_mask & (1 << wav_ch))
                 ch_remap[nchannels++] = wav_map[wav_ch];
-        avctx->channel_layout = wav_mask;
+
+        av_channel_layout_from_mask(&avctx->ch_layout, wav_mask);
     }
 
-    avctx->channels = nchannels;
     return nchannels;
 }
 
@@ -141,12 +147,11 @@ void ff_dca_downmix_to_stereo_float(AVFloatDSPContext *fdsp, float **samples,
     }
 }
 
-static int dcadec_decode_frame(AVCodecContext *avctx, void *data,
+static int dcadec_decode_frame(AVCodecContext *avctx, AVFrame *frame,
                                int *got_frame_ptr, AVPacket *avpkt)
 {
     DCAContext *s = avctx->priv_data;
-    AVFrame *frame = data;
-    uint8_t *input = avpkt->data;
+    const uint8_t *input = avpkt->data;
     int input_size = avpkt->size;
     int i, ret, prev_packet = s->packet;
     uint32_t mrk;
@@ -318,8 +323,15 @@ static av_cold int dcadec_close(AVCodecContext *avctx)
     return 0;
 }
 
+static av_cold void dcadec_init_static(void)
+{
+    ff_dca_lbr_init_tables();
+    ff_dca_init_vlcs();
+}
+
 static av_cold int dcadec_init(AVCodecContext *avctx)
 {
+    static AVOnce init_static_once = AV_ONCE_INIT;
     DCAContext *s = avctx->priv_data;
 
     s->avctx = avctx;
@@ -328,8 +340,6 @@ static av_cold int dcadec_init(AVCodecContext *avctx)
     s->xll.avctx = avctx;
     s->lbr.avctx = avctx;
 
-    ff_dca_init_vlcs();
-
     if (ff_dca_core_init(&s->core) < 0)
         return AVERROR(ENOMEM);
 
@@ -343,24 +353,31 @@ static av_cold int dcadec_init(AVCodecContext *avctx)
 
     s->crctab = av_crc_get_table(AV_CRC_16_CCITT);
 
-    switch (avctx->request_channel_layout & ~AV_CH_LAYOUT_NATIVE) {
-    case 0:
-        s->request_channel_layout = 0;
-        break;
-    case AV_CH_LAYOUT_STEREO:
-    case AV_CH_LAYOUT_STEREO_DOWNMIX:
-        s->request_channel_layout = DCA_SPEAKER_LAYOUT_STEREO;
-        break;
-    case AV_CH_LAYOUT_5POINT0:
-        s->request_channel_layout = DCA_SPEAKER_LAYOUT_5POINT0;
-        break;
-    case AV_CH_LAYOUT_5POINT1:
-        s->request_channel_layout = DCA_SPEAKER_LAYOUT_5POINT1;
-        break;
-    default:
-        av_log(avctx, AV_LOG_WARNING, "Invalid request_channel_layout\n");
-        break;
+#if FF_API_OLD_CHANNEL_LAYOUT
+FF_DISABLE_DEPRECATION_WARNINGS
+    if (avctx->request_channel_layout & AV_CH_LAYOUT_NATIVE)
+        s->output_channel_order = CHANNEL_ORDER_CODED;
+
+    if (avctx->request_channel_layout & ~AV_CH_LAYOUT_NATIVE) {
+        av_channel_layout_uninit(&s->downmix_layout);
+        av_channel_layout_from_mask(&s->downmix_layout, avctx->request_channel_layout & ~AV_CH_LAYOUT_NATIVE);
     }
+FF_ENABLE_DEPRECATION_WARNINGS
+#endif
+
+    if (s->downmix_layout.nb_channels) {
+        if (!av_channel_layout_compare(&s->downmix_layout, &(AVChannelLayout)AV_CHANNEL_LAYOUT_STEREO) ||
+            !av_channel_layout_compare(&s->downmix_layout, &(AVChannelLayout)AV_CHANNEL_LAYOUT_STEREO_DOWNMIX))
+            s->request_channel_layout = DCA_SPEAKER_LAYOUT_STEREO;
+        else if (!av_channel_layout_compare(&s->downmix_layout, &(AVChannelLayout)AV_CHANNEL_LAYOUT_5POINT0))
+            s->request_channel_layout = DCA_SPEAKER_LAYOUT_5POINT0;
+        else if (!av_channel_layout_compare(&s->downmix_layout, &(AVChannelLayout)AV_CHANNEL_LAYOUT_5POINT1))
+            s->request_channel_layout = DCA_SPEAKER_LAYOUT_5POINT1;
+        else
+            av_log(avctx, AV_LOG_WARNING, "Invalid downmix layout\n");
+    }
+
+    ff_thread_once(&init_static_once, dcadec_init_static);
 
     return 0;
 }
@@ -370,6 +387,18 @@ static av_cold int dcadec_init(AVCodecContext *avctx)
 
 static const AVOption dcadec_options[] = {
     { "core_only", "Decode core only without extensions", OFFSET(core_only), AV_OPT_TYPE_BOOL, { .i64 = 0 }, 0, 1, PARAM },
+
+    { "channel_order", "Order in which the channels are to be exported",
+        OFFSET(output_channel_order), AV_OPT_TYPE_INT,
+        { .i64 = CHANNEL_ORDER_DEFAULT }, 0, 1, PARAM, "channel_order" },
+      { "default", "normal libavcodec channel order", 0, AV_OPT_TYPE_CONST,
+        { .i64 = CHANNEL_ORDER_DEFAULT }, .flags = PARAM, "channel_order" },
+      { "coded",    "order in which the channels are coded in the bitstream",
+        0, AV_OPT_TYPE_CONST, { .i64 = CHANNEL_ORDER_CODED }, .flags = PARAM, "channel_order" },
+
+    { "downmix", "Request a specific channel layout from the decoder", OFFSET(downmix_layout),
+        AV_OPT_TYPE_CHLAYOUT, {.str = NULL}, .flags = PARAM },
+
     { NULL }
 };
 
@@ -381,20 +410,20 @@ static const AVClass dcadec_class = {
     .category   = AV_CLASS_CATEGORY_DECODER,
 };
 
-AVCodec ff_dca_decoder = {
-    .name           = "dca",
-    .long_name      = NULL_IF_CONFIG_SMALL("DCA (DTS Coherent Acoustics)"),
-    .type           = AVMEDIA_TYPE_AUDIO,
-    .id             = AV_CODEC_ID_DTS,
+const FFCodec ff_dca_decoder = {
+    .p.name         = "dca",
+    .p.long_name    = NULL_IF_CONFIG_SMALL("DCA (DTS Coherent Acoustics)"),
+    .p.type         = AVMEDIA_TYPE_AUDIO,
+    .p.id           = AV_CODEC_ID_DTS,
     .priv_data_size = sizeof(DCAContext),
     .init           = dcadec_init,
-    .decode         = dcadec_decode_frame,
+    FF_CODEC_DECODE_CB(dcadec_decode_frame),
     .close          = dcadec_close,
     .flush          = dcadec_flush,
-    .capabilities   = AV_CODEC_CAP_DR1 | AV_CODEC_CAP_CHANNEL_CONF,
-    .sample_fmts    = (const enum AVSampleFormat[]) { AV_SAMPLE_FMT_S16P, AV_SAMPLE_FMT_S32P,
+    .p.capabilities = AV_CODEC_CAP_DR1 | AV_CODEC_CAP_CHANNEL_CONF,
+    .p.sample_fmts  = (const enum AVSampleFormat[]) { AV_SAMPLE_FMT_S16P, AV_SAMPLE_FMT_S32P,
                                                       AV_SAMPLE_FMT_FLTP, AV_SAMPLE_FMT_NONE },
-    .priv_class     = &dcadec_class,
-    .profiles       = NULL_IF_CONFIG_SMALL(ff_dca_profiles),
-    .caps_internal  = FF_CODEC_CAP_INIT_CLEANUP,
+    .p.priv_class   = &dcadec_class,
+    .p.profiles     = NULL_IF_CONFIG_SMALL(ff_dca_profiles),
+    .caps_internal  = FF_CODEC_CAP_INIT_THREADSAFE | FF_CODEC_CAP_INIT_CLEANUP,
 };
