@@ -20,11 +20,13 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
-#include "libavutil/common.h"
+#include <stddef.h>
+
+#include "libavutil/avassert.h"
+#include "libavutil/macros.h"
 
-#include "avcodec.h"
-#include "get_bits.h"
 #include "dcahuff.h"
+#include "put_bits.h"
 
 #define TMODE_COUNT 4
 static const uint8_t tmode_vlc_bits[TMODE_COUNT] = { 3, 3, 3, 2 };
@@ -1262,13 +1264,9 @@ VLC     ff_dca_vlc_rsd;
 
 av_cold void ff_dca_init_vlcs(void)
 {
-    static VLC_TYPE dca_table[30214][2];
-    static int vlcs_initialized = 0;
+    static VLCElem dca_table[30214];
     int i, j, k = 0;
 
-    if (vlcs_initialized)
-        return;
-
 #define DCA_INIT_VLC(vlc, a, b, c, d)                                       \
     do {                                                                    \
         vlc.table           = &dca_table[vlc_offs[k]];                      \
@@ -1331,8 +1329,6 @@ av_cold void ff_dca_init_vlcs(void)
     LBR_INIT_VLC(ff_dca_vlc_grid_2,      grid_2,      9);
     LBR_INIT_VLC(ff_dca_vlc_grid_3,      grid_3,      9);
     LBR_INIT_VLC(ff_dca_vlc_rsd,         rsd,         6);
-
-    vlcs_initialized = 1;
 }
 
 uint32_t ff_dca_vlc_calc_quant_bits(int *values, uint8_t n, uint8_t sel, uint8_t table)
