@@ -48,9 +48,11 @@
 
 #include "libavutil/imgutils.h"
 #include "avcodec.h"
-#include "internal.h"
+#include "codec_internal.h"
+#include "encode.h"
 #include "put_bits.h"
 #include "bytestream.h"
+#include "zlib_wrapper.h"
 
 #define HAS_IFRAME_IMAGE 0x02
 #define HAS_PALLET_INFO 0x01
@@ -111,6 +113,7 @@ typedef struct FlashSV2Context {
     uint8_t use_custom_palette;
     uint8_t palette_type;       ///< 0=>default, 1=>custom - changed when palette regenerated.
     Palette palette;
+    FFZStream zstream;
 #ifndef FLASHSV2_DUMB
     double tot_blocks;          ///< blocks encoded since last keyframe
     double diff_blocks;         ///< blocks that were different since last keyframe
@@ -135,6 +138,7 @@ static av_cold void cleanup(FlashSV2Context * s)
 
     av_freep(&s->frame_blocks);
     av_freep(&s->key_blocks);
+    ff_deflate_end(&s->zstream);
 }
 
 static void init_blocks(FlashSV2Context * s, Block * blocks,
@@ -142,6 +146,7 @@ static void init_blocks(FlashSV2Context * s, Block * blocks,
 {
     int row, col;
     Block *b;
+    memset(blocks, 0, s->cols * s->rows * sizeof(*blocks));
     for (col = 0; col < s->cols; col++) {
         for (row = 0; row < s->rows; row++) {
             b = blocks + (col + row * s->cols);
@@ -158,7 +163,7 @@ static void init_blocks(FlashSV2Context * s, Block * blocks,
             b->enc   = encbuf;
             b->data  = databuf;
             encbuf  += b->width * b->height * 3;
-            databuf += !databuf ? 0 : b->width * b->height * 6;
+            databuf  = databuf ? databuf + b->width * b->height * 6 : NULL;
         }
     }
 }
@@ -174,9 +179,37 @@ static void reset_stats(FlashSV2Context * s)
 #endif
 }
 
+static int update_block_dimensions(FlashSV2Context *s, int block_width, int block_height)
+{
+    s->block_width  = block_width;
+    s->block_height = block_height;
+    s->rows = (s->image_height + s->block_height - 1) / s->block_height;
+    s->cols = (s->image_width  + s->block_width  - 1) / s->block_width;
+    if (s->rows * s->cols > s->blocks_size / sizeof(Block)) {
+        s->frame_blocks = av_realloc_array(s->frame_blocks, s->rows, s->cols * sizeof(Block));
+        s->key_blocks = av_realloc_array(s->key_blocks, s->cols, s->rows * sizeof(Block));
+        if (!s->frame_blocks || !s->key_blocks) {
+            av_log(s->avctx, AV_LOG_ERROR, "Memory allocation failed.\n");
+            return AVERROR(ENOMEM);
+        }
+        s->blocks_size = s->rows * s->cols * sizeof(Block);
+    }
+    init_blocks(s, s->frame_blocks, s->encbuffer, s->databuffer);
+    init_blocks(s, s->key_blocks, s->keybuffer, 0);
+
+    av_fast_malloc(&s->blockbuffer, &s->blockbuffer_size, block_width * block_height * 6);
+    if (!s->blockbuffer) {
+        av_log(s->avctx, AV_LOG_ERROR, "Could not allocate block buffer.\n");
+        return AVERROR(ENOMEM);
+    }
+    return 0;
+}
+
+
 static av_cold int flashsv2_encode_init(AVCodecContext * avctx)
 {
     FlashSV2Context *s = avctx->priv_data;
+    int ret;
 
     s->avctx = avctx;
 
@@ -186,64 +219,45 @@ static av_cold int flashsv2_encode_init(AVCodecContext * avctx)
     if (s->comp < 0 || s->comp > 9) {
         av_log(avctx, AV_LOG_ERROR,
                "Compression level should be 0-9, not %d\n", s->comp);
-        return -1;
+        return AVERROR(EINVAL);
     }
 
 
     if ((avctx->width > 4095) || (avctx->height > 4095)) {
         av_log(avctx, AV_LOG_ERROR,
                "Input dimensions too large, input must be max 4095x4095 !\n");
-        return -1;
+        return AVERROR(EINVAL);
     }
     if ((avctx->width < 16) || (avctx->height < 16)) {
         av_log(avctx, AV_LOG_ERROR,
                "Input dimensions too small, input must be at least 16x16 !\n");
-        return -1;
+        return AVERROR(EINVAL);
     }
 
-    if (av_image_check_size(avctx->width, avctx->height, 0, avctx) < 0)
-        return -1;
-
+    if ((ret = av_image_check_size(avctx->width, avctx->height, 0, avctx)) < 0)
+        return ret;
 
+    ret = ff_deflate_init(&s->zstream, s->comp, avctx);
+    if (ret < 0)
+        return ret;
     s->last_key_frame = 0;
 
     s->image_width  = avctx->width;
     s->image_height = avctx->height;
 
-    s->block_width  = (s->image_width /  12) & ~15;
-    s->block_height = (s->image_height / 12) & ~15;
-
-    if(!s->block_width)
-        s->block_width = 1;
-    if(!s->block_height)
-        s->block_height = 1;
-
-    s->rows = (s->image_height + s->block_height - 1) / s->block_height;
-    s->cols = (s->image_width +  s->block_width -  1) / s->block_width;
-
     s->frame_size  = s->image_width * s->image_height * 3;
-    s->blocks_size = s->rows * s->cols * sizeof(Block);
 
     s->encbuffer     = av_mallocz(s->frame_size);
     s->keybuffer     = av_mallocz(s->frame_size);
     s->databuffer    = av_mallocz(s->frame_size * 6);
     s->current_frame = av_mallocz(s->frame_size);
     s->key_frame     = av_mallocz(s->frame_size);
-    s->frame_blocks  = av_mallocz(s->blocks_size);
-    s->key_blocks    = av_mallocz(s->blocks_size);
     if (!s->encbuffer || !s->keybuffer || !s->databuffer
-        || !s->current_frame || !s->key_frame || !s->key_blocks
-        || !s->frame_blocks) {
+        || !s->current_frame || !s->key_frame) {
         av_log(avctx, AV_LOG_ERROR, "Memory allocation failed.\n");
-        cleanup(s);
         return AVERROR(ENOMEM);
     }
 
-    s->blockbuffer      = NULL;
-    s->blockbuffer_size = 0;
-
-    init_blocks(s, s->frame_blocks, s->encbuffer, s->databuffer);
-    init_blocks(s, s->key_blocks,   s->keybuffer, 0);
     reset_stats(s);
 #ifndef FLASHSV2_DUMB
     s->total_bits = 1;
@@ -252,7 +266,7 @@ static av_cold int flashsv2_encode_init(AVCodecContext * avctx)
     s->use_custom_palette =  0;
     s->palette_type       = -1;        // so that the palette will be generated in reconfigure_at_keyframe
 
-    return 0;
+    return update_block_dimensions(s, 64, 64);
 }
 
 static int new_key_frame(FlashSV2Context * s)
@@ -348,41 +362,47 @@ static int write_block(Block * b, uint8_t * buf, int buf_size)
     return buf_pos;
 }
 
-static int encode_zlib(Block * b, uint8_t * buf, unsigned long *buf_size, int comp)
+static int encode_zlib(Block *b, uint8_t *buf, unsigned long *buf_size,
+                       z_stream *zstream)
 {
-    int res = compress2(buf, buf_size, b->sl_begin, b->sl_end - b->sl_begin, comp);
-    return res == Z_OK ? 0 : -1;
+    int res;
+
+    if (deflateReset(zstream) != Z_OK)
+        return AVERROR_EXTERNAL;
+    zstream->next_out  = buf;
+    zstream->avail_out = *buf_size;
+    zstream->next_in   = b->sl_begin;
+    zstream->avail_in  = b->sl_end - b->sl_begin;
+    res = deflate(zstream, Z_FINISH);
+    if (res != Z_STREAM_END)
+        return AVERROR_EXTERNAL;
+    *buf_size -= zstream->avail_out;
+    return 0;
 }
 
 static int encode_zlibprime(Block * b, Block * prime, uint8_t * buf,
-                            int *buf_size, int comp)
+                            int *buf_size, z_stream *zstream)
 {
-    z_stream s;
     int res;
-    s.zalloc = NULL;
-    s.zfree  = NULL;
-    s.opaque = NULL;
-    res = deflateInit(&s, comp);
-    if (res < 0)
-        return -1;
 
-    s.next_in  = prime->enc;
-    s.avail_in = prime->enc_size;
-    while (s.avail_in > 0) {
-        s.next_out  = buf;
-        s.avail_out = *buf_size;
-        res = deflate(&s, Z_SYNC_FLUSH);
+    if (deflateReset(zstream) != Z_OK)
+        return AVERROR_EXTERNAL;
+    zstream->next_in  = prime->enc;
+    zstream->avail_in = prime->enc_size;
+    while (zstream->avail_in > 0) {
+        zstream->next_out  = buf;
+        zstream->avail_out = *buf_size;
+        res = deflate(zstream, Z_SYNC_FLUSH);
         if (res < 0)
             return -1;
     }
 
-    s.next_in   = b->sl_begin;
-    s.avail_in  = b->sl_end - b->sl_begin;
-    s.next_out  = buf;
-    s.avail_out = *buf_size;
-    res = deflate(&s, Z_FINISH);
-    deflateEnd(&s);
-    *buf_size -= s.avail_out;
+    zstream->next_in   = b->sl_begin;
+    zstream->avail_in  = b->sl_end - b->sl_begin;
+    zstream->next_out  = buf;
+    zstream->avail_out = *buf_size;
+    res = deflate(zstream, Z_FINISH);
+    *buf_size -= zstream->avail_out;
     if (res != Z_STREAM_END)
         return -1;
     return 0;
@@ -550,7 +570,7 @@ static int encode_15_7(Palette * palette, Block * b, const uint8_t * src,
 }
 
 static int encode_block(FlashSV2Context *s, Palette * palette, Block * b,
-                        Block * prev, const uint8_t * src, int stride, int comp,
+                        Block *prev, const uint8_t *src, int stride,
                         int dist, int keyframe)
 {
     unsigned buf_size = b->width * b->height * 6;
@@ -565,12 +585,12 @@ static int encode_block(FlashSV2Context *s, Palette * palette, Block * b,
 
     if (b->len > 0) {
         b->data_size = buf_size;
-        res = encode_zlib(b, b->data, &b->data_size, comp);
+        res = encode_zlib(b, b->data, &b->data_size, &s->zstream.zstream);
         if (res)
             return res;
 
         if (!keyframe) {
-            res = encode_zlibprime(b, prev, buf, &buf_size, comp);
+            res = encode_zlibprime(b, prev, buf, &buf_size, &s->zstream.zstream);
             if (res)
                 return res;
 
@@ -647,7 +667,8 @@ static int encode_all_blocks(FlashSV2Context * s, int keyframe)
                 b->flags |= HAS_DIFF_BLOCKS;
             }
             data = s->current_frame + s->image_width * 3 * s->block_height * row + s->block_width * col * 3;
-            res = encode_block(s, &s->palette, b, prev, data, s->image_width * 3, s->comp, s->dist, keyframe);
+            res = encode_block(s, &s->palette, b, prev, data,
+                               s->image_width * 3, s->dist, keyframe);
 #ifndef FLASHSV2_DUMB
             if (b->dirty)
                 s->diff_blocks++;
@@ -800,29 +821,10 @@ static int reconfigure_at_keyframe(FlashSV2Context * s, const uint8_t * image,
     int block_width  = optimum_block_width (s);
     int block_height = optimum_block_height(s);
 
-    s->rows = (s->image_height + block_height - 1) / block_height;
-    s->cols = (s->image_width  + block_width  - 1) / block_width;
-
     if (block_width != s->block_width || block_height != s->block_height) {
-        s->block_width  = block_width;
-        s->block_height = block_height;
-        if (s->rows * s->cols > s->blocks_size / sizeof(Block)) {
-            s->frame_blocks = av_realloc_array(s->frame_blocks, s->rows, s->cols * sizeof(Block));
-            s->key_blocks = av_realloc_array(s->key_blocks, s->cols, s->rows * sizeof(Block));
-            if (!s->frame_blocks || !s->key_blocks) {
-                av_log(s->avctx, AV_LOG_ERROR, "Memory allocation failed.\n");
-                return -1;
-            }
-            s->blocks_size = s->rows * s->cols * sizeof(Block);
-        }
-        init_blocks(s, s->frame_blocks, s->encbuffer, s->databuffer);
-        init_blocks(s, s->key_blocks, s->keybuffer, 0);
-
-        av_fast_malloc(&s->blockbuffer, &s->blockbuffer_size, block_width * block_height * 6);
-        if (!s->blockbuffer) {
-            av_log(s->avctx, AV_LOG_ERROR, "Could not allocate block buffer.\n");
-            return AVERROR(ENOMEM);
-        }
+        res = update_block_dimensions(s, block_width, block_height);
+        if (res < 0)
+            return res;
     }
 
     s->use15_7 = optimum_use15_7(s);
@@ -855,7 +857,7 @@ static int flashsv2_encode_frame(AVCodecContext *avctx, AVPacket *pkt,
     int res;
     int keyframe = 0;
 
-    if ((res = ff_alloc_packet2(avctx, pkt, s->frame_size + AV_INPUT_BUFFER_MIN_SIZE, 0)) < 0)
+    if ((res = ff_alloc_packet(avctx, pkt, s->frame_size + AV_INPUT_BUFFER_MIN_SIZE)) < 0)
         return res;
 
     /* First frame needs to be a keyframe */
@@ -908,14 +910,15 @@ static av_cold int flashsv2_encode_end(AVCodecContext * avctx)
     return 0;
 }
 
-AVCodec ff_flashsv2_encoder = {
-    .name           = "flashsv2",
-    .long_name      = NULL_IF_CONFIG_SMALL("Flash Screen Video Version 2"),
-    .type           = AVMEDIA_TYPE_VIDEO,
-    .id             = AV_CODEC_ID_FLASHSV2,
+const FFCodec ff_flashsv2_encoder = {
+    .p.name         = "flashsv2",
+    .p.long_name    = NULL_IF_CONFIG_SMALL("Flash Screen Video Version 2"),
+    .p.type         = AVMEDIA_TYPE_VIDEO,
+    .p.id           = AV_CODEC_ID_FLASHSV2,
     .priv_data_size = sizeof(FlashSV2Context),
     .init           = flashsv2_encode_init,
-    .encode2        = flashsv2_encode_frame,
+    FF_CODEC_ENCODE_CB(flashsv2_encode_frame),
     .close          = flashsv2_encode_end,
-    .pix_fmts       = (const enum AVPixelFormat[]){ AV_PIX_FMT_BGR24, AV_PIX_FMT_NONE },
+    .p.pix_fmts     = (const enum AVPixelFormat[]){ AV_PIX_FMT_BGR24, AV_PIX_FMT_NONE },
+    .caps_internal  = FF_CODEC_CAP_INIT_THREADSAFE | FF_CODEC_CAP_INIT_CLEANUP,
 };
