@@ -34,8 +34,9 @@
 
 #include "avcodec.h"
 #include "celp_math.h"
+#include "codec_internal.h"
+#include "encode.h"
 #include "g723_1.h"
-#include "internal.h"
 
 #define BITSTREAM_WRITER_LE
 #include "put_bits.h"
@@ -98,11 +99,6 @@ static av_cold int g723_1_encode_init(AVCodecContext *avctx)
         return AVERROR(EINVAL);
     }
 
-    if (avctx->channels != 1) {
-        av_log(avctx, AV_LOG_ERROR, "Only mono supported\n");
-        return AVERROR(EINVAL);
-    }
-
     if (avctx->bit_rate == 6300) {
         p->cur_rate = RATE_6300;
     } else if (avctx->bit_rate == 5300) {
@@ -1044,10 +1040,9 @@ static void fcb_search(G723_1_ChannelContext *p, int16_t *impulse_resp,
  * @param frame output buffer
  * @param size  size of the buffer
  */
-static int pack_bitstream(G723_1_ChannelContext *p, AVPacket *avpkt)
+static void pack_bitstream(G723_1_ChannelContext *p, AVPacket *avpkt, int info_bits)
 {
     PutBitContext pb;
-    int info_bits = 0;
     int i, temp;
 
     init_put_bits(&pb, avpkt->data, avpkt->size);
@@ -1099,7 +1094,6 @@ static int pack_bitstream(G723_1_ChannelContext *p, AVPacket *avpkt)
     }
 
     flush_put_bits(&pb);
-    return frame_size[info_bits];
 }
 
 static int g723_1_encode_frame(AVCodecContext *avctx, AVPacket *avpkt,
@@ -1112,15 +1106,14 @@ static int g723_1_encode_frame(AVCodecContext *avctx, AVPacket *avpkt,
     int16_t cur_lsp[LPC_ORDER];
     int16_t weighted_lpc[LPC_ORDER * SUBFRAMES << 1];
     int16_t vector[FRAME_LEN + PITCH_MAX];
-    int offset, ret, i, j;
+    int offset, ret, i, j, info_bits = 0;
     int16_t *in, *start;
     HFParam hf[4];
 
     /* duplicate input */
-    start = in = av_malloc(frame->nb_samples * sizeof(int16_t));
+    start = in = av_memdup(frame->data[0], frame->nb_samples * sizeof(int16_t));
     if (!in)
         return AVERROR(ENOMEM);
-    memcpy(in, frame->data[0], frame->nb_samples * sizeof(int16_t));
 
     highpass_filter(in, &p->hpf_fir_mem, &p->hpf_iir_mem);
 
@@ -1231,29 +1224,35 @@ static int g723_1_encode_frame(AVCodecContext *avctx, AVPacket *avpkt,
 
     av_free(start);
 
-    if ((ret = ff_alloc_packet2(avctx, avpkt, 24, 0)) < 0)
+    ret = ff_get_encode_buffer(avctx, avpkt, frame_size[info_bits], 0);
+    if (ret < 0)
         return ret;
 
     *got_packet_ptr = 1;
-    avpkt->size = pack_bitstream(p, avpkt);
+    pack_bitstream(p, avpkt, info_bits);
     return 0;
 }
 
-static const AVCodecDefault defaults[] = {
+static const FFCodecDefault defaults[] = {
     { "b", "6300" },
     { NULL },
 };
 
-AVCodec ff_g723_1_encoder = {
-    .name           = "g723_1",
-    .long_name      = NULL_IF_CONFIG_SMALL("G.723.1"),
-    .type           = AVMEDIA_TYPE_AUDIO,
-    .id             = AV_CODEC_ID_G723_1,
+const FFCodec ff_g723_1_encoder = {
+    .p.name         = "g723_1",
+    .p.long_name    = NULL_IF_CONFIG_SMALL("G.723.1"),
+    .p.type         = AVMEDIA_TYPE_AUDIO,
+    .p.id           = AV_CODEC_ID_G723_1,
+    .p.capabilities = AV_CODEC_CAP_DR1,
     .priv_data_size = sizeof(G723_1_Context),
     .init           = g723_1_encode_init,
-    .encode2        = g723_1_encode_frame,
+    FF_CODEC_ENCODE_CB(g723_1_encode_frame),
     .defaults       = defaults,
-    .sample_fmts    = (const enum AVSampleFormat[]) {
+    .p.sample_fmts  = (const enum AVSampleFormat[]) {
         AV_SAMPLE_FMT_S16, AV_SAMPLE_FMT_NONE
     },
+    .p.ch_layouts   = (const AVChannelLayout[]){
+        AV_CHANNEL_LAYOUT_MONO, { 0 }
+    },
+    .caps_internal  = FF_CODEC_CAP_INIT_THREADSAFE,
 };
