@@ -20,8 +20,11 @@
  */
 
 #include <codec2/codec2.h>
+#include "libavutil/channel_layout.h"
 #include "avcodec.h"
 #include "libavutil/opt.h"
+#include "codec_internal.h"
+#include "encode.h"
 #include "internal.h"
 #include "codec2utils.h"
 
@@ -45,12 +48,6 @@ static const AVClass libcodec2_enc_class = {
     .version    = LIBAVUTIL_VERSION_INT,
 };
 
-static const AVClass libcodec2_dec_class = {
-    .class_name = "libcodec2 decoder",
-    .item_name  = av_default_item_name,
-    .version    = LIBAVUTIL_VERSION_INT,
-};
-
 static av_cold int libcodec2_init_common(AVCodecContext *avctx, int mode)
 {
     LibCodec2Context *c2 = avctx->priv_data;
@@ -89,9 +86,9 @@ libcodec2_init_common_error:
 static av_cold int libcodec2_init_decoder(AVCodecContext *avctx)
 {
     avctx->sample_rate      = 8000;
-    avctx->channels         = 1;
     avctx->sample_fmt       = AV_SAMPLE_FMT_S16;
-    avctx->channel_layout   = AV_CH_LAYOUT_MONO;
+    av_channel_layout_uninit(&avctx->ch_layout);
+    avctx->ch_layout = (AVChannelLayout)AV_CHANNEL_LAYOUT_MONO;
 
     if (avctx->extradata_size != CODEC2_EXTRADATA_SIZE) {
         av_log(avctx, AV_LOG_ERROR, "must have exactly %i bytes of extradata (got %i)\n",
@@ -108,7 +105,7 @@ static av_cold int libcodec2_init_encoder(AVCodecContext *avctx)
 
     //will need to be smarter once we get wideband support
     if (avctx->sample_rate != 8000 ||
-        avctx->channels != 1 ||
+        avctx->ch_layout.nb_channels != 1 ||
         avctx->sample_fmt != AV_SAMPLE_FMT_S16) {
         av_log(avctx, AV_LOG_ERROR, "only 8 kHz 16-bit mono allowed\n");
         return AVERROR(EINVAL);
@@ -133,13 +130,12 @@ static av_cold int libcodec2_close(AVCodecContext *avctx)
     return 0;
 }
 
-static int libcodec2_decode(AVCodecContext *avctx, void *data,
+static int libcodec2_decode(AVCodecContext *avctx, AVFrame *frame,
                             int *got_frame_ptr, AVPacket *pkt)
 {
     LibCodec2Context *c2 = avctx->priv_data;
-    AVFrame *frame = data;
     int ret, nframes, i;
-    uint8_t *input;
+    const uint8_t *input;
     int16_t *output;
 
     nframes           = pkt->size / avctx->block_align;
@@ -169,7 +165,7 @@ static int libcodec2_encode(AVCodecContext *avctx, AVPacket *avpkt,
     LibCodec2Context *c2 = avctx->priv_data;
     int16_t *samples = (int16_t *)frame->data[0];
 
-    int ret = ff_alloc_packet2(avctx, avpkt, avctx->block_align, 0);
+    int ret = ff_get_encode_buffer(avctx, avpkt, avctx->block_align, 0);
     if (ret < 0) {
         return ret;
     }
@@ -180,34 +176,39 @@ static int libcodec2_encode(AVCodecContext *avctx, AVPacket *avpkt,
     return 0;
 }
 
-AVCodec ff_libcodec2_decoder = {
-    .name                   = "libcodec2",
-    .long_name              = NULL_IF_CONFIG_SMALL("codec2 decoder using libcodec2"),
-    .type                   = AVMEDIA_TYPE_AUDIO,
-    .id                     = AV_CODEC_ID_CODEC2,
+const FFCodec ff_libcodec2_decoder = {
+    .p.name                 = "libcodec2",
+    .p.long_name            = NULL_IF_CONFIG_SMALL("codec2 decoder using libcodec2"),
+    .p.type                 = AVMEDIA_TYPE_AUDIO,
+    .p.id                   = AV_CODEC_ID_CODEC2,
+    .p.capabilities         = AV_CODEC_CAP_CHANNEL_CONF,
+    .p.supported_samplerates = (const int[]){ 8000, 0 },
+    .p.sample_fmts          = (const enum AVSampleFormat[]) { AV_SAMPLE_FMT_S16, AV_SAMPLE_FMT_NONE },
+    .p.ch_layouts           = (const AVChannelLayout[]) { AV_CHANNEL_LAYOUT_MONO, { 0 } },
     .priv_data_size         = sizeof(LibCodec2Context),
     .init                   = libcodec2_init_decoder,
     .close                  = libcodec2_close,
-    .decode                 = libcodec2_decode,
-    .capabilities           = AV_CODEC_CAP_CHANNEL_CONF,
-    .supported_samplerates  = (const int[]){ 8000, 0 },
-    .sample_fmts            = (const enum AVSampleFormat[]) { AV_SAMPLE_FMT_S16, AV_SAMPLE_FMT_NONE },
-    .channel_layouts        = (const uint64_t[]) { AV_CH_LAYOUT_MONO, 0 },
-    .priv_class             = &libcodec2_dec_class,
+    FF_CODEC_DECODE_CB(libcodec2_decode),
+#if FF_API_OLD_CHANNEL_LAYOUT
+    .p.channel_layouts      = (const uint64_t[]) { AV_CH_LAYOUT_MONO, 0 },
+#endif
 };
 
-AVCodec ff_libcodec2_encoder = {
-    .name                   = "libcodec2",
-    .long_name              = NULL_IF_CONFIG_SMALL("codec2 encoder using libcodec2"),
-    .type                   = AVMEDIA_TYPE_AUDIO,
-    .id                     = AV_CODEC_ID_CODEC2,
+const FFCodec ff_libcodec2_encoder = {
+    .p.name                 = "libcodec2",
+    .p.long_name            = NULL_IF_CONFIG_SMALL("codec2 encoder using libcodec2"),
+    .p.type                 = AVMEDIA_TYPE_AUDIO,
+    .p.id                   = AV_CODEC_ID_CODEC2,
+    .p.capabilities         = AV_CODEC_CAP_DR1,
+    .p.supported_samplerates = (const int[]){ 8000, 0 },
+    .p.sample_fmts          = (const enum AVSampleFormat[]) { AV_SAMPLE_FMT_S16, AV_SAMPLE_FMT_NONE },
+    .p.ch_layouts           = (const AVChannelLayout[]) { AV_CHANNEL_LAYOUT_MONO, { 0 } },
+    .p.priv_class           = &libcodec2_enc_class,
     .priv_data_size         = sizeof(LibCodec2Context),
     .init                   = libcodec2_init_encoder,
     .close                  = libcodec2_close,
-    .encode2                = libcodec2_encode,
-    .capabilities           = 0,
-    .supported_samplerates  = (const int[]){ 8000, 0 },
-    .sample_fmts            = (const enum AVSampleFormat[]) { AV_SAMPLE_FMT_S16, AV_SAMPLE_FMT_NONE },
-    .channel_layouts        = (const uint64_t[]) { AV_CH_LAYOUT_MONO, 0 },
-    .priv_class             = &libcodec2_enc_class,
+    FF_CODEC_ENCODE_CB(libcodec2_encode),
+#if FF_API_OLD_CHANNEL_LAYOUT
+    .p.channel_layouts      = (const uint64_t[]) { AV_CH_LAYOUT_MONO, 0 },
+#endif
 };
