@@ -24,7 +24,8 @@
 #include "libavutil/opt.h"
 #include "avcodec.h"
 #include "audio_frame_queue.h"
-#include "internal.h"
+#include "codec_internal.h"
+#include "encode.h"
 #include "profiles.h"
 
 #ifdef AACENCODER_LIB_VL0
@@ -112,7 +113,6 @@ static int aac_encode_close(AVCodecContext *avctx)
 
     if (s->handle)
         aacEncClose(&s->handle);
-    av_freep(&avctx->extradata);
     ff_af_queue_close(&s->afq);
 
     return 0;
@@ -128,7 +128,7 @@ static av_cold int aac_encode_init(AVCodecContext *avctx)
     int aot = FF_PROFILE_AAC_LOW + 1;
     int sce = 0, cpe = 0;
 
-    if ((err = aacEncOpen(&s->handle, 0, avctx->channels)) != AACENC_OK) {
+    if ((err = aacEncOpen(&s->handle, 0, avctx->ch_layout.nb_channels)) != AACENC_OK) {
         av_log(avctx, AV_LOG_ERROR, "Unable to open the encoder: %s\n",
                aac_get_error(err));
         goto error;
@@ -159,7 +159,7 @@ static av_cold int aac_encode_init(AVCodecContext *avctx)
         goto error;
     }
 
-    switch (avctx->channels) {
+    switch (avctx->ch_layout.nb_channels) {
     case 1: mode = MODE_1;       sce = 1; cpe = 0; break;
     case 2:
 #if FDKENC_VER_AT_LEAST(4, 0) // 4.0.0
@@ -193,7 +193,7 @@ static av_cold int aac_encode_init(AVCodecContext *avctx)
     case 8:
         sce = 2;
         cpe = 3;
-        if (avctx->channel_layout == AV_CH_LAYOUT_7POINT1) {
+        if (!av_channel_layout_compare(&avctx->ch_layout, &(AVChannelLayout)AV_CHANNEL_LAYOUT_7POINT1)) {
             mode = MODE_7_1_REAR_SURROUND;
         } else {
             // MODE_1_2_2_2_1 and MODE_7_1_FRONT_CENTER use the same channel layout
@@ -203,7 +203,7 @@ static av_cold int aac_encode_init(AVCodecContext *avctx)
 #endif
     default:
         av_log(avctx, AV_LOG_ERROR,
-               "Unsupported number of channels %d\n", avctx->channels);
+               "Unsupported number of channels %d\n", avctx->ch_layout.nb_channels);
         goto error;
     }
 
@@ -375,9 +375,9 @@ static int aac_encode_frame(AVCodecContext *avctx, AVPacket *avpkt,
         in_args.numInSamples = -1;
     } else {
         in_ptr               = frame->data[0];
-        in_buffer_size       = 2 * avctx->channels * frame->nb_samples;
+        in_buffer_size       = 2 * avctx->ch_layout.nb_channels * frame->nb_samples;
 
-        in_args.numInSamples = avctx->channels * frame->nb_samples;
+        in_args.numInSamples = avctx->ch_layout.nb_channels * frame->nb_samples;
 
         /* add current frame to the queue */
         if ((ret = ff_af_queue_add(&s->afq, frame)) < 0)
@@ -392,7 +392,8 @@ static int aac_encode_frame(AVCodecContext *avctx, AVPacket *avpkt,
     in_buf.bufElSizes        = &in_buffer_element_size;
 
     /* The maximum packet size is 6144 bits aka 768 bytes per channel. */
-    if ((ret = ff_alloc_packet2(avctx, avpkt, FFMAX(8192, 768 * avctx->channels), 0)) < 0)
+    ret = ff_alloc_packet(avctx, avpkt, FFMAX(8192, 768 * avctx->ch_layout.nb_channels));
+    if (ret < 0)
         return ret;
 
     out_ptr                   = avpkt->data;
@@ -434,11 +435,12 @@ static const AVProfile profiles[] = {
     { FF_PROFILE_UNKNOWN },
 };
 
-static const AVCodecDefault aac_encode_defaults[] = {
+static const FFCodecDefault aac_encode_defaults[] = {
     { "b", "0" },
     { NULL }
 };
 
+#if FF_API_OLD_CHANNEL_LAYOUT
 static const uint64_t aac_channel_layout[] = {
     AV_CH_LAYOUT_MONO,
     AV_CH_LAYOUT_STEREO,
@@ -452,28 +454,46 @@ static const uint64_t aac_channel_layout[] = {
 #endif
     0,
 };
+#endif /* FF_API_OLD_CHANNEL_LAYOUT */
+
+static const AVChannelLayout aac_ch_layouts[16] = {
+    AV_CHANNEL_LAYOUT_MONO,
+    AV_CHANNEL_LAYOUT_STEREO,
+    AV_CHANNEL_LAYOUT_SURROUND,
+    AV_CHANNEL_LAYOUT_4POINT0,
+    AV_CHANNEL_LAYOUT_5POINT0_BACK,
+    AV_CHANNEL_LAYOUT_5POINT1_BACK,
+#ifdef AACENCODER_LIB_VL0
+    AV_CHANNEL_LAYOUT_7POINT1_WIDE_BACK,
+    AV_CHANNEL_LAYOUT_7POINT1,
+#endif
+    { 0 },
+};
 
 static const int aac_sample_rates[] = {
     96000, 88200, 64000, 48000, 44100, 32000,
     24000, 22050, 16000, 12000, 11025, 8000, 0
 };
 
-AVCodec ff_libfdk_aac_encoder = {
-    .name                  = "libfdk_aac",
-    .long_name             = NULL_IF_CONFIG_SMALL("Fraunhofer FDK AAC"),
-    .type                  = AVMEDIA_TYPE_AUDIO,
-    .id                    = AV_CODEC_ID_AAC,
+const FFCodec ff_libfdk_aac_encoder = {
+    .p.name                = "libfdk_aac",
+    .p.long_name           = NULL_IF_CONFIG_SMALL("Fraunhofer FDK AAC"),
+    .p.type                = AVMEDIA_TYPE_AUDIO,
+    .p.id                  = AV_CODEC_ID_AAC,
     .priv_data_size        = sizeof(AACContext),
     .init                  = aac_encode_init,
-    .encode2               = aac_encode_frame,
+    FF_CODEC_ENCODE_CB(aac_encode_frame),
     .close                 = aac_encode_close,
-    .capabilities          = AV_CODEC_CAP_SMALL_LAST_FRAME | AV_CODEC_CAP_DELAY,
-    .sample_fmts           = (const enum AVSampleFormat[]){ AV_SAMPLE_FMT_S16,
+    .p.capabilities        = AV_CODEC_CAP_SMALL_LAST_FRAME | AV_CODEC_CAP_DELAY,
+    .p.sample_fmts         = (const enum AVSampleFormat[]){ AV_SAMPLE_FMT_S16,
                                                             AV_SAMPLE_FMT_NONE },
-    .priv_class            = &aac_enc_class,
+    .p.priv_class          = &aac_enc_class,
     .defaults              = aac_encode_defaults,
-    .profiles              = profiles,
-    .supported_samplerates = aac_sample_rates,
-    .channel_layouts       = aac_channel_layout,
-    .wrapper_name          = "libfdk",
+    .p.profiles            = profiles,
+    .p.supported_samplerates = aac_sample_rates,
+    .p.wrapper_name        = "libfdk",
+#if FF_API_OLD_CHANNEL_LAYOUT
+    .p.channel_layouts     = aac_channel_layout,
+#endif
+    .p.ch_layouts          = aac_ch_layouts,
 };
