@@ -27,6 +27,8 @@
 #include "libavutil/bprint.h"
 #include "libavutil/intreadwrite.h"
 #include "libavutil/mem.h"
+#include "bytestream.h"
+#include "codec_internal.h"
 
 #define STYLE_FLAG_BOLD         (1<<0)
 #define STYLE_FLAG_ITALIC       (1<<1)
@@ -52,37 +54,31 @@
 #define RGB_TO_BGR(c) (((c) & 0xff) << 16 | ((c) & 0xff00) | (((c) >> 16) & 0xff))
 
 typedef struct {
-    uint16_t fontID;
-    const char *font;
-    uint8_t fontsize;
-    int color;
-    uint8_t alpha;
-    int back_color;
-    uint8_t back_alpha;
-    uint8_t bold;
-    uint8_t italic;
-    uint8_t underline;
-    int alignment;
-} MovTextDefault;
-
-typedef struct {
-    uint16_t fontID;
+    uint16_t font_id;
     char *font;
 } FontRecord;
 
 typedef struct {
-    uint16_t style_start;
-    uint16_t style_end;
-    uint8_t style_flag;
+    uint16_t start;
+    uint16_t end;
+    uint8_t flags;
     uint8_t bold;
     uint8_t italic;
     uint8_t underline;
     int color;
     uint8_t alpha;
     uint8_t fontsize;
-    uint16_t style_fontID;
+    uint16_t font_id;
 } StyleBox;
 
+typedef struct {
+    StyleBox style;
+    const char *font;
+    int back_color;
+    uint8_t back_alpha;
+    int alignment;
+} MovTextDefault;
+
 typedef struct {
     uint16_t hlit_start;
     uint16_t hlit_end;
@@ -106,8 +102,6 @@ typedef struct {
     MovTextDefault d;
     uint8_t box_flags;
     uint16_t style_entries, ftab_entries;
-    uint64_t tracksize;
-    int size_var;
     int readorder;
     int frame_width;
     int frame_height;
@@ -115,8 +109,8 @@ typedef struct {
 
 typedef struct {
     uint32_t type;
-    size_t base_size;
-    int (*decode)(const uint8_t *tsmb, MovTextContext *m, AVPacket *avpkt);
+    unsigned base_size;
+    int (*decode)(const uint8_t *tsmb, MovTextContext *m, uint64_t size);
 } Box;
 
 static void mov_text_cleanup(MovTextContext *m)
@@ -135,13 +129,29 @@ static void mov_text_cleanup_ftab(MovTextContext *m)
     m->ftab_entries = 0;
 }
 
+static void mov_text_parse_style_record(StyleBox *style, const uint8_t **ptr)
+{
+    // fontID
+    style->font_id   = bytestream_get_be16(ptr);
+    // face-style-flags
+    style->flags     = bytestream_get_byte(ptr);
+    style->bold      = !!(style->flags & STYLE_FLAG_BOLD);
+    style->italic    = !!(style->flags & STYLE_FLAG_ITALIC);
+    style->underline = !!(style->flags & STYLE_FLAG_UNDERLINE);
+    // fontsize
+    style->fontsize  = bytestream_get_byte(ptr);
+    // Primary color
+    style->color     = bytestream_get_be24(ptr);
+    style->color     = RGB_TO_BGR(style->color);
+    style->alpha     = bytestream_get_byte(ptr);
+}
+
 static int mov_text_tx3g(AVCodecContext *avctx, MovTextContext *m)
 {
-    uint8_t *tx3g_ptr = avctx->extradata;
-    int i, font_length, remaining = avctx->extradata_size - BOX_SIZE_INITIAL;
+    const uint8_t *tx3g_ptr = avctx->extradata;
+    int i, j = -1, font_length, remaining = avctx->extradata_size - BOX_SIZE_INITIAL;
     int8_t v_align, h_align;
     unsigned ftab_entries;
-    StyleBox s_default;
 
     m->ftab_entries = 0;
     if (remaining < 0)
@@ -150,8 +160,8 @@ static int mov_text_tx3g(AVCodecContext *avctx, MovTextContext *m)
     // Display Flags
     tx3g_ptr += 4;
     // Alignment
-    h_align = *tx3g_ptr++;
-    v_align = *tx3g_ptr++;
+    h_align = bytestream_get_byte(&tx3g_ptr);
+    v_align = bytestream_get_byte(&tx3g_ptr);
     if (h_align == 0) {
         if (v_align == 0)
             m->d.alignment = TOP_LEFT;
@@ -177,29 +187,14 @@ static int mov_text_tx3g(AVCodecContext *avctx, MovTextContext *m)
             m->d.alignment = BOTTOM_RIGHT;
     }
     // Background Color
-    m->d.back_color = AV_RB24(tx3g_ptr);
-    tx3g_ptr += 3;
-    m->d.back_alpha = AV_RB8(tx3g_ptr);
-    tx3g_ptr += 1;
+    m->d.back_color = bytestream_get_be24(&tx3g_ptr);
+    m->d.back_color = RGB_TO_BGR(m->d.back_color);
+    m->d.back_alpha = bytestream_get_byte(&tx3g_ptr);
     // BoxRecord
     tx3g_ptr += 8;
     // StyleRecord
     tx3g_ptr += 4;
-    // fontID
-    m->d.fontID = AV_RB16(tx3g_ptr);
-    tx3g_ptr += 2;
-    // face-style-flags
-    s_default.style_flag = *tx3g_ptr++;
-    m->d.bold = !!(s_default.style_flag & STYLE_FLAG_BOLD);
-    m->d.italic = !!(s_default.style_flag & STYLE_FLAG_ITALIC);
-    m->d.underline = !!(s_default.style_flag & STYLE_FLAG_UNDERLINE);
-    // fontsize
-    m->d.fontsize = *tx3g_ptr++;
-    // Primary color
-    m->d.color = AV_RB24(tx3g_ptr);
-    tx3g_ptr += 3;
-    m->d.alpha = AV_RB8(tx3g_ptr);
-    tx3g_ptr += 1;
+    mov_text_parse_style_record(&m->d.style, &tx3g_ptr);
     // FontRecord
     // FontRecord Size
     tx3g_ptr += 4;
@@ -209,7 +204,7 @@ static int mov_text_tx3g(AVCodecContext *avctx, MovTextContext *m)
     // In case of broken header, init default font
     m->d.font = ASS_DEFAULT_FONT;
 
-    ftab_entries = AV_RB16(tx3g_ptr);
+    ftab_entries = bytestream_get_be16(&tx3g_ptr);
     if (!ftab_entries)
         return 0;
     remaining   -= 3 * ftab_entries;
@@ -219,12 +214,12 @@ static int mov_text_tx3g(AVCodecContext *avctx, MovTextContext *m)
     if (!m->ftab)
         return AVERROR(ENOMEM);
     m->ftab_entries = ftab_entries;
-    tx3g_ptr += 2;
 
     for (i = 0; i < m->ftab_entries; i++) {
-        m->ftab[i].fontID = AV_RB16(tx3g_ptr);
-        tx3g_ptr += 2;
-        font_length = *tx3g_ptr++;
+        m->ftab[i].font_id = bytestream_get_be16(&tx3g_ptr);
+        if (m->ftab[i].font_id == m->d.style.font_id)
+            j = i;
+        font_length = bytestream_get_byte(&tx3g_ptr);
 
         remaining  -= font_length;
         if (remaining < 0) {
@@ -236,50 +231,52 @@ static int mov_text_tx3g(AVCodecContext *avctx, MovTextContext *m)
             mov_text_cleanup_ftab(m);
             return AVERROR(ENOMEM);
         }
-        memcpy(m->ftab[i].font, tx3g_ptr, font_length);
+        bytestream_get_buffer(&tx3g_ptr, m->ftab[i].font, font_length);
         m->ftab[i].font[font_length] = '\0';
-        tx3g_ptr = tx3g_ptr + font_length;
-    }
-    for (i = 0; i < m->ftab_entries; i++) {
-        if (m->d.fontID == m->ftab[i].fontID)
-            m->d.font = m->ftab[i].font;
     }
+    if (j >= 0)
+        m->d.font = m->ftab[j].font;
     return 0;
 }
 
-static int decode_twrp(const uint8_t *tsmb, MovTextContext *m, AVPacket *avpkt)
+static int decode_twrp(const uint8_t *tsmb, MovTextContext *m, uint64_t size)
 {
     m->box_flags |= TWRP_BOX;
-    m->w.wrap_flag = *tsmb++;
+    m->w.wrap_flag = bytestream_get_byte(&tsmb);
     return 0;
 }
 
-static int decode_hlit(const uint8_t *tsmb, MovTextContext *m, AVPacket *avpkt)
+static int decode_hlit(const uint8_t *tsmb, MovTextContext *m, uint64_t size)
 {
     m->box_flags |= HLIT_BOX;
-    m->h.hlit_start = AV_RB16(tsmb);
-    tsmb += 2;
-    m->h.hlit_end = AV_RB16(tsmb);
-    tsmb += 2;
+    m->h.hlit_start = bytestream_get_be16(&tsmb);
+    m->h.hlit_end   = bytestream_get_be16(&tsmb);
     return 0;
 }
 
-static int decode_hclr(const uint8_t *tsmb, MovTextContext *m, AVPacket *avpkt)
+static int decode_hclr(const uint8_t *tsmb, MovTextContext *m, uint64_t size)
 {
     m->box_flags |= HCLR_BOX;
-    memcpy(m->c.hlit_color, tsmb, 4);
-    tsmb += 4;
+    bytestream_get_buffer(&tsmb, m->c.hlit_color, 4);
     return 0;
 }
 
-static int decode_styl(const uint8_t *tsmb, MovTextContext *m, AVPacket *avpkt)
+static int styles_equivalent(const StyleBox *a, const StyleBox *b)
+{
+#define CMP(field) ((a)->field == (b)->field)
+    return CMP(bold)  && CMP(italic)   && CMP(underline) && CMP(color) &&
+           CMP(alpha) && CMP(fontsize) && CMP(font_id);
+#undef CMP
+}
+
+static int decode_styl(const uint8_t *tsmb, MovTextContext *m, uint64_t size)
 {
     int i;
-    int style_entries = AV_RB16(tsmb);
+    int style_entries = bytestream_get_be16(&tsmb);
     StyleBox *tmp;
-    tsmb += 2;
+
     // A single style record is of length 12 bytes.
-    if (m->tracksize + m->size_var + 2 + style_entries * 12 > avpkt->size)
+    if (2 + style_entries * 12 > size)
         return -1;
 
     tmp = av_realloc_array(m->s, style_entries, sizeof(*m->s));
@@ -291,30 +288,36 @@ static int decode_styl(const uint8_t *tsmb, MovTextContext *m, AVPacket *avpkt)
     m->box_flags |= STYL_BOX;
     for(i = 0; i < m->style_entries; i++) {
         StyleBox *style = &m->s[i];
-        style->style_start = AV_RB16(tsmb);
-        tsmb += 2;
-        style->style_end = AV_RB16(tsmb);
 
-        if (   style->style_end < style->style_start
-            || (i && style->style_start < m->s[i - 1].style_end)) {
+        style->start = bytestream_get_be16(&tsmb);
+        style->end   = bytestream_get_be16(&tsmb);
+        if (style->end < style->start ||
+            (i && style->start < m->s[i - 1].end)) {
             mov_text_cleanup(m);
-            return AVERROR(ENOMEM);
+            return AVERROR_INVALIDDATA;
+        }
+        if (style->start == style->end) {
+            /* Skip this style as it applies to no character */
+            tsmb += 8;
+            m->style_entries--;
+            i--;
+            continue;
         }
 
-        tsmb += 2;
-        style->style_fontID = AV_RB16(tsmb);
-        tsmb += 2;
-        style->style_flag = AV_RB8(tsmb);
-        style->bold      = !!(style->style_flag & STYLE_FLAG_BOLD);
-        style->italic    = !!(style->style_flag & STYLE_FLAG_ITALIC);
-        style->underline = !!(style->style_flag & STYLE_FLAG_UNDERLINE);
-        tsmb++;
-        style->fontsize = AV_RB8(tsmb);
-        tsmb++;
-        style->color = AV_RB24(tsmb);
-        tsmb += 3;
-        style->alpha = AV_RB8(tsmb);
-        tsmb++;
+        mov_text_parse_style_record(style, &tsmb);
+        if (styles_equivalent(style, &m->d.style)) {
+            /* Skip this style as it is equivalent to the default style */
+            m->style_entries--;
+            i--;
+            continue;
+        } else if (i && style->start == style[-1].end &&
+                   styles_equivalent(style, &style[-1])) {
+            /* Merge the two adjacent styles */
+            style[-1].end = style->end;
+            m->style_entries--;
+            i--;
+            continue;
+        }
     }
     return 0;
 }
@@ -346,11 +349,11 @@ static int text_to_ass(AVBPrint *buf, const char *text, const char *text_end,
                        AVCodecContext *avctx)
 {
     MovTextContext *m = avctx->priv_data;
+    const StyleBox *const default_style = &m->d.style;
     int i = 0;
     int text_pos = 0;
-    int style_active = 0;
     int entry = 0;
-    int color = m->d.color;
+    int color = default_style->color;
 
     if (text < text_end && m->box_flags & TWRP_BOX) {
         if (m->w.wrap_flag == 1) {
@@ -365,36 +368,33 @@ static int text_to_ass(AVBPrint *buf, const char *text, const char *text_end,
 
         if ((m->box_flags & STYL_BOX) && entry < m->style_entries) {
             const StyleBox *style = &m->s[entry];
-            if (text_pos == style->style_start) {
-                style_active = 1;
-                if (style->bold ^ m->d.bold)
+            if (text_pos == style->end) {
+                av_bprintf(buf, "{\\r}");
+                color = default_style->color;
+                entry++;
+                style++;
+            }
+            if (entry < m->style_entries && text_pos == style->start) {
+                if (style->bold ^ default_style->bold)
                     av_bprintf(buf, "{\\b%d}", style->bold);
-                if (style->italic ^ m->d.italic)
+                if (style->italic ^ default_style->italic)
                     av_bprintf(buf, "{\\i%d}", style->italic);
-                if (style->underline ^ m->d.underline)
+                if (style->underline ^ default_style->underline)
                     av_bprintf(buf, "{\\u%d}", style->underline);
-                if (style->fontsize != m->d.fontsize)
+                if (style->fontsize != default_style->fontsize)
                     av_bprintf(buf, "{\\fs%d}", style->fontsize);
-                if (style->style_fontID != m->d.fontID)
+                if (style->font_id != default_style->font_id)
                     for (i = 0; i < m->ftab_entries; i++) {
-                        if (style->style_fontID == m->ftab[i].fontID)
+                        if (style->font_id == m->ftab[i].font_id)
                             av_bprintf(buf, "{\\fn%s}", m->ftab[i].font);
                     }
-                if (m->d.color != style->color) {
+                if (default_style->color != style->color) {
                     color = style->color;
-                    av_bprintf(buf, "{\\1c&H%X&}", RGB_TO_BGR(color));
+                    av_bprintf(buf, "{\\1c&H%X&}", color);
                 }
-                if (m->d.alpha != style->alpha)
+                if (default_style->alpha != style->alpha)
                     av_bprintf(buf, "{\\1a&H%02X&}", 255 - style->alpha);
             }
-            if (text_pos == style->style_end) {
-                if (style_active) {
-                    av_bprintf(buf, "{\\r}");
-                    style_active = 0;
-                    color = m->d.color;
-                }
-                entry++;
-            }
         }
         if (m->box_flags & HLIT_BOX) {
             if (text_pos == m->h.hlit_start) {
@@ -413,10 +413,10 @@ static int text_to_ass(AVBPrint *buf, const char *text, const char *text_end,
             }
             if (text_pos == m->h.hlit_end) {
                 if (m->box_flags & HCLR_BOX) {
-                    av_bprintf(buf, "{\\2c&H%X&}", RGB_TO_BGR(m->d.color));
+                    av_bprintf(buf, "{\\2c&H%X&}", default_style->color);
                 } else {
                     av_bprintf(buf, "{\\1c&H%X&}{\\2c&H%X&}",
-                               RGB_TO_BGR(color), RGB_TO_BGR(m->d.color));
+                               color, default_style->color);
                 }
             }
         }
@@ -426,19 +426,17 @@ static int text_to_ass(AVBPrint *buf, const char *text, const char *text_end,
             av_log(avctx, AV_LOG_ERROR, "invalid UTF-8 byte in subtitle\n");
             len = 1;
         }
-        for (i = 0; i < len; i++) {
-            switch (*text) {
-            case '\r':
-                break;
-            case '\n':
-                av_bprintf(buf, "\\N");
-                break;
-            default:
-                av_bprint_chars(buf, *text, 1);
-                break;
-            }
-            text++;
+        switch (*text) {
+        case '\r':
+            break;
+        case '\n':
+            av_bprintf(buf, "\\N");
+            break;
+        default:
+            av_bprint_append_data(buf, text, len);
+            break;
         }
+        text += len;
         text_pos++;
     }
 
@@ -456,35 +454,32 @@ static int mov_text_init(AVCodecContext *avctx) {
     MovTextContext *m = avctx->priv_data;
     ret = mov_text_tx3g(avctx, m);
     if (ret == 0) {
+        const StyleBox *const default_style = &m->d.style;
         if (!m->frame_width || !m->frame_height) {
             m->frame_width = ASS_DEFAULT_PLAYRESX;
             m->frame_height = ASS_DEFAULT_PLAYRESY;
         }
         return ff_ass_subtitle_header_full(avctx,
                     m->frame_width, m->frame_height,
-                    m->d.font, m->d.fontsize,
-                    (255U - m->d.alpha) << 24 | RGB_TO_BGR(m->d.color),
-                    (255U - m->d.alpha) << 24 | RGB_TO_BGR(m->d.color),
-                    (255U - m->d.back_alpha) << 24 | RGB_TO_BGR(m->d.back_color),
-                    (255U - m->d.back_alpha) << 24 | RGB_TO_BGR(m->d.back_color),
-                    m->d.bold, m->d.italic, m->d.underline,
+                    m->d.font, default_style->fontsize,
+                    (255U - default_style->alpha) << 24 | default_style->color,
+                    (255U - default_style->alpha) << 24 | default_style->color,
+                    (255U - m->d.back_alpha) << 24 | m->d.back_color,
+                    (255U - m->d.back_alpha) << 24 | m->d.back_color,
+                    default_style->bold, default_style->italic, default_style->underline,
                     ASS_DEFAULT_BORDERSTYLE, m->d.alignment);
     } else
         return ff_ass_subtitle_header_default(avctx);
 }
 
-static int mov_text_decode_frame(AVCodecContext *avctx,
-                            void *data, int *got_sub_ptr, AVPacket *avpkt)
+static int mov_text_decode_frame(AVCodecContext *avctx, AVSubtitle *sub,
+                                 int *got_sub_ptr, const AVPacket *avpkt)
 {
-    AVSubtitle *sub = data;
     MovTextContext *m = avctx->priv_data;
     int ret;
     AVBPrint buf;
-    char *ptr = avpkt->data;
-    char *end;
-    int text_length, tsmb_type, ret_tsmb;
-    uint64_t tsmb_size;
-    const uint8_t *tsmb;
+    const char *ptr = avpkt->data, *end;
+    int text_length;
     size_t i;
 
     if (!ptr || avpkt->size < 2)
@@ -511,49 +506,47 @@ static int mov_text_decode_frame(AVCodecContext *avctx,
 
     mov_text_cleanup(m);
 
-    tsmb_size = 0;
-    m->tracksize = 2 + text_length;
     m->style_entries = 0;
     m->box_flags = 0;
     // Note that the spec recommends lines be no longer than 2048 characters.
     av_bprint_init(&buf, 0, AV_BPRINT_SIZE_UNLIMITED);
-    if (text_length + 2 != avpkt->size) {
-        while (m->tracksize + 8 <= avpkt->size) {
-            // A box is a minimum of 8 bytes.
-            tsmb = ptr + m->tracksize - 2;
-            tsmb_size = AV_RB32(tsmb);
-            tsmb += 4;
-            tsmb_type = AV_RB32(tsmb);
-            tsmb += 4;
+    if (text_length + 2 < avpkt->size) {
+        const uint8_t *tsmb = end;
+        const uint8_t *const tsmb_end = avpkt->data + avpkt->size;
+        // A box is a minimum of 8 bytes.
+        while (tsmb_end - tsmb >= 8) {
+            uint64_t tsmb_size = bytestream_get_be32(&tsmb);
+            uint32_t tsmb_type = bytestream_get_be32(&tsmb);
+            int size_var, ret_tsmb;
 
             if (tsmb_size == 1) {
-                if (m->tracksize + 16 > avpkt->size)
+                if (tsmb_end - tsmb < 8)
                     break;
-                tsmb_size = AV_RB64(tsmb);
-                tsmb += 8;
-                m->size_var = 16;
+                tsmb_size = bytestream_get_be64(&tsmb);
+                size_var = 16;
             } else
-                m->size_var = 8;
+                size_var = 8;
             //size_var is equal to 8 or 16 depending on the size of box
 
-            if (tsmb_size == 0) {
-                av_log(avctx, AV_LOG_ERROR, "tsmb_size is 0\n");
+            if (tsmb_size < size_var) {
+                av_log(avctx, AV_LOG_ERROR, "tsmb_size invalid\n");
                 return AVERROR_INVALIDDATA;
             }
+            tsmb_size -= size_var;
 
-            if (tsmb_size > avpkt->size - m->tracksize)
+            if (tsmb_end - tsmb < tsmb_size)
                 break;
 
             for (i = 0; i < box_count; i++) {
                 if (tsmb_type == box_types[i].type) {
-                    if (m->tracksize + m->size_var + box_types[i].base_size > avpkt->size)
+                    if (tsmb_size < box_types[i].base_size)
                         break;
-                    ret_tsmb = box_types[i].decode(tsmb, m, avpkt);
+                    ret_tsmb = box_types[i].decode(tsmb, m, tsmb_size);
                     if (ret_tsmb == -1)
                         break;
                 }
             }
-            m->tracksize = m->tracksize + tsmb_size;
+            tsmb += tsmb_size;
         }
         text_to_ass(&buf, ptr, end, avctx);
         mov_text_cleanup(m);
@@ -598,15 +591,16 @@ static const AVClass mov_text_decoder_class = {
     .version    = LIBAVUTIL_VERSION_INT,
 };
 
-AVCodec ff_movtext_decoder = {
-    .name         = "mov_text",
-    .long_name    = NULL_IF_CONFIG_SMALL("3GPP Timed Text subtitle"),
-    .type         = AVMEDIA_TYPE_SUBTITLE,
-    .id           = AV_CODEC_ID_MOV_TEXT,
+const FFCodec ff_movtext_decoder = {
+    .p.name       = "mov_text",
+    .p.long_name  = NULL_IF_CONFIG_SMALL("3GPP Timed Text subtitle"),
+    .p.type       = AVMEDIA_TYPE_SUBTITLE,
+    .p.id         = AV_CODEC_ID_MOV_TEXT,
     .priv_data_size = sizeof(MovTextContext),
-    .priv_class   = &mov_text_decoder_class,
+    .p.priv_class = &mov_text_decoder_class,
     .init         = mov_text_init,
-    .decode       = mov_text_decode_frame,
+    FF_CODEC_DECODE_SUB_CB(mov_text_decode_frame),
     .close        = mov_text_decode_close,
     .flush        = mov_text_flush,
+    .caps_internal  = FF_CODEC_CAP_INIT_THREADSAFE,
 };
