@@ -21,7 +21,9 @@
  */
 
 #include "avcodec.h"
+#include "internal.h"
 #include "mpeg4video.h"
+#include "mpeg4videodec.h"
 #include "nvdec.h"
 #include "decode.h"
 
@@ -104,7 +106,7 @@ static int nvdec_mpeg4_frame_params(AVCodecContext *avctx,
                                   AVBufferRef *hw_frames_ctx)
 {
     // Each frame can at most have one P and one B reference
-    return ff_nvdec_frame_params(avctx, hw_frames_ctx, 2);
+    return ff_nvdec_frame_params(avctx, hw_frames_ctx, 2, 0);
 }
 
 const AVHWAccel ff_mpeg4_nvdec_hwaccel = {
