@@ -24,13 +24,14 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "libavutil/avassert.h"
 #include "libavutil/imgutils.h"
 #include "libavutil/opt.h"
 
 #include "avcodec.h"
 #include "bytestream.h"
+#include "codec_internal.h"
 #include "internal.h"
+#include "zlib_wrapper.h"
 
 #include <zlib.h>
 
@@ -59,10 +60,10 @@ typedef struct RASCContext {
     unsigned        cursor_y;
     int             stride;
     int             bpp;
-    z_stream        zstream;
     AVFrame        *frame;
     AVFrame        *frame1;
     AVFrame        *frame2;
+    FFZStream       zstream;
 } RASCContext;
 
 static void clear_plane(AVCodecContext *avctx, AVFrame *frame)
@@ -112,7 +113,7 @@ static int init_frames(AVCodecContext *avctx)
 }
 
 static int decode_fint(AVCodecContext *avctx,
-                       AVPacket *avpkt, unsigned size)
+                       const AVPacket *avpkt, unsigned size)
 {
     RASCContext *s = avctx->priv_data;
     GetByteContext *gb = &s->gb;
@@ -171,14 +172,15 @@ static int decode_fint(AVCodecContext *avctx,
     return 0;
 }
 
-static int decode_zlib(AVCodecContext *avctx, AVPacket *avpkt,
+static int decode_zlib(AVCodecContext *avctx, const AVPacket *avpkt,
                        unsigned size, unsigned uncompressed_size)
 {
     RASCContext *s = avctx->priv_data;
+    z_stream *const zstream = &s->zstream.zstream;
     GetByteContext *gb = &s->gb;
     int zret;
 
-    zret = inflateReset(&s->zstream);
+    zret = inflateReset(zstream);
     if (zret != Z_OK) {
         av_log(avctx, AV_LOG_ERROR, "Inflate reset error: %d\n", zret);
         return AVERROR_EXTERNAL;
@@ -188,13 +190,13 @@ static int decode_zlib(AVCodecContext *avctx, AVPacket *avpkt,
     if (!s->delta)
         return AVERROR(ENOMEM);
 
-    s->zstream.next_in  = avpkt->data + bytestream2_tell(gb);
-    s->zstream.avail_in = FFMIN(size, bytestream2_get_bytes_left(gb));
+    zstream->next_in  = avpkt->data + bytestream2_tell(gb);
+    zstream->avail_in = FFMIN(size, bytestream2_get_bytes_left(gb));
 
-    s->zstream.next_out  = s->delta;
-    s->zstream.avail_out = s->delta_size;
+    zstream->next_out  = s->delta;
+    zstream->avail_out = s->delta_size;
 
-    zret = inflate(&s->zstream, Z_FINISH);
+    zret = inflate(zstream, Z_FINISH);
     if (zret != Z_STREAM_END) {
         av_log(avctx, AV_LOG_ERROR,
                "Inflate failed with return code: %d.\n", zret);
@@ -205,7 +207,7 @@ static int decode_zlib(AVCodecContext *avctx, AVPacket *avpkt,
 }
 
 static int decode_move(AVCodecContext *avctx,
-                       AVPacket *avpkt, unsigned size)
+                       const AVPacket *avpkt, unsigned size)
 {
     RASCContext *s = avctx->priv_data;
     GetByteContext *gb = &s->gb;
@@ -329,7 +331,7 @@ static int decode_move(AVCodecContext *avctx,
     len--;
 
 static int decode_dlta(AVCodecContext *avctx,
-                       AVPacket *avpkt, unsigned size)
+                       const AVPacket *avpkt, unsigned size)
 {
     RASCContext *s = avctx->priv_data;
     GetByteContext *gb = &s->gb;
@@ -471,9 +473,10 @@ static int decode_dlta(AVCodecContext *avctx,
 }
 
 static int decode_kfrm(AVCodecContext *avctx,
-                       AVPacket *avpkt, unsigned size)
+                       const AVPacket *avpkt, unsigned size)
 {
     RASCContext *s = avctx->priv_data;
+    z_stream *const zstream = &s->zstream.zstream;
     GetByteContext *gb = &s->gb;
     uint8_t *dst;
     unsigned pos;
@@ -489,21 +492,21 @@ static int decode_kfrm(AVCodecContext *avctx,
     if (!s->frame2->data[0])
         return AVERROR_INVALIDDATA;
 
-    zret = inflateReset(&s->zstream);
+    zret = inflateReset(zstream);
     if (zret != Z_OK) {
         av_log(avctx, AV_LOG_ERROR, "Inflate reset error: %d\n", zret);
         return AVERROR_EXTERNAL;
     }
 
-    s->zstream.next_in  = avpkt->data + bytestream2_tell(gb);
-    s->zstream.avail_in = bytestream2_get_bytes_left(gb);
+    zstream->next_in  = avpkt->data + bytestream2_tell(gb);
+    zstream->avail_in = bytestream2_get_bytes_left(gb);
 
     dst = s->frame2->data[0] + (avctx->height - 1) * s->frame2->linesize[0];
     for (int i = 0; i < avctx->height; i++) {
-        s->zstream.next_out  = dst;
-        s->zstream.avail_out = s->stride;
+        zstream->next_out  = dst;
+        zstream->avail_out = s->stride;
 
-        zret = inflate(&s->zstream, Z_SYNC_FLUSH);
+        zret = inflate(zstream, Z_SYNC_FLUSH);
         if (zret != Z_OK && zret != Z_STREAM_END) {
             av_log(avctx, AV_LOG_ERROR,
                    "Inflate failed with return code: %d.\n", zret);
@@ -515,10 +518,10 @@ static int decode_kfrm(AVCodecContext *avctx,
 
     dst = s->frame1->data[0] + (avctx->height - 1) * s->frame1->linesize[0];
     for (int i = 0; i < avctx->height; i++) {
-        s->zstream.next_out  = dst;
-        s->zstream.avail_out = s->stride;
+        zstream->next_out  = dst;
+        zstream->avail_out = s->stride;
 
-        zret = inflate(&s->zstream, Z_SYNC_FLUSH);
+        zret = inflate(zstream, Z_SYNC_FLUSH);
         if (zret != Z_OK && zret != Z_STREAM_END) {
             av_log(avctx, AV_LOG_ERROR,
                    "Inflate failed with return code: %d.\n", zret);
@@ -534,7 +537,7 @@ static int decode_kfrm(AVCodecContext *avctx,
 }
 
 static int decode_mous(AVCodecContext *avctx,
-                       AVPacket *avpkt, unsigned size)
+                       const AVPacket *avpkt, unsigned size)
 {
     RASCContext *s = avctx->priv_data;
     GetByteContext *gb = &s->gb;
@@ -574,7 +577,7 @@ static int decode_mous(AVCodecContext *avctx,
 }
 
 static int decode_mpos(AVCodecContext *avctx,
-                       AVPacket *avpkt, unsigned size)
+                       const AVPacket *avpkt, unsigned size)
 {
     RASCContext *s = avctx->priv_data;
     GetByteContext *gb = &s->gb;
@@ -668,14 +671,12 @@ static void draw_cursor(AVCodecContext *avctx)
     }
 }
 
-static int decode_frame(AVCodecContext *avctx,
-                        void *data, int *got_frame,
-                        AVPacket *avpkt)
+static int decode_frame(AVCodecContext *avctx, AVFrame *frame,
+                        int *got_frame, AVPacket *avpkt)
 {
     RASCContext *s = avctx->priv_data;
     GetByteContext *gb = &s->gb;
     int ret, intra = 0;
-    AVFrame *frame = data;
 
     bytestream2_init(gb, avpkt->data, avpkt->size);
 
@@ -722,6 +723,7 @@ static int decode_frame(AVCodecContext *avctx,
             break;
         default:
             bytestream2_skip(gb, size);
+            ret = 0;
         }
 
         if (ret < 0)
@@ -751,23 +753,13 @@ static int decode_frame(AVCodecContext *avctx,
 static av_cold int decode_init(AVCodecContext *avctx)
 {
     RASCContext *s = avctx->priv_data;
-    int zret;
-
-    s->zstream.zalloc = Z_NULL;
-    s->zstream.zfree = Z_NULL;
-    s->zstream.opaque = Z_NULL;
-    zret = inflateInit(&s->zstream);
-    if (zret != Z_OK) {
-        av_log(avctx, AV_LOG_ERROR, "Inflate init error: %d\n", zret);
-        return AVERROR_EXTERNAL;
-    }
 
     s->frame1 = av_frame_alloc();
     s->frame2 = av_frame_alloc();
     if (!s->frame1 || !s->frame2)
         return AVERROR(ENOMEM);
 
-    return 0;
+    return ff_inflate_init(&s->zstream, avctx);
 }
 
 static av_cold int decode_close(AVCodecContext *avctx)
@@ -780,7 +772,7 @@ static av_cold int decode_close(AVCodecContext *avctx)
     s->delta_size = 0;
     av_frame_free(&s->frame1);
     av_frame_free(&s->frame2);
-    inflateEnd(&s->zstream);
+    ff_inflate_end(&s->zstream);
 
     return 0;
 }
@@ -805,18 +797,18 @@ static const AVClass rasc_decoder_class = {
     .version    = LIBAVUTIL_VERSION_INT,
 };
 
-AVCodec ff_rasc_decoder = {
-    .name             = "rasc",
-    .long_name        = NULL_IF_CONFIG_SMALL("RemotelyAnywhere Screen Capture"),
-    .type             = AVMEDIA_TYPE_VIDEO,
-    .id               = AV_CODEC_ID_RASC,
+const FFCodec ff_rasc_decoder = {
+    .p.name           = "rasc",
+    .p.long_name      = NULL_IF_CONFIG_SMALL("RemotelyAnywhere Screen Capture"),
+    .p.type           = AVMEDIA_TYPE_VIDEO,
+    .p.id             = AV_CODEC_ID_RASC,
     .priv_data_size   = sizeof(RASCContext),
     .init             = decode_init,
     .close            = decode_close,
-    .decode           = decode_frame,
+    FF_CODEC_DECODE_CB(decode_frame),
     .flush            = decode_flush,
-    .capabilities     = AV_CODEC_CAP_DR1,
+    .p.capabilities   = AV_CODEC_CAP_DR1,
     .caps_internal    = FF_CODEC_CAP_INIT_THREADSAFE |
                         FF_CODEC_CAP_INIT_CLEANUP,
-    .priv_class       = &rasc_decoder_class,
+    .p.priv_class     = &rasc_decoder_class,
 };
