@@ -29,10 +29,9 @@
 #include "libavutil/internal.h"
 
 #include "avcodec.h"
-#include "internal.h"
 #include "ratecontrol.h"
 #include "mpegutils.h"
-#include "mpegvideo.h"
+#include "mpegvideoenc.h"
 #include "libavutil/eval.h"
 
 void ff_write_pass1_stats(MpegEncContext *s)
