@@ -20,10 +20,13 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
+#include "libavutil/channel_layout.h"
 #include "libavutil/intreadwrite.h"
 #include "libavutil/opt.h"
 #include "libavutil/log.h"
+#include "libavutil/reverse.h"
 #include "avcodec.h"
+#include "codec_internal.h"
 #include "internal.h"
 #include "mathops.h"
 
@@ -71,32 +74,37 @@ static int s302m_parse_frame_header(AVCodecContext *avctx, const uint8_t *buf,
     else
         avctx->sample_fmt = AV_SAMPLE_FMT_S16;
 
-    avctx->channels    = channels;
+    av_channel_layout_uninit(&avctx->ch_layout);
     switch(channels) {
         case 2:
-            avctx->channel_layout = AV_CH_LAYOUT_STEREO;
+            avctx->ch_layout = (AVChannelLayout)AV_CHANNEL_LAYOUT_STEREO;
             break;
         case 4:
-            avctx->channel_layout = AV_CH_LAYOUT_QUAD;
+            avctx->ch_layout = (AVChannelLayout)AV_CHANNEL_LAYOUT_QUAD;
             break;
         case 6:
-            avctx->channel_layout = AV_CH_LAYOUT_5POINT1_BACK;
+            avctx->ch_layout = (AVChannelLayout)AV_CHANNEL_LAYOUT_5POINT1_BACK;
             break;
         case 8:
-            avctx->channel_layout = AV_CH_LAYOUT_5POINT1_BACK | AV_CH_LAYOUT_STEREO_DOWNMIX;
+            av_channel_layout_from_mask(&avctx->ch_layout,
+                                        AV_CH_LAYOUT_5POINT1_BACK | AV_CH_LAYOUT_STEREO_DOWNMIX);
+            break;
+        default:
+            avctx->ch_layout.order       = AV_CHANNEL_ORDER_UNSPEC;
+            avctx->ch_layout.nb_channels = channels;
+            break;
     }
 
     return frame_size;
 }
 
-static int s302m_decode_frame(AVCodecContext *avctx, void *data,
+static int s302m_decode_frame(AVCodecContext *avctx, AVFrame *frame,
                               int *got_frame_ptr, AVPacket *avpkt)
 {
     S302Context *s = avctx->priv_data;
-    AVFrame *frame     = data;
     const uint8_t *buf = avpkt->data;
     int buf_size       = avpkt->size;
-    int block_size, ret;
+    int block_size, ret, channels;
     int i;
     int non_pcm_data_type = -1;
 
@@ -109,13 +117,14 @@ static int s302m_decode_frame(AVCodecContext *avctx, void *data,
 
     /* get output buffer */
     block_size = (avctx->bits_per_raw_sample + 4) / 4;
-    frame->nb_samples = 2 * (buf_size / block_size) / avctx->channels;
+    channels = avctx->ch_layout.nb_channels;
+    frame->nb_samples = 2 * (buf_size / block_size) / channels;
     if ((ret = ff_get_buffer(avctx, frame, 0)) < 0)
         return ret;
 
-    avctx->bit_rate = 48000 * avctx->channels * (avctx->bits_per_raw_sample + 4) +
+    avctx->bit_rate = 48000 * channels * (avctx->bits_per_raw_sample + 4) +
                       32 * 48000 / frame->nb_samples;
-    buf_size = (frame->nb_samples * avctx->channels / 2) * block_size;
+    buf_size = (frame->nb_samples * channels / 2) * block_size;
 
     if (avctx->bits_per_raw_sample == 24) {
         uint32_t *o = (uint32_t *)frame->data[0];
@@ -130,7 +139,7 @@ static int s302m_decode_frame(AVCodecContext *avctx, void *data,
             buf += 7;
         }
         o = (uint32_t *)frame->data[0];
-        if (avctx->channels == 2)
+        if (channels == 2)
             for (i=0; i<frame->nb_samples * 2 - 6; i+=2) {
                 if (o[i] || o[i+1] || o[i+2] || o[i+3])
                     break;
@@ -151,7 +160,7 @@ static int s302m_decode_frame(AVCodecContext *avctx, void *data,
             buf += 6;
         }
         o = (uint32_t *)frame->data[0];
-        if (avctx->channels == 2)
+        if (channels == 2)
             for (i=0; i<frame->nb_samples * 2 - 6; i+=2) {
                 if (o[i] || o[i+1] || o[i+2] || o[i+3])
                     break;
@@ -171,7 +180,7 @@ static int s302m_decode_frame(AVCodecContext *avctx, void *data,
             buf += 5;
         }
         o = (uint16_t *)frame->data[0];
-        if (avctx->channels == 2)
+        if (channels == 2)
             for (i=0; i<frame->nb_samples * 2 - 6; i+=2) {
                 if (o[i] || o[i+1] || o[i+2] || o[i+3])
                     break;
@@ -212,19 +221,20 @@ static const AVOption s302m_options[] = {
 };
 
 static const AVClass s302m_class = {
-    "SMPTE 302M Decoder",
-    av_default_item_name,
-    s302m_options,
-    LIBAVUTIL_VERSION_INT,
+    .class_name = "SMPTE 302M Decoder",
+    .item_name  = av_default_item_name,
+    .option     = s302m_options,
+    .version    = LIBAVUTIL_VERSION_INT,
 };
 
-AVCodec ff_s302m_decoder = {
-    .name           = "s302m",
-    .long_name      = NULL_IF_CONFIG_SMALL("SMPTE 302M"),
-    .type           = AVMEDIA_TYPE_AUDIO,
-    .id             = AV_CODEC_ID_S302M,
+const FFCodec ff_s302m_decoder = {
+    .p.name         = "s302m",
+    .p.long_name    = NULL_IF_CONFIG_SMALL("SMPTE 302M"),
+    .p.type         = AVMEDIA_TYPE_AUDIO,
+    .p.id           = AV_CODEC_ID_S302M,
+    .p.priv_class   = &s302m_class,
     .priv_data_size = sizeof(S302Context),
-    .decode         = s302m_decode_frame,
-    .capabilities   = AV_CODEC_CAP_DR1,
-    .priv_class     = &s302m_class,
+    FF_CODEC_DECODE_CB(s302m_decode_frame),
+    .p.capabilities = AV_CODEC_CAP_CHANNEL_CONF |
+                      AV_CODEC_CAP_DR1,
 };
