@@ -217,17 +217,18 @@ static void dec_gain(TwinVQContext *tctx,
     const TwinVQModeTab   *mtab =  tctx->mtab;
     const TwinVQFrameData *bits = &tctx->bits[tctx->cur_frame];
     int i, j;
+    int channels   = tctx->avctx->ch_layout.nb_channels;
     int sub        = mtab->fmode[ftype].sub;
     float step     = TWINVQ_AMP_MAX     / ((1 << TWINVQ_GAIN_BITS)     - 1);
     float sub_step = TWINVQ_SUB_AMP_MAX / ((1 << TWINVQ_SUB_GAIN_BITS) - 1);
 
     if (ftype == TWINVQ_FT_LONG) {
-        for (i = 0; i < tctx->avctx->channels; i++)
+        for (i = 0; i < channels; i++)
             out[i] = (1.0 / (1 << 13)) *
                      twinvq_mulawinv(step * 0.5 + step * bits->gain_bits[i],
                                      TWINVQ_AMP_MAX, TWINVQ_MULAW_MU);
     } else {
-        for (i = 0; i < tctx->avctx->channels; i++) {
+        for (i = 0; i < channels; i++) {
             float val = (1.0 / (1 << 23)) *
                         twinvq_mulawinv(step * 0.5 + step * bits->gain_bits[i],
                                         TWINVQ_AMP_MAX, TWINVQ_MULAW_MU);
@@ -380,10 +381,11 @@ static void imdct_output(TwinVQContext *tctx, enum TwinVQFrameType ftype,
 {
     const TwinVQModeTab *mtab = tctx->mtab;
     float *prev_buf           = tctx->prev_frame + tctx->last_block_pos[0];
+    int channels              = tctx->avctx->ch_layout.nb_channels;
     int size1, size2, i;
     float *out1, *out2;
 
-    for (i = 0; i < tctx->avctx->channels; i++)
+    for (i = 0; i < channels; i++)
         imdct_and_window(tctx, ftype, wtype,
                          tctx->spectrum + i * mtab->size,
                          prev_buf + 2 * i * mtab->size,
@@ -399,7 +401,7 @@ static void imdct_output(TwinVQContext *tctx, enum TwinVQFrameType ftype,
     memcpy(out1,         prev_buf,         size1 * sizeof(*out1));
     memcpy(out1 + size1, tctx->curr_frame, size2 * sizeof(*out1));
 
-    if (tctx->avctx->channels == 2) {
+    if (channels == 2) {
         out2 = &out[1][0] + offset;
         memcpy(out2, &prev_buf[2 * mtab->size],
                size1 * sizeof(*out2));
@@ -414,7 +416,7 @@ static void read_and_decode_spectrum(TwinVQContext *tctx, float *out,
 {
     const TwinVQModeTab *mtab = tctx->mtab;
     TwinVQFrameData *bits     = &tctx->bits[tctx->cur_frame];
-    int channels              = tctx->avctx->channels;
+    int channels              = tctx->avctx->ch_layout.nb_channels;
     int sub        = mtab->fmode[ftype].sub;
     int block_size = mtab->size / sub;
     float gain[TWINVQ_CHANNELS_MAX * TWINVQ_SUBBLOCKS_MAX];
@@ -473,10 +475,9 @@ const enum TwinVQFrameType ff_twinvq_wtype_to_ftype_table[] = {
     TWINVQ_FT_MEDIUM
 };
 
-int ff_twinvq_decode_frame(AVCodecContext *avctx, void *data,
+int ff_twinvq_decode_frame(AVCodecContext *avctx, AVFrame *frame,
                            int *got_frame_ptr, AVPacket *avpkt)
 {
-    AVFrame *frame     = data;
     const uint8_t *buf = avpkt->data;
     int buf_size       = avpkt->size;
     TwinVQContext *tctx = avctx->priv_data;
@@ -536,8 +537,9 @@ static av_cold int init_mdct_win(TwinVQContext *tctx)
     const TwinVQModeTab *mtab = tctx->mtab;
     int size_s = mtab->size / mtab->fmode[TWINVQ_FT_SHORT].sub;
     int size_m = mtab->size / mtab->fmode[TWINVQ_FT_MEDIUM].sub;
-    int channels = tctx->avctx->channels;
+    int channels = tctx->avctx->ch_layout.nb_channels;
     float norm = channels == 1 ? 2.0 : 1.0;
+    int table_size = 2 * mtab->size * channels;
 
     for (i = 0; i < 3; i++) {
         int bsize = tctx->mtab->size / tctx->mtab->fmode[i].sub;
@@ -546,25 +548,17 @@ static av_cold int init_mdct_win(TwinVQContext *tctx)
             return ret;
     }
 
-    FF_ALLOC_ARRAY_OR_GOTO(tctx->avctx, tctx->tmp_buf,
-                     mtab->size, sizeof(*tctx->tmp_buf), alloc_fail);
-
-    FF_ALLOC_ARRAY_OR_GOTO(tctx->avctx, tctx->spectrum,
-                     2 * mtab->size, channels * sizeof(*tctx->spectrum),
-                     alloc_fail);
-    FF_ALLOC_ARRAY_OR_GOTO(tctx->avctx, tctx->curr_frame,
-                     2 * mtab->size, channels * sizeof(*tctx->curr_frame),
-                     alloc_fail);
-    FF_ALLOC_ARRAY_OR_GOTO(tctx->avctx, tctx->prev_frame,
-                     2 * mtab->size, channels * sizeof(*tctx->prev_frame),
-                     alloc_fail);
+    if (!FF_ALLOC_TYPED_ARRAY(tctx->tmp_buf,    mtab->size) ||
+        !FF_ALLOC_TYPED_ARRAY(tctx->spectrum,   table_size) ||
+        !FF_ALLOC_TYPED_ARRAY(tctx->curr_frame, table_size) ||
+        !FF_ALLOC_TYPED_ARRAY(tctx->prev_frame, table_size))
+        return AVERROR(ENOMEM);
 
     for (i = 0; i < 3; i++) {
         int m       = 4 * mtab->size / mtab->fmode[i].sub;
         double freq = 2 * M_PI / m;
-        FF_ALLOC_ARRAY_OR_GOTO(tctx->avctx, tctx->cos_tabs[i],
-                         (m / 4), sizeof(*tctx->cos_tabs[i]), alloc_fail);
-
+        if (!FF_ALLOC_TYPED_ARRAY(tctx->cos_tabs[i], m / 4))
+            return AVERROR(ENOMEM);
         for (j = 0; j <= m / 8; j++)
             tctx->cos_tabs[i][j] = cos((2 * j + 1) * freq);
         for (j = 1; j < m / 8; j++)
@@ -576,9 +570,6 @@ static av_cold int init_mdct_win(TwinVQContext *tctx)
     ff_init_ff_sine_windows(av_log2(mtab->size));
 
     return 0;
-
-alloc_fail:
-    return AVERROR(ENOMEM);
 }
 
 /**
@@ -655,10 +646,10 @@ static av_cold void construct_perm_table(TwinVQContext *tctx,
     int16_t *tmp_perm = (int16_t *)tctx->tmp_buf;
 
     if (ftype == TWINVQ_FT_PPC) {
-        size       = tctx->avctx->channels;
+        size       = tctx->avctx->ch_layout.nb_channels;
         block_size = mtab->ppc_shape_len;
     } else {
-        size       = tctx->avctx->channels * mtab->fmode[ftype].sub;
+        size       = tctx->avctx->ch_layout.nb_channels * mtab->fmode[ftype].sub;
         block_size = mtab->size / mtab->fmode[ftype].sub;
     }
 
@@ -676,7 +667,7 @@ static av_cold void construct_perm_table(TwinVQContext *tctx,
 static av_cold void init_bitstream_params(TwinVQContext *tctx)
 {
     const TwinVQModeTab *mtab = tctx->mtab;
-    int n_ch                  = tctx->avctx->channels;
+    int n_ch                  = tctx->avctx->ch_layout.nb_channels;
     int total_fr_bits         = tctx->avctx->bit_rate * mtab->size /
                                 tctx->avctx->sample_rate;
 
@@ -793,13 +784,10 @@ av_cold int ff_twinvq_decode_init(AVCodecContext *avctx)
     tctx->frames_per_packet = frames_per_packet;
 
     tctx->fdsp = avpriv_float_dsp_alloc(avctx->flags & AV_CODEC_FLAG_BITEXACT);
-    if (!tctx->fdsp) {
-        ff_twinvq_decode_close(avctx);
+    if (!tctx->fdsp)
         return AVERROR(ENOMEM);
-    }
     if ((ret = init_mdct_win(tctx))) {
         av_log(avctx, AV_LOG_ERROR, "Error initializing MDCT\n");
-        ff_twinvq_decode_close(avctx);
         return ret;
     }
     init_bitstream_params(tctx);
