@@ -20,8 +20,10 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
+#include "config_components.h"
+
 #include "hwconfig.h"
-#include "internal.h"
+#include "mpegvideodec.h"
 #include "vaapi_decode.h"
 #include "vc1.h"
 #include "vc1data.h"
