@@ -27,6 +27,7 @@
 #include <string.h>
 
 #include "avcodec.h"
+#include "codec_internal.h"
 #include "internal.h"
 
 #include "vp56.h"
@@ -282,7 +283,7 @@ static av_cold int vp5_decode_init(AVCodecContext *avctx)
     VP56Context *s = avctx->priv_data;
     int ret;
 
-    if ((ret = ff_vp56_init(avctx, 1, 0)) < 0)
+    if ((ret = ff_vp56_init_context(avctx, s, 1, 0)) < 0)
         return ret;
     ff_vp5dsp_init(&s->vp56dsp);
     s->vp56_coord_div = vp5_coord_div;
@@ -296,14 +297,21 @@ static av_cold int vp5_decode_init(AVCodecContext *avctx)
     return 0;
 }
 
-const AVCodec ff_vp5_decoder = {
-    .name           = "vp5",
-    .long_name      = NULL_IF_CONFIG_SMALL("On2 VP5"),
-    .type           = AVMEDIA_TYPE_VIDEO,
-    .id             = AV_CODEC_ID_VP5,
+static av_cold int vp56_free(AVCodecContext *avctx)
+{
+    VP56Context *const s = avctx->priv_data;
+    return ff_vp56_free_context(s);
+}
+
+const FFCodec ff_vp5_decoder = {
+    .p.name         = "vp5",
+    .p.long_name    = NULL_IF_CONFIG_SMALL("On2 VP5"),
+    .p.type         = AVMEDIA_TYPE_VIDEO,
+    .p.id           = AV_CODEC_ID_VP5,
     .priv_data_size = sizeof(VP56Context),
     .init           = vp5_decode_init,
-    .close          = ff_vp56_free,
-    .decode         = ff_vp56_decode_frame,
-    .capabilities   = AV_CODEC_CAP_DR1,
+    .close          = vp56_free,
+    FF_CODEC_DECODE_CB(ff_vp56_decode_frame),
+    .p.capabilities = AV_CODEC_CAP_DR1,
+    .caps_internal  = FF_CODEC_CAP_INIT_THREADSAFE | FF_CODEC_CAP_INIT_CLEANUP,
 };
