@@ -26,9 +26,10 @@
 #define BITSTREAM_READER_LE
 #include "avcodec.h"
 #include "bytestream.h"
+#include "codec_internal.h"
 #include "get_bits.h"
-#include "internal.h"
 #include "thread.h"
+#include "threadframe.h"
 #include "unary.h"
 #include "wavpack.h"
 #include "dsd.h"
@@ -128,7 +129,7 @@ static av_always_inline unsigned get_tail(GetBitContext *gb, int k)
     e   = (1 << (p + 1)) - k - 1;
     res = get_bitsz(gb, p);
     if (res >= e)
-        res = (res << 1) - e + get_bits1(gb);
+        res = res * 2U - e + get_bits1(gb);
     return res;
 }
 
@@ -498,6 +499,8 @@ static int wv_unpack_dsd_high(WavpackFrameContext *s, uint8_t *dst_left, uint8_t
                 sp[0].fltr0 = 0;
             }
 
+            if (DSD_BYTE_READY(high, low) && !bytestream2_get_bytes_left(&s->gbyte))
+                return AVERROR_INVALIDDATA;
             while (DSD_BYTE_READY(high, low) && bytestream2_get_bytes_left(&s->gbyte)) {
                 value = (value << 8) | bytestream2_get_byte(&s->gbyte);
                 high = (high << 8) | 0xff;
@@ -533,6 +536,8 @@ static int wv_unpack_dsd_high(WavpackFrameContext *s, uint8_t *dst_left, uint8_t
                 sp[1].fltr0 = 0;
             }
 
+            if (DSD_BYTE_READY(high, low) && !bytestream2_get_bytes_left(&s->gbyte))
+                return AVERROR_INVALIDDATA;
             while (DSD_BYTE_READY(high, low) && bytestream2_get_bytes_left(&s->gbyte)) {
                 value = (value << 8) | bytestream2_get_byte(&s->gbyte);
                 high = (high << 8) | 0xff;
@@ -1018,7 +1023,7 @@ static int update_thread_context(AVCodecContext *dst, const AVCodecContext *src)
     if (dst == src)
         return 0;
 
-    ff_thread_release_buffer(dst, &fdst->curr_frame);
+    ff_thread_release_ext_buffer(dst, &fdst->curr_frame);
     if (fsrc->curr_frame.f->data[0]) {
         if ((ret = ff_thread_ref_frame(&fdst->curr_frame, &fsrc->curr_frame)) < 0)
             return ret;
@@ -1065,10 +1070,10 @@ static av_cold int wavpack_decode_end(AVCodecContext *avctx)
         av_freep(&s->fdec[i]);
     s->fdec_num = 0;
 
-    ff_thread_release_buffer(avctx, &s->curr_frame);
+    ff_thread_release_ext_buffer(avctx, &s->curr_frame);
     av_frame_free(&s->curr_frame.f);
 
-    ff_thread_release_buffer(avctx, &s->prev_frame);
+    ff_thread_release_ext_buffer(avctx, &s->prev_frame);
     av_frame_free(&s->prev_frame.f);
 
     av_buffer_unref(&s->dsd_ref);
@@ -1302,14 +1307,16 @@ static int wavpack_decode_block(AVCodecContext *avctx, int block_no,
                 av_log(avctx, AV_LOG_ERROR,
                        "Invalid INT32INFO, extra_bits = %d (> 30)\n", val[0]);
                 continue;
-            } else if (val[0]) {
+            } else {
                 s->extra_bits = val[0];
-            } else if (val[1]) {
+            }
+            if (val[1])
                 s->shift = val[1];
-            } else if (val[2]) {
+            if (val[2]) {
                 s->and   = s->or = 1;
                 s->shift = val[2];
-            } else if (val[3]) {
+            }
+            if (val[3]) {
                 s->and   = 1;
                 s->shift = val[3];
             }
@@ -1412,25 +1419,23 @@ static int wavpack_decode_block(AVCodecContext *avctx, int block_no,
                 size = bytestream2_get_byte(&gb);
                 chan  |= (bytestream2_get_byte(&gb) & 0xF) << 8;
                 chan  += 1;
-                if (avctx->channels != chan)
+                if (avctx->ch_layout.nb_channels != chan)
                     av_log(avctx, AV_LOG_WARNING, "%i channels signalled"
-                           " instead of %i.\n", chan, avctx->channels);
+                           " instead of %i.\n", chan, avctx->ch_layout.nb_channels);
                 chmask = bytestream2_get_le24(&gb);
                 break;
             case 5:
                 size = bytestream2_get_byte(&gb);
                 chan  |= (bytestream2_get_byte(&gb) & 0xF) << 8;
                 chan  += 1;
-                if (avctx->channels != chan)
+                if (avctx->ch_layout.nb_channels != chan)
                     av_log(avctx, AV_LOG_WARNING, "%i channels signalled"
-                           " instead of %i.\n", chan, avctx->channels);
+                           " instead of %i.\n", chan, avctx->ch_layout.nb_channels);
                 chmask = bytestream2_get_le32(&gb);
                 break;
             default:
                 av_log(avctx, AV_LOG_ERROR, "Invalid channel info size %d\n",
                        size);
-                chan   = avctx->channels;
-                chmask = avctx->channel_layout;
             }
             break;
         case WP_ID_SAMPLE_RATE:
@@ -1494,8 +1499,7 @@ static int wavpack_decode_block(AVCodecContext *avctx, int block_no,
     }
 
     if (!wc->ch_offset) {
-        int      new_channels = avctx->channels;
-        uint64_t new_chmask   = avctx->channel_layout;
+        AVChannelLayout new_ch_layout = { 0 };
         int new_samplerate;
         int sr = (s->frame_flags >> 23) & 0xf;
         if (sr == 0xf) {
@@ -1512,53 +1516,55 @@ static int wavpack_decode_block(AVCodecContext *avctx, int block_no,
         new_samplerate *= rate_x;
 
         if (multiblock) {
-            if (chan)
-                new_channels = chan;
-            if (chmask)
-                new_chmask = chmask;
+            if (chmask) {
+                av_channel_layout_from_mask(&new_ch_layout, chmask);
+                if (chan && new_ch_layout.nb_channels != chan) {
+                    av_log(avctx, AV_LOG_ERROR, "Channel mask does not match the channel count\n");
+                    return AVERROR_INVALIDDATA;
+                }
+            } else {
+                ret = av_channel_layout_copy(&new_ch_layout, &avctx->ch_layout);
+                if (ret < 0) {
+                    av_log(avctx, AV_LOG_ERROR, "Error copying channel layout\n");
+                    return ret;
+                }
+            }
         } else {
-            new_channels = s->stereo ? 2 : 1;
-            new_chmask   = s->stereo ? AV_CH_LAYOUT_STEREO :
-                                       AV_CH_LAYOUT_MONO;
-        }
-
-        if (new_chmask &&
-            av_get_channel_layout_nb_channels(new_chmask) != new_channels) {
-            av_log(avctx, AV_LOG_ERROR, "Channel mask does not match the channel count\n");
-            return AVERROR_INVALIDDATA;
+            av_channel_layout_default(&new_ch_layout, s->stereo + 1);
         }
 
         /* clear DSD state if stream properties change */
-        if (new_channels   != wc->dsd_channels      ||
-            new_chmask     != avctx->channel_layout ||
+        if (new_ch_layout.nb_channels != wc->dsd_channels ||
+            av_channel_layout_compare(&new_ch_layout, &avctx->ch_layout) ||
             new_samplerate != avctx->sample_rate    ||
             !!got_dsd      != !!wc->dsdctx) {
-            ret = wv_dsd_reset(wc, got_dsd ? new_channels : 0);
+            ret = wv_dsd_reset(wc, got_dsd ? new_ch_layout.nb_channels : 0);
             if (ret < 0) {
                 av_log(avctx, AV_LOG_ERROR, "Error reinitializing the DSD context\n");
                 return ret;
             }
-            ff_thread_release_buffer(avctx, &wc->curr_frame);
+            ff_thread_release_ext_buffer(avctx, &wc->curr_frame);
         }
-        avctx->channels            = new_channels;
-        avctx->channel_layout      = new_chmask;
+        av_channel_layout_copy(&avctx->ch_layout, &new_ch_layout);
         avctx->sample_rate         = new_samplerate;
         avctx->sample_fmt          = sample_fmt;
         avctx->bits_per_raw_sample = orig_bpp;
 
-        ff_thread_release_buffer(avctx, &wc->prev_frame);
+        ff_thread_release_ext_buffer(avctx, &wc->prev_frame);
         FFSWAP(ThreadFrame, wc->curr_frame, wc->prev_frame);
 
         /* get output buffer */
         wc->curr_frame.f->nb_samples = s->samples;
-        if ((ret = ff_thread_get_buffer(avctx, &wc->curr_frame, AV_GET_BUFFER_FLAG_REF)) < 0)
+        ret = ff_thread_get_ext_buffer(avctx, &wc->curr_frame,
+                                       AV_GET_BUFFER_FLAG_REF);
+        if (ret < 0)
             return ret;
 
         wc->frame = wc->curr_frame.f;
         ff_thread_finish_setup(avctx);
     }
 
-    if (wc->ch_offset + s->stereo >= avctx->channels) {
+    if (wc->ch_offset + s->stereo >= avctx->ch_layout.nb_channels) {
         av_log(avctx, AV_LOG_WARNING, "Too many channels coded in a packet.\n");
         return ((avctx->err_recognition & AV_EF_EXPLODE) || !wc->ch_offset) ? AVERROR_INVALIDDATA : 0;
     }
@@ -1624,7 +1630,7 @@ static int dsd_channel(AVCodecContext *avctx, void *frmptr, int jobnr, int threa
     return 0;
 }
 
-static int wavpack_decode_frame(AVCodecContext *avctx, void *data,
+static int wavpack_decode_frame(AVCodecContext *avctx, AVFrame *rframe,
                                 int *got_frame_ptr, AVPacket *avpkt)
 {
     WavpackContext *s  = avctx->priv_data;
@@ -1668,21 +1674,21 @@ static int wavpack_decode_frame(AVCodecContext *avctx, void *data,
         buf_size -= frame_size;
     }
 
-    if (s->ch_offset != avctx->channels) {
+    if (s->ch_offset != avctx->ch_layout.nb_channels) {
         av_log(avctx, AV_LOG_ERROR, "Not enough channels coded in a packet.\n");
         ret = AVERROR_INVALIDDATA;
         goto error;
     }
 
     ff_thread_await_progress(&s->prev_frame, INT_MAX, 0);
-    ff_thread_release_buffer(avctx, &s->prev_frame);
+    ff_thread_release_ext_buffer(avctx, &s->prev_frame);
 
     if (s->modulation == MODULATION_DSD)
-        avctx->execute2(avctx, dsd_channel, s->frame, NULL, avctx->channels);
+        avctx->execute2(avctx, dsd_channel, s->frame, NULL, avctx->ch_layout.nb_channels);
 
     ff_thread_report_progress(&s->curr_frame, INT_MAX, 0);
 
-    if ((ret = av_frame_ref(data, s->frame)) < 0)
+    if ((ret = av_frame_ref(rframe, s->frame)) < 0)
         return ret;
 
     *got_frame_ptr = 1;
@@ -1692,25 +1698,25 @@ static int wavpack_decode_frame(AVCodecContext *avctx, void *data,
 error:
     if (s->frame) {
         ff_thread_await_progress(&s->prev_frame, INT_MAX, 0);
-        ff_thread_release_buffer(avctx, &s->prev_frame);
+        ff_thread_release_ext_buffer(avctx, &s->prev_frame);
         ff_thread_report_progress(&s->curr_frame, INT_MAX, 0);
     }
 
     return ret;
 }
 
-AVCodec ff_wavpack_decoder = {
-    .name           = "wavpack",
-    .long_name      = NULL_IF_CONFIG_SMALL("WavPack"),
-    .type           = AVMEDIA_TYPE_AUDIO,
-    .id             = AV_CODEC_ID_WAVPACK,
+const FFCodec ff_wavpack_decoder = {
+    .p.name         = "wavpack",
+    .p.long_name    = NULL_IF_CONFIG_SMALL("WavPack"),
+    .p.type         = AVMEDIA_TYPE_AUDIO,
+    .p.id           = AV_CODEC_ID_WAVPACK,
     .priv_data_size = sizeof(WavpackContext),
     .init           = wavpack_decode_init,
     .close          = wavpack_decode_end,
-    .decode         = wavpack_decode_frame,
+    FF_CODEC_DECODE_CB(wavpack_decode_frame),
     .flush          = wavpack_decode_flush,
     .update_thread_context = ONLY_IF_THREADS_ENABLED(update_thread_context),
-    .capabilities   = AV_CODEC_CAP_DR1 | AV_CODEC_CAP_FRAME_THREADS |
+    .p.capabilities = AV_CODEC_CAP_DR1 | AV_CODEC_CAP_FRAME_THREADS |
                       AV_CODEC_CAP_SLICE_THREADS | AV_CODEC_CAP_CHANNEL_CONF,
     .caps_internal  = FF_CODEC_CAP_INIT_THREADSAFE | FF_CODEC_CAP_INIT_CLEANUP |
                       FF_CODEC_CAP_ALLOCATE_PROGRESS,
