@@ -18,47 +18,45 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
-#include "config.h"
-#include "libavutil/thread.h"
 #include "libavformat/internal.h"
 #include "avdevice.h"
 
 /* devices */
-extern AVInputFormat  ff_alsa_demuxer;
-extern AVOutputFormat ff_alsa_muxer;
-extern AVInputFormat  ff_android_camera_demuxer;
-extern AVOutputFormat ff_audiotoolbox_muxer;
-extern AVInputFormat  ff_avfoundation_demuxer;
-extern AVInputFormat  ff_bktr_demuxer;
-extern AVOutputFormat ff_caca_muxer;
-extern AVInputFormat  ff_decklink_demuxer;
-extern AVOutputFormat ff_decklink_muxer;
-extern AVInputFormat  ff_dshow_demuxer;
-extern AVInputFormat  ff_fbdev_demuxer;
-extern AVOutputFormat ff_fbdev_muxer;
-extern AVInputFormat  ff_gdigrab_demuxer;
-extern AVInputFormat  ff_iec61883_demuxer;
-extern AVInputFormat  ff_jack_demuxer;
-extern AVInputFormat  ff_kmsgrab_demuxer;
-extern AVInputFormat  ff_lavfi_demuxer;
-extern AVInputFormat  ff_openal_demuxer;
-extern AVOutputFormat ff_opengl_muxer;
-extern AVInputFormat  ff_oss_demuxer;
-extern AVOutputFormat ff_oss_muxer;
-extern AVInputFormat  ff_pulse_demuxer;
-extern AVOutputFormat ff_pulse_muxer;
-extern AVOutputFormat ff_sdl2_muxer;
-extern AVInputFormat  ff_sndio_demuxer;
-extern AVOutputFormat ff_sndio_muxer;
-extern AVInputFormat  ff_v4l2_demuxer;
-extern AVOutputFormat ff_v4l2_muxer;
-extern AVInputFormat  ff_vfwcap_demuxer;
-extern AVInputFormat  ff_xcbgrab_demuxer;
-extern AVOutputFormat ff_xv_muxer;
+extern const AVInputFormat  ff_alsa_demuxer;
+extern const AVOutputFormat ff_alsa_muxer;
+extern const AVInputFormat  ff_android_camera_demuxer;
+extern const AVOutputFormat ff_audiotoolbox_muxer;
+extern const AVInputFormat  ff_avfoundation_demuxer;
+extern const AVInputFormat  ff_bktr_demuxer;
+extern const AVOutputFormat ff_caca_muxer;
+extern const AVInputFormat  ff_decklink_demuxer;
+extern const AVOutputFormat ff_decklink_muxer;
+extern const AVInputFormat  ff_dshow_demuxer;
+extern const AVInputFormat  ff_fbdev_demuxer;
+extern const AVOutputFormat ff_fbdev_muxer;
+extern const AVInputFormat  ff_gdigrab_demuxer;
+extern const AVInputFormat  ff_iec61883_demuxer;
+extern const AVInputFormat  ff_jack_demuxer;
+extern const AVInputFormat  ff_kmsgrab_demuxer;
+extern const AVInputFormat  ff_lavfi_demuxer;
+extern const AVInputFormat  ff_openal_demuxer;
+extern const AVOutputFormat ff_opengl_muxer;
+extern const AVInputFormat  ff_oss_demuxer;
+extern const AVOutputFormat ff_oss_muxer;
+extern const AVInputFormat  ff_pulse_demuxer;
+extern const AVOutputFormat ff_pulse_muxer;
+extern const AVOutputFormat ff_sdl2_muxer;
+extern const AVInputFormat  ff_sndio_demuxer;
+extern const AVOutputFormat ff_sndio_muxer;
+extern const AVInputFormat  ff_v4l2_demuxer;
+extern const AVOutputFormat ff_v4l2_muxer;
+extern const AVInputFormat  ff_vfwcap_demuxer;
+extern const AVInputFormat  ff_xcbgrab_demuxer;
+extern const AVOutputFormat ff_xv_muxer;
 
 /* external libraries */
-extern AVInputFormat  ff_libcdio_demuxer;
-extern AVInputFormat  ff_libdc1394_demuxer;
+extern const AVInputFormat  ff_libcdio_demuxer;
+extern const AVInputFormat  ff_libdc1394_demuxer;
 
 #include "libavdevice/outdev_list.c"
 #include "libavdevice/indev_list.c"
@@ -68,7 +66,7 @@ void avdevice_register_all(void)
     avpriv_register_devices(outdev_list, indev_list);
 }
 
-static void *next_input(const AVInputFormat *prev, AVClassCategory c2)
+static const void *next_input(const AVInputFormat *prev, AVClassCategory c2)
 {
     const AVClass *pc;
     const AVClassCategory c1 = AV_CLASS_CATEGORY_DEVICE_INPUT;
@@ -91,10 +89,10 @@ static void *next_input(const AVInputFormat *prev, AVClassCategory c2)
             continue;
         category = pc->category;
     } while (category != c1 && category != c2);
-    return (AVInputFormat *)fmt;
+    return fmt;
 }
 
-static void *next_output(const AVOutputFormat *prev, AVClassCategory c2)
+static const void *next_output(const AVOutputFormat *prev, AVClassCategory c2)
 {
     const AVClass *pc;
     const AVClassCategory c1 = AV_CLASS_CATEGORY_DEVICE_OUTPUT;
@@ -117,25 +115,25 @@ static void *next_output(const AVOutputFormat *prev, AVClassCategory c2)
             continue;
         category = pc->category;
     } while (category != c1 && category != c2);
-    return (AVOutputFormat *)fmt;
+    return fmt;
 }
 
-AVInputFormat *av_input_audio_device_next(AVInputFormat  *d)
+const AVInputFormat *av_input_audio_device_next(const AVInputFormat  *d)
 {
     return next_input(d, AV_CLASS_CATEGORY_DEVICE_AUDIO_INPUT);
 }
 
-AVInputFormat *av_input_video_device_next(AVInputFormat  *d)
+const AVInputFormat *av_input_video_device_next(const AVInputFormat  *d)
 {
     return next_input(d, AV_CLASS_CATEGORY_DEVICE_VIDEO_INPUT);
 }
 
-AVOutputFormat *av_output_audio_device_next(AVOutputFormat *d)
+const AVOutputFormat *av_output_audio_device_next(const AVOutputFormat *d)
 {
     return next_output(d, AV_CLASS_CATEGORY_DEVICE_AUDIO_OUTPUT);
 }
 
-AVOutputFormat *av_output_video_device_next(AVOutputFormat *d)
+const AVOutputFormat *av_output_video_device_next(const AVOutputFormat *d)
 {
     return next_output(d, AV_CLASS_CATEGORY_DEVICE_VIDEO_OUTPUT);
 }
