@@ -25,13 +25,17 @@
 
 #include <float.h>
 
+#include "libavutil/tx.h"
 #include "libavutil/avstring.h"
+#include "libavutil/channel_layout.h"
 #include "libavutil/common.h"
 #include "libavutil/float_dsp.h"
+#include "libavutil/frame.h"
 #include "libavutil/intreadwrite.h"
+#include "libavutil/log.h"
 #include "libavutil/opt.h"
+#include "libavutil/rational.h"
 #include "libavutil/xga_font_data.h"
-#include "libavcodec/avfft.h"
 
 #include "audio.h"
 #include "avfilter.h"
@@ -39,199 +43,7 @@
 #include "formats.h"
 #include "internal.h"
 #include "af_afir.h"
-
-static void fcmul_add_c(float *sum, const float *t, const float *c, ptrdiff_t len)
-{
-    int n;
-
-    for (n = 0; n < len; n++) {
-        const float cre = c[2 * n    ];
-        const float cim = c[2 * n + 1];
-        const float tre = t[2 * n    ];
-        const float tim = t[2 * n + 1];
-
-        sum[2 * n    ] += tre * cre - tim * cim;
-        sum[2 * n + 1] += tre * cim + tim * cre;
-    }
-
-    sum[2 * n] += t[2 * n] * c[2 * n];
-}
-
-static void direct(const float *in, const FFTComplex *ir, int len, float *out)
-{
-    for (int n = 0; n < len; n++)
-        for (int m = 0; m <= n; m++)
-            out[n] += ir[m].re * in[n - m];
-}
-
-static int fir_quantum(AVFilterContext *ctx, AVFrame *out, int ch, int offset)
-{
-    AudioFIRContext *s = ctx->priv;
-    const float *in = (const float *)s->in->extended_data[ch] + offset;
-    float *block, *buf, *ptr = (float *)out->extended_data[ch] + offset;
-    const int nb_samples = FFMIN(s->min_part_size, out->nb_samples - offset);
-    int n, i, j;
-
-    for (int segment = 0; segment < s->nb_segments; segment++) {
-        AudioFIRSegment *seg = &s->seg[segment];
-        float *src = (float *)seg->input->extended_data[ch];
-        float *dst = (float *)seg->output->extended_data[ch];
-        float *sum = (float *)seg->sum->extended_data[ch];
-
-        if (s->min_part_size >= 8) {
-            s->fdsp->vector_fmul_scalar(src + seg->input_offset, in, s->dry_gain, FFALIGN(nb_samples, 4));
-            emms_c();
-        } else {
-            for (n = 0; n < nb_samples; n++)
-                src[seg->input_offset + n] = in[n] * s->dry_gain;
-        }
-
-        seg->output_offset[ch] += s->min_part_size;
-        if (seg->output_offset[ch] == seg->part_size) {
-            seg->output_offset[ch] = 0;
-        } else {
-            memmove(src, src + s->min_part_size, (seg->input_size - s->min_part_size) * sizeof(*src));
-
-            dst += seg->output_offset[ch];
-            for (n = 0; n < nb_samples; n++) {
-                ptr[n] += dst[n];
-            }
-            continue;
-        }
-
-        if (seg->part_size < 8) {
-            memset(dst, 0, sizeof(*dst) * seg->part_size * seg->nb_partitions);
-
-            j = seg->part_index[ch];
-
-            for (i = 0; i < seg->nb_partitions; i++) {
-                const int coffset = j * seg->coeff_size;
-                const FFTComplex *coeff = (const FFTComplex *)seg->coeff->extended_data[ch * !s->one2many] + coffset;
-
-                direct(src, coeff, nb_samples, dst);
-
-                if (j == 0)
-                    j = seg->nb_partitions;
-                j--;
-            }
-
-            seg->part_index[ch] = (seg->part_index[ch] + 1) % seg->nb_partitions;
-
-            memmove(src, src + s->min_part_size, (seg->input_size - s->min_part_size) * sizeof(*src));
-
-            for (n = 0; n < nb_samples; n++) {
-                ptr[n] += dst[n];
-            }
-            continue;
-        }
-
-        memset(sum, 0, sizeof(*sum) * seg->fft_length);
-        block = (float *)seg->block->extended_data[ch] + seg->part_index[ch] * seg->block_size;
-        memset(block + seg->part_size, 0, sizeof(*block) * (seg->fft_length - seg->part_size));
-
-        memcpy(block, src, sizeof(*src) * seg->part_size);
-
-        av_rdft_calc(seg->rdft[ch], block);
-        block[2 * seg->part_size] = block[1];
-        block[1] = 0;
-
-        j = seg->part_index[ch];
-
-        for (i = 0; i < seg->nb_partitions; i++) {
-            const int coffset = j * seg->coeff_size;
-            const float *block = (const float *)seg->block->extended_data[ch] + i * seg->block_size;
-            const FFTComplex *coeff = (const FFTComplex *)seg->coeff->extended_data[ch * !s->one2many] + coffset;
-
-            s->afirdsp.fcmul_add(sum, block, (const float *)coeff, seg->part_size);
-
-            if (j == 0)
-                j = seg->nb_partitions;
-            j--;
-        }
-
-        sum[1] = sum[2 * seg->part_size];
-        av_rdft_calc(seg->irdft[ch], sum);
-
-        buf = (float *)seg->buffer->extended_data[ch];
-        for (n = 0; n < seg->part_size; n++) {
-            buf[n] += sum[n];
-        }
-
-        memcpy(dst, buf, seg->part_size * sizeof(*dst));
-
-        buf = (float *)seg->buffer->extended_data[ch];
-        memcpy(buf, sum + seg->part_size, seg->part_size * sizeof(*buf));
-
-        seg->part_index[ch] = (seg->part_index[ch] + 1) % seg->nb_partitions;
-
-        memmove(src, src + s->min_part_size, (seg->input_size - s->min_part_size) * sizeof(*src));
-
-        for (n = 0; n < nb_samples; n++) {
-            ptr[n] += dst[n];
-        }
-    }
-
-    if (s->min_part_size >= 8) {
-        s->fdsp->vector_fmul_scalar(ptr, ptr, s->wet_gain, FFALIGN(nb_samples, 4));
-        emms_c();
-    } else {
-        for (n = 0; n < nb_samples; n++)
-            ptr[n] *= s->wet_gain;
-    }
-
-    return 0;
-}
-
-static int fir_channel(AVFilterContext *ctx, AVFrame *out, int ch)
-{
-    AudioFIRContext *s = ctx->priv;
-
-    for (int offset = 0; offset < out->nb_samples; offset += s->min_part_size) {
-        fir_quantum(ctx, out, ch, offset);
-    }
-
-    return 0;
-}
-
-static int fir_channels(AVFilterContext *ctx, void *arg, int jobnr, int nb_jobs)
-{
-    AVFrame *out = arg;
-    const int start = (out->channels * jobnr) / nb_jobs;
-    const int end = (out->channels * (jobnr+1)) / nb_jobs;
-
-    for (int ch = start; ch < end; ch++) {
-        fir_channel(ctx, out, ch);
-    }
-
-    return 0;
-}
-
-static int fir_frame(AudioFIRContext *s, AVFrame *in, AVFilterLink *outlink)
-{
-    AVFilterContext *ctx = outlink->src;
-    AVFrame *out = NULL;
-
-    out = ff_get_audio_buffer(outlink, in->nb_samples);
-    if (!out) {
-        av_frame_free(&in);
-        return AVERROR(ENOMEM);
-    }
-
-    if (s->pts == AV_NOPTS_VALUE)
-        s->pts = in->pts;
-    s->in = in;
-    ctx->internal->execute(ctx, fir_channels, out, NULL, FFMIN(outlink->channels,
-                                                               ff_filter_get_nb_threads(ctx)));
-
-    out->pts = s->pts;
-    if (s->pts != AV_NOPTS_VALUE)
-        s->pts += av_rescale_q(out->nb_samples, (AVRational){1, outlink->sample_rate}, outlink->time_base);
-
-    av_frame_free(&in);
-    s->in = NULL;
-
-    return ff_filter_frame(outlink, out);
-}
+#include "af_afirdsp.h"
 
 static void drawtext(AVFrame *pic, int x, int y, const char *txt, uint32_t color)
 {
@@ -282,93 +94,64 @@ static void draw_line(AVFrame *out, int x0, int y0, int x1, int y1, uint32_t col
     }
 }
 
-static void draw_response(AVFilterContext *ctx, AVFrame *out)
+#define DEPTH 32
+#include "afir_template.c"
+
+#undef DEPTH
+#define DEPTH 64
+#include "afir_template.c"
+
+static int fir_channel(AVFilterContext *ctx, AVFrame *out, int ch)
 {
     AudioFIRContext *s = ctx->priv;
-    float *mag, *phase, *delay, min = FLT_MAX, max = FLT_MIN;
-    float min_delay = FLT_MAX, max_delay = FLT_MIN;
-    int prev_ymag = -1, prev_yphase = -1, prev_ydelay = -1;
-    char text[32];
-    int channel, i, x;
 
-    memset(out->data[0], 0, s->h * out->linesize[0]);
-
-    phase = av_malloc_array(s->w, sizeof(*phase));
-    mag = av_malloc_array(s->w, sizeof(*mag));
-    delay = av_malloc_array(s->w, sizeof(*delay));
-    if (!mag || !phase || !delay)
-        goto end;
-
-    channel = av_clip(s->ir_channel, 0, s->ir[s->selir]->channels - 1);
-    for (i = 0; i < s->w; i++) {
-        const float *src = (const float *)s->ir[s->selir]->extended_data[channel];
-        double w = i * M_PI / (s->w - 1);
-        double div, real_num = 0., imag_num = 0., real = 0., imag = 0.;
-
-        for (x = 0; x < s->nb_taps; x++) {
-            real += cos(-x * w) * src[x];
-            imag += sin(-x * w) * src[x];
-            real_num += cos(-x * w) * src[x] * x;
-            imag_num += sin(-x * w) * src[x] * x;
+    for (int offset = 0; offset < out->nb_samples; offset += s->min_part_size) {
+        switch (s->format) {
+        case AV_SAMPLE_FMT_FLTP:
+            fir_quantum_float(ctx, out, ch, offset);
+            break;
+        case AV_SAMPLE_FMT_DBLP:
+            fir_quantum_double(ctx, out, ch, offset);
+            break;
         }
-
-        mag[i] = hypot(real, imag);
-        phase[i] = atan2(imag, real);
-        div = real * real + imag * imag;
-        delay[i] = (real_num * real + imag_num * imag) / div;
-        min = fminf(min, mag[i]);
-        max = fmaxf(max, mag[i]);
-        min_delay = fminf(min_delay, delay[i]);
-        max_delay = fmaxf(max_delay, delay[i]);
     }
 
-    for (i = 0; i < s->w; i++) {
-        int ymag = mag[i] / max * (s->h - 1);
-        int ydelay = (delay[i] - min_delay) / (max_delay - min_delay) * (s->h - 1);
-        int yphase = (0.5 * (1. + phase[i] / M_PI)) * (s->h - 1);
+    return 0;
+}
 
-        ymag = s->h - 1 - av_clip(ymag, 0, s->h - 1);
-        yphase = s->h - 1 - av_clip(yphase, 0, s->h - 1);
-        ydelay = s->h - 1 - av_clip(ydelay, 0, s->h - 1);
+static int fir_channels(AVFilterContext *ctx, void *arg, int jobnr, int nb_jobs)
+{
+    AVFrame *out = arg;
+    const int start = (out->ch_layout.nb_channels * jobnr) / nb_jobs;
+    const int end = (out->ch_layout.nb_channels * (jobnr+1)) / nb_jobs;
 
-        if (prev_ymag < 0)
-            prev_ymag = ymag;
-        if (prev_yphase < 0)
-            prev_yphase = yphase;
-        if (prev_ydelay < 0)
-            prev_ydelay = ydelay;
-
-        draw_line(out, i,   ymag, FFMAX(i - 1, 0),   prev_ymag, 0xFFFF00FF);
-        draw_line(out, i, yphase, FFMAX(i - 1, 0), prev_yphase, 0xFF00FF00);
-        draw_line(out, i, ydelay, FFMAX(i - 1, 0), prev_ydelay, 0xFF00FFFF);
-
-        prev_ymag   = ymag;
-        prev_yphase = yphase;
-        prev_ydelay = ydelay;
+    for (int ch = start; ch < end; ch++) {
+        fir_channel(ctx, out, ch);
     }
 
-    if (s->w > 400 && s->h > 100) {
-        drawtext(out, 2, 2, "Max Magnitude:", 0xDDDDDDDD);
-        snprintf(text, sizeof(text), "%.2f", max);
-        drawtext(out, 15 * 8 + 2, 2, text, 0xDDDDDDDD);
+    return 0;
+}
 
-        drawtext(out, 2, 12, "Min Magnitude:", 0xDDDDDDDD);
-        snprintf(text, sizeof(text), "%.2f", min);
-        drawtext(out, 15 * 8 + 2, 12, text, 0xDDDDDDDD);
+static int fir_frame(AudioFIRContext *s, AVFrame *in, AVFilterLink *outlink)
+{
+    AVFilterContext *ctx = outlink->src;
+    AVFrame *out = NULL;
 
-        drawtext(out, 2, 22, "Max Delay:", 0xDDDDDDDD);
-        snprintf(text, sizeof(text), "%.2f", max_delay);
-        drawtext(out, 11 * 8 + 2, 22, text, 0xDDDDDDDD);
-
-        drawtext(out, 2, 32, "Min Delay:", 0xDDDDDDDD);
-        snprintf(text, sizeof(text), "%.2f", min_delay);
-        drawtext(out, 11 * 8 + 2, 32, text, 0xDDDDDDDD);
+    out = ff_get_audio_buffer(outlink, in->nb_samples);
+    if (!out) {
+        av_frame_free(&in);
+        return AVERROR(ENOMEM);
     }
+    out->pts = in->pts;
 
-end:
-    av_free(delay);
-    av_free(phase);
-    av_free(mag);
+    s->in = in;
+    ff_filter_execute(ctx, fir_channels, out, NULL,
+                      FFMIN(outlink->ch_layout.nb_channels, ff_filter_get_nb_threads(ctx)));
+
+    av_frame_free(&in);
+    s->in = NULL;
+
+    return ff_filter_frame(outlink, out);
 }
 
 static int init_segment(AVFilterContext *ctx, AudioFIRSegment *seg,
@@ -376,9 +159,9 @@ static int init_segment(AVFilterContext *ctx, AudioFIRSegment *seg,
 {
     AudioFIRContext *s = ctx->priv;
 
-    seg->rdft  = av_calloc(ctx->inputs[0]->channels, sizeof(*seg->rdft));
-    seg->irdft = av_calloc(ctx->inputs[0]->channels, sizeof(*seg->irdft));
-    if (!seg->rdft || !seg->irdft)
+    seg->tx  = av_calloc(ctx->inputs[0]->ch_layout.nb_channels, sizeof(*seg->tx));
+    seg->itx = av_calloc(ctx->inputs[0]->ch_layout.nb_channels, sizeof(*seg->itx));
+    if (!seg->tx || !seg->itx)
         return AVERROR(ENOMEM);
 
     seg->fft_length    = part_size * 2 + 1;
@@ -389,25 +172,39 @@ static int init_segment(AVFilterContext *ctx, AudioFIRSegment *seg,
     seg->input_size    = offset + s->min_part_size;
     seg->input_offset  = offset;
 
-    seg->part_index    = av_calloc(ctx->inputs[0]->channels, sizeof(*seg->part_index));
-    seg->output_offset = av_calloc(ctx->inputs[0]->channels, sizeof(*seg->output_offset));
+    seg->part_index    = av_calloc(ctx->inputs[0]->ch_layout.nb_channels, sizeof(*seg->part_index));
+    seg->output_offset = av_calloc(ctx->inputs[0]->ch_layout.nb_channels, sizeof(*seg->output_offset));
     if (!seg->part_index || !seg->output_offset)
         return AVERROR(ENOMEM);
 
-    for (int ch = 0; ch < ctx->inputs[0]->channels && part_size >= 8; ch++) {
-        seg->rdft[ch]  = av_rdft_init(av_log2(2 * part_size), DFT_R2C);
-        seg->irdft[ch] = av_rdft_init(av_log2(2 * part_size), IDFT_C2R);
-        if (!seg->rdft[ch] || !seg->irdft[ch])
+    for (int ch = 0; ch < ctx->inputs[0]->ch_layout.nb_channels && part_size >= 8; ch++) {
+        double dscale = 1.0, idscale = 1.0 / part_size;
+        float fscale = 1.f, ifscale = 1.f / part_size;
+
+        switch (s->format) {
+        case AV_SAMPLE_FMT_FLTP:
+            av_tx_init(&seg->tx[ch],  &seg->tx_fn,  AV_TX_FLOAT_RDFT, 0, 2 * part_size, &fscale,  0);
+            av_tx_init(&seg->itx[ch], &seg->itx_fn, AV_TX_FLOAT_RDFT, 1, 2 * part_size, &ifscale, 0);
+            break;
+        case AV_SAMPLE_FMT_DBLP:
+            av_tx_init(&seg->tx[ch],  &seg->tx_fn,  AV_TX_DOUBLE_RDFT, 0, 2 * part_size, &dscale,  0);
+            av_tx_init(&seg->itx[ch], &seg->itx_fn, AV_TX_DOUBLE_RDFT, 1, 2 * part_size, &idscale, 0);
+            break;
+        }
+
+        if (!seg->tx[ch] || !seg->itx[ch])
             return AVERROR(ENOMEM);
     }
 
-    seg->sum    = ff_get_audio_buffer(ctx->inputs[0], seg->fft_length);
-    seg->block  = ff_get_audio_buffer(ctx->inputs[0], seg->nb_partitions * seg->block_size);
+    seg->sumin  = ff_get_audio_buffer(ctx->inputs[0], seg->fft_length);
+    seg->sumout = ff_get_audio_buffer(ctx->inputs[0], seg->fft_length);
+    seg->blockin  = ff_get_audio_buffer(ctx->inputs[0], seg->nb_partitions * seg->block_size);
+    seg->blockout = ff_get_audio_buffer(ctx->inputs[0], seg->nb_partitions * seg->block_size);
     seg->buffer = ff_get_audio_buffer(ctx->inputs[0], seg->part_size);
     seg->coeff  = ff_get_audio_buffer(ctx->inputs[1 + s->selir], seg->nb_partitions * seg->coeff_size * 2);
     seg->input  = ff_get_audio_buffer(ctx->inputs[0], seg->input_size);
     seg->output = ff_get_audio_buffer(ctx->inputs[0], seg->part_size);
-    if (!seg->buffer || !seg->sum || !seg->block || !seg->coeff || !seg->input || !seg->output)
+    if (!seg->buffer || !seg->sumin || !seg->sumout || !seg->blockin || !seg->blockout || !seg->coeff || !seg->input || !seg->output)
         return AVERROR(ENOMEM);
 
     return 0;
@@ -417,25 +214,27 @@ static void uninit_segment(AVFilterContext *ctx, AudioFIRSegment *seg)
 {
     AudioFIRContext *s = ctx->priv;
 
-    if (seg->rdft) {
+    if (seg->tx) {
         for (int ch = 0; ch < s->nb_channels; ch++) {
-            av_rdft_end(seg->rdft[ch]);
+            av_tx_uninit(&seg->tx[ch]);
         }
     }
-    av_freep(&seg->rdft);
+    av_freep(&seg->tx);
 
-    if (seg->irdft) {
+    if (seg->itx) {
         for (int ch = 0; ch < s->nb_channels; ch++) {
-            av_rdft_end(seg->irdft[ch]);
+            av_tx_uninit(&seg->itx[ch]);
         }
     }
-    av_freep(&seg->irdft);
+    av_freep(&seg->itx);
 
     av_freep(&seg->output_offset);
     av_freep(&seg->part_index);
 
-    av_frame_free(&seg->block);
-    av_frame_free(&seg->sum);
+    av_frame_free(&seg->blockin);
+    av_frame_free(&seg->blockout);
+    av_frame_free(&seg->sumin);
+    av_frame_free(&seg->sumout);
     av_frame_free(&seg->buffer);
     av_frame_free(&seg->coeff);
     av_frame_free(&seg->input);
@@ -446,8 +245,7 @@ static void uninit_segment(AVFilterContext *ctx, AudioFIRSegment *seg)
 static int convert_coeffs(AVFilterContext *ctx)
 {
     AudioFIRContext *s = ctx->priv;
-    int ret, i, ch, n, cur_nb_taps;
-    float power = 0;
+    int ret, i, cur_nb_taps;
 
     if (!s->nb_taps) {
         int part_size, max_part_size;
@@ -490,113 +288,42 @@ static int convert_coeffs(AVFilterContext *ctx)
             return AVERROR_BUG;
     }
 
-    if (s->response)
-        draw_response(ctx, s->video);
+    if (s->response) {
+        switch (s->format) {
+        case AV_SAMPLE_FMT_FLTP:
+            draw_response_float(ctx, s->video);
+            break;
+        case AV_SAMPLE_FMT_DBLP:
+            draw_response_double(ctx, s->video);
+            break;
+        }
+    }
 
     s->gain = 1;
     cur_nb_taps = s->ir[s->selir]->nb_samples;
 
-    switch (s->gtype) {
-    case -1:
-        /* nothing to do */
+    switch (s->format) {
+    case AV_SAMPLE_FMT_FLTP:
+        ret = get_power_float(ctx, s, cur_nb_taps);
         break;
-    case 0:
-        for (ch = 0; ch < ctx->inputs[1 + s->selir]->channels; ch++) {
-            float *time = (float *)s->ir[s->selir]->extended_data[!s->one2many * ch];
-
-            for (i = 0; i < cur_nb_taps; i++)
-                power += FFABS(time[i]);
-        }
-        s->gain = ctx->inputs[1 + s->selir]->channels / power;
+    case AV_SAMPLE_FMT_DBLP:
+        ret = get_power_double(ctx, s, cur_nb_taps);
         break;
-    case 1:
-        for (ch = 0; ch < ctx->inputs[1 + s->selir]->channels; ch++) {
-            float *time = (float *)s->ir[s->selir]->extended_data[!s->one2many * ch];
-
-            for (i = 0; i < cur_nb_taps; i++)
-                power += time[i];
-        }
-        s->gain = ctx->inputs[1 + s->selir]->channels / power;
-        break;
-    case 2:
-        for (ch = 0; ch < ctx->inputs[1 + s->selir]->channels; ch++) {
-            float *time = (float *)s->ir[s->selir]->extended_data[!s->one2many * ch];
-
-            for (i = 0; i < cur_nb_taps; i++)
-                power += time[i] * time[i];
-        }
-        s->gain = sqrtf(ch / power);
-        break;
-    default:
-        return AVERROR_BUG;
     }
 
-    s->gain = FFMIN(s->gain * s->ir_gain, 1.f);
-    av_log(ctx, AV_LOG_DEBUG, "power %f, gain %f\n", power, s->gain);
-    for (ch = 0; ch < ctx->inputs[1 + s->selir]->channels; ch++) {
-        float *time = (float *)s->ir[s->selir]->extended_data[!s->one2many * ch];
-
-        s->fdsp->vector_fmul_scalar(time, time, s->gain, FFALIGN(cur_nb_taps, 4));
-    }
+    if (ret < 0)
+        return ret;
 
     av_log(ctx, AV_LOG_DEBUG, "nb_taps: %d\n", cur_nb_taps);
     av_log(ctx, AV_LOG_DEBUG, "nb_segments: %d\n", s->nb_segments);
 
-    for (ch = 0; ch < ctx->inputs[1 + s->selir]->channels; ch++) {
-        float *time = (float *)s->ir[s->selir]->extended_data[!s->one2many * ch];
-        int toffset = 0;
-
-        for (i = FFMAX(1, s->length * s->nb_taps); i < s->nb_taps; i++)
-            time[i] = 0;
-
-        av_log(ctx, AV_LOG_DEBUG, "channel: %d\n", ch);
-
-        for (int segment = 0; segment < s->nb_segments; segment++) {
-            AudioFIRSegment *seg = &s->seg[segment];
-            float *block = (float *)seg->block->extended_data[ch];
-            FFTComplex *coeff = (FFTComplex *)seg->coeff->extended_data[ch];
-
-            av_log(ctx, AV_LOG_DEBUG, "segment: %d\n", segment);
-
-            for (i = 0; i < seg->nb_partitions; i++) {
-                const float scale = 1.f / seg->part_size;
-                const int coffset = i * seg->coeff_size;
-                const int remaining = s->nb_taps - toffset;
-                const int size = remaining >= seg->part_size ? seg->part_size : remaining;
-
-                if (size < 8) {
-                    for (n = 0; n < size; n++)
-                        coeff[coffset + n].re = time[toffset + n];
-
-                    toffset += size;
-                    continue;
-                }
-
-                memset(block, 0, sizeof(*block) * seg->fft_length);
-                memcpy(block, time + toffset, size * sizeof(*block));
-
-                av_rdft_calc(seg->rdft[0], block);
-
-                coeff[coffset].re = block[0] * scale;
-                coeff[coffset].im = 0;
-                for (n = 1; n < seg->part_size; n++) {
-                    coeff[coffset + n].re = block[2 * n] * scale;
-                    coeff[coffset + n].im = block[2 * n + 1] * scale;
-                }
-                coeff[coffset + seg->part_size].re = block[1] * scale;
-                coeff[coffset + seg->part_size].im = 0;
-
-                toffset += size;
-            }
-
-            av_log(ctx, AV_LOG_DEBUG, "nb_partitions: %d\n", seg->nb_partitions);
-            av_log(ctx, AV_LOG_DEBUG, "partition size: %d\n", seg->part_size);
-            av_log(ctx, AV_LOG_DEBUG, "block size: %d\n", seg->block_size);
-            av_log(ctx, AV_LOG_DEBUG, "fft_length: %d\n", seg->fft_length);
-            av_log(ctx, AV_LOG_DEBUG, "coeff_size: %d\n", seg->coeff_size);
-            av_log(ctx, AV_LOG_DEBUG, "input_size: %d\n", seg->input_size);
-            av_log(ctx, AV_LOG_DEBUG, "input_offset: %d\n", seg->input_offset);
-        }
+    switch (s->format) {
+    case AV_SAMPLE_FMT_FLTP:
+        convert_channels_float(ctx, s);
+        break;
+    case AV_SAMPLE_FMT_DBLP:
+        convert_channels_double(ctx, s);
+        break;
     }
 
     s->have_coeffs = 1;
@@ -604,7 +331,7 @@ static int convert_coeffs(AVFilterContext *ctx)
     return 0;
 }
 
-static int check_ir(AVFilterLink *link, AVFrame *frame)
+static int check_ir(AVFilterLink *link)
 {
     AVFilterContext *ctx = link->dst;
     AudioFIRContext *s = ctx->priv;
@@ -632,9 +359,7 @@ static int activate(AVFilterContext *ctx)
     if (s->response)
         FF_FILTER_FORWARD_STATUS_BACK_ALL(ctx->outputs[1], ctx);
     if (!s->eof_coeffs[s->selir]) {
-        AVFrame *ir = NULL;
-
-        ret = check_ir(ctx->inputs[1 + s->selir], ir);
+        ret = check_ir(ctx->inputs[1 + s->selir]);
         if (ret < 0)
             return ret;
 
@@ -712,11 +437,10 @@ static int activate(AVFilterContext *ctx)
 static int query_formats(AVFilterContext *ctx)
 {
     AudioFIRContext *s = ctx->priv;
-    AVFilterFormats *formats;
-    AVFilterChannelLayouts *layouts;
-    static const enum AVSampleFormat sample_fmts[] = {
-        AV_SAMPLE_FMT_FLTP,
-        AV_SAMPLE_FMT_NONE
+    static const enum AVSampleFormat sample_fmts[3][3] = {
+        { AV_SAMPLE_FMT_FLTP, AV_SAMPLE_FMT_DBLP, AV_SAMPLE_FMT_NONE },
+        { AV_SAMPLE_FMT_FLTP, AV_SAMPLE_FMT_NONE },
+        { AV_SAMPLE_FMT_DBLP, AV_SAMPLE_FMT_NONE },
     };
     static const enum AVPixelFormat pix_fmts[] = {
         AV_PIX_FMT_RGB0,
@@ -726,58 +450,60 @@ static int query_formats(AVFilterContext *ctx)
 
     if (s->response) {
         AVFilterLink *videolink = ctx->outputs[1];
-        formats = ff_make_format_list(pix_fmts);
-        if ((ret = ff_formats_ref(formats, &videolink->in_formats)) < 0)
+        AVFilterFormats *formats = ff_make_format_list(pix_fmts);
+        if ((ret = ff_formats_ref(formats, &videolink->incfg.formats)) < 0)
             return ret;
     }
 
-    layouts = ff_all_channel_counts();
-    if (!layouts)
-        return AVERROR(ENOMEM);
-
     if (s->ir_format) {
-        ret = ff_set_common_channel_layouts(ctx, layouts);
+        ret = ff_set_common_all_channel_counts(ctx);
         if (ret < 0)
             return ret;
     } else {
         AVFilterChannelLayouts *mono = NULL;
+        AVFilterChannelLayouts *layouts = ff_all_channel_counts();
 
-        if ((ret = ff_channel_layouts_ref(layouts, &ctx->inputs[0]->out_channel_layouts)) < 0)
+        if ((ret = ff_channel_layouts_ref(layouts, &ctx->inputs[0]->outcfg.channel_layouts)) < 0)
             return ret;
-        if ((ret = ff_channel_layouts_ref(layouts, &ctx->outputs[0]->in_channel_layouts)) < 0)
+        if ((ret = ff_channel_layouts_ref(layouts, &ctx->outputs[0]->incfg.channel_layouts)) < 0)
             return ret;
 
-        ret = ff_add_channel_layout(&mono, AV_CH_LAYOUT_MONO);
+        ret = ff_add_channel_layout(&mono, &(AVChannelLayout)AV_CHANNEL_LAYOUT_MONO);
         if (ret)
             return ret;
         for (int i = 1; i < ctx->nb_inputs; i++) {
-            if ((ret = ff_channel_layouts_ref(mono, &ctx->inputs[i]->out_channel_layouts)) < 0)
+            if ((ret = ff_channel_layouts_ref(mono, &ctx->inputs[i]->outcfg.channel_layouts)) < 0)
                 return ret;
         }
     }
 
-    formats = ff_make_format_list(sample_fmts);
-    if ((ret = ff_set_common_formats(ctx, formats)) < 0)
+    if ((ret = ff_set_common_formats_from_list(ctx, sample_fmts[s->precision])) < 0)
         return ret;
 
-    formats = ff_all_samplerates();
-    return ff_set_common_samplerates(ctx, formats);
+    return ff_set_common_all_samplerates(ctx);
 }
 
 static int config_output(AVFilterLink *outlink)
 {
     AVFilterContext *ctx = outlink->src;
     AudioFIRContext *s = ctx->priv;
+    int ret;
 
-    s->one2many = ctx->inputs[1 + s->selir]->channels == 1;
+    s->one2many = ctx->inputs[1 + s->selir]->ch_layout.nb_channels == 1;
     outlink->sample_rate = ctx->inputs[0]->sample_rate;
     outlink->time_base   = ctx->inputs[0]->time_base;
+#if FF_API_OLD_CHANNEL_LAYOUT
+FF_DISABLE_DEPRECATION_WARNINGS
     outlink->channel_layout = ctx->inputs[0]->channel_layout;
-    outlink->channels = ctx->inputs[0]->channels;
+FF_ENABLE_DEPRECATION_WARNINGS
+#endif
+    if ((ret = av_channel_layout_copy(&outlink->ch_layout, &ctx->inputs[0]->ch_layout)) < 0)
+        return ret;
+    outlink->ch_layout.nb_channels = ctx->inputs[0]->ch_layout.nb_channels;
 
-    s->nb_channels = outlink->channels;
-    s->nb_coef_channels = ctx->inputs[1 + s->selir]->channels;
-    s->pts = AV_NOPTS_VALUE;
+    s->nb_channels = outlink->ch_layout.nb_channels;
+    s->nb_coef_channels = ctx->inputs[1 + s->selir]->ch_layout.nb_channels;
+    s->format = outlink->format;
 
     return 0;
 }
@@ -796,11 +522,6 @@ static av_cold void uninit(AVFilterContext *ctx)
         av_frame_free(&s->ir[i]);
     }
 
-    for (int i = 0; i < ctx->nb_inputs; i++)
-        av_freep(&ctx->input_pads[i].name);
-
-    for (int i = 0; i < ctx->nb_outputs; i++)
-        av_freep(&ctx->output_pads[i].name);
     av_frame_free(&s->video);
 }
 
@@ -823,14 +544,6 @@ static int config_video(AVFilterLink *outlink)
     return 0;
 }
 
-void ff_afir_init(AudioFIRDSPContext *dsp)
-{
-    dsp->fcmul_add = fcmul_add_c;
-
-    if (ARCH_X86)
-        ff_afir_init_x86(dsp);
-}
-
 static av_cold int init(AVFilterContext *ctx)
 {
     AudioFIRContext *s = ctx->priv;
@@ -838,18 +551,13 @@ static av_cold int init(AVFilterContext *ctx)
     int ret;
 
     pad = (AVFilterPad) {
-        .name = av_strdup("main"),
+        .name = "main",
         .type = AVMEDIA_TYPE_AUDIO,
     };
 
-    if (!pad.name)
-        return AVERROR(ENOMEM);
-
-    ret = ff_insert_inpad(ctx, 0, &pad);
-    if (ret < 0) {
-        av_freep(&pad.name);
+    ret = ff_append_inpad(ctx, &pad);
+    if (ret < 0)
         return ret;
-    }
 
     for (int n = 0; n < s->nb_irs; n++) {
         pad = (AVFilterPad) {
@@ -860,42 +568,31 @@ static av_cold int init(AVFilterContext *ctx)
         if (!pad.name)
             return AVERROR(ENOMEM);
 
-        ret = ff_insert_inpad(ctx, n + 1, &pad);
-        if (ret < 0) {
-            av_freep(&pad.name);
+        ret = ff_append_inpad_free_name(ctx, &pad);
+        if (ret < 0)
             return ret;
-        }
     }
 
     pad = (AVFilterPad) {
-        .name          = av_strdup("default"),
+        .name          = "default",
         .type          = AVMEDIA_TYPE_AUDIO,
         .config_props  = config_output,
     };
 
-    if (!pad.name)
-        return AVERROR(ENOMEM);
-
-    ret = ff_insert_outpad(ctx, 0, &pad);
-    if (ret < 0) {
-        av_freep(&pad.name);
+    ret = ff_append_outpad(ctx, &pad);
+    if (ret < 0)
         return ret;
-    }
 
     if (s->response) {
         vpad = (AVFilterPad){
-            .name         = av_strdup("filter_response"),
+            .name         = "filter_response",
             .type         = AVMEDIA_TYPE_VIDEO,
             .config_props = config_video,
         };
-        if (!vpad.name)
-            return AVERROR(ENOMEM);
 
-        ret = ff_insert_outpad(ctx, 1, &vpad);
-        if (ret < 0) {
-            av_freep(&vpad.name);
+        ret = ff_append_outpad(ctx, &vpad);
+        if (ret < 0)
             return ret;
-        }
     }
 
     s->fdsp = avpriv_float_dsp_alloc(0);
@@ -957,17 +654,21 @@ static const AVOption afir_options[] = {
     { "maxp",   "set max partition size", OFFSET(maxp),  AV_OPT_TYPE_INT,   {.i64=8192}, 8, 32768, AF },
     { "nbirs",  "set number of input IRs",OFFSET(nb_irs),AV_OPT_TYPE_INT,   {.i64=1},    1,    32, AF },
     { "ir",     "select IR",              OFFSET(selir), AV_OPT_TYPE_INT,   {.i64=0},    0,    31, AFR },
+    { "precision", "set processing precision",    OFFSET(precision), AV_OPT_TYPE_INT,   {.i64=0}, 0, 2, AF, "precision" },
+    {  "auto", "set auto processing precision",                   0, AV_OPT_TYPE_CONST, {.i64=0}, 0, 0, AF, "precision" },
+    {  "float", "set single-floating point processing precision", 0, AV_OPT_TYPE_CONST, {.i64=1}, 0, 0, AF, "precision" },
+    {  "double","set double-floating point processing precision", 0, AV_OPT_TYPE_CONST, {.i64=2}, 0, 0, AF, "precision" },
     { NULL }
 };
 
 AVFILTER_DEFINE_CLASS(afir);
 
-AVFilter ff_af_afir = {
+const AVFilter ff_af_afir = {
     .name          = "afir",
     .description   = NULL_IF_CONFIG_SMALL("Apply Finite Impulse Response filter with supplied coefficients in additional stream(s)."),
     .priv_size     = sizeof(AudioFIRContext),
     .priv_class    = &afir_class,
-    .query_formats = query_formats,
+    FILTER_QUERY_FUNC(query_formats),
     .init          = init,
     .activate      = activate,
     .uninit        = uninit,
