@@ -23,6 +23,8 @@
  * Set timebase for the output link.
  */
 
+#include "config_components.h"
+
 #include <inttypes.h>
 #include <stdio.h>
 
@@ -167,7 +169,6 @@ static const AVFilterPad avfilter_vf_settb_inputs[] = {
         .name         = "default",
         .type         = AVMEDIA_TYPE_VIDEO,
     },
-    { NULL }
 };
 
 static const AVFilterPad avfilter_vf_settb_outputs[] = {
@@ -176,17 +177,17 @@ static const AVFilterPad avfilter_vf_settb_outputs[] = {
         .type         = AVMEDIA_TYPE_VIDEO,
         .config_props = config_output_props,
     },
-    { NULL }
 };
 
-AVFilter ff_vf_settb = {
+const AVFilter ff_vf_settb = {
     .name        = "settb",
     .description = NULL_IF_CONFIG_SMALL("Set timebase for the video output link."),
     .priv_size   = sizeof(SetTBContext),
     .priv_class  = &settb_class,
-    .inputs      = avfilter_vf_settb_inputs,
-    .outputs     = avfilter_vf_settb_outputs,
+    FILTER_INPUTS(avfilter_vf_settb_inputs),
+    FILTER_OUTPUTS(avfilter_vf_settb_outputs),
     .activate    = activate,
+    .flags       = AVFILTER_FLAG_METADATA_ONLY,
 };
 #endif /* CONFIG_SETTB_FILTER */
 
@@ -200,7 +201,6 @@ static const AVFilterPad avfilter_af_asettb_inputs[] = {
         .name         = "default",
         .type         = AVMEDIA_TYPE_AUDIO,
     },
-    { NULL }
 };
 
 static const AVFilterPad avfilter_af_asettb_outputs[] = {
@@ -209,16 +209,16 @@ static const AVFilterPad avfilter_af_asettb_outputs[] = {
         .type         = AVMEDIA_TYPE_AUDIO,
         .config_props = config_output_props,
     },
-    { NULL }
 };
 
-AVFilter ff_af_asettb = {
+const AVFilter ff_af_asettb = {
     .name        = "asettb",
     .description = NULL_IF_CONFIG_SMALL("Set timebase for the audio output link."),
     .priv_size   = sizeof(SetTBContext),
-    .inputs      = avfilter_af_asettb_inputs,
-    .outputs     = avfilter_af_asettb_outputs,
+    FILTER_INPUTS(avfilter_af_asettb_inputs),
+    FILTER_OUTPUTS(avfilter_af_asettb_outputs),
     .priv_class  = &asettb_class,
     .activate    = activate,
+    .flags       = AVFILTER_FLAG_METADATA_ONLY,
 };
 #endif /* CONFIG_ASETTB_FILTER */
