@@ -32,6 +32,7 @@
 #include "libavutil/imgutils.h"
 #include "libavutil/motion_vector.h"
 #include "libavutil/opt.h"
+#include "libavutil/video_enc_params.h"
 #include "avfilter.h"
 #include "qp_table.h"
 #include "internal.h"
@@ -52,6 +53,7 @@ typedef struct CodecViewContext {
     unsigned mv_type;
     int hsub, vsub;
     int qp;
+    int block;
 } CodecViewContext;
 
 #define OFFSET(x) offsetof(CodecViewContext, x)
@@ -73,22 +75,12 @@ static const AVOption codecview_options[] = {
         CONST("if", "I-frames", FRAME_TYPE_I, "frame_type"),
         CONST("pf", "P-frames", FRAME_TYPE_P, "frame_type"),
         CONST("bf", "B-frames", FRAME_TYPE_B, "frame_type"),
+    { "block",      "set block partitioning structure to visualize", OFFSET(block), AV_OPT_TYPE_BOOL, {.i64=0}, 0, 1, FLAGS },
     { NULL }
 };
 
 AVFILTER_DEFINE_CLASS(codecview);
 
-static int query_formats(AVFilterContext *ctx)
-{
-    // TODO: we can probably add way more pixel formats without any other
-    // changes; anything with 8-bit luma in first plane should be working
-    static const enum AVPixelFormat pix_fmts[] = {AV_PIX_FMT_YUV420P, AV_PIX_FMT_NONE};
-    AVFilterFormats *fmts_list = ff_make_format_list(pix_fmts);
-    if (!fmts_list)
-        return AVERROR(ENOMEM);
-    return ff_set_common_formats(ctx, fmts_list);
-}
-
 static int clip_line(int *sx, int *sy, int *ex, int *ey, int maxx)
 {
     if(*sx > *ex)
@@ -213,6 +205,18 @@ static void draw_arrow(uint8_t *buf, int sx, int sy, int ex,
     draw_line(buf, sx, sy, ex, ey, w, h, stride, color);
 }
 
+static void draw_block_rectangle(uint8_t *buf, int sx, int sy, int w, int h, int stride, int color)
+{
+    for (int x = sx; x < sx + w; x++)
+        buf[x] = color;
+
+    for (int y = sy; y < sy + h; y++) {
+        buf[sx] = color;
+        buf[sx + w - 1] = color;
+        buf += stride;
+    }
+}
+
 static int filter_frame(AVFilterLink *inlink, AVFrame *frame)
 {
     AVFilterContext *ctx = inlink->dst;
@@ -220,7 +224,8 @@ static int filter_frame(AVFilterLink *inlink, AVFrame *frame)
     AVFilterLink *outlink = ctx->outputs[0];
 
     if (s->qp) {
-        int qstride, qp_type, ret;
+        enum AVVideoEncParamsType qp_type;
+        int qstride, ret;
         int8_t *qp_table;
 
         ret = ff_qp_table_extract(frame, &qp_table, &qstride, NULL, &qp_type);
@@ -250,6 +255,23 @@ static int filter_frame(AVFilterLink *inlink, AVFrame *frame)
         av_freep(&qp_table);
     }
 
+    if (s->block) {
+        AVFrameSideData *sd = av_frame_get_side_data(frame, AV_FRAME_DATA_VIDEO_ENC_PARAMS);
+        if (sd) {
+            AVVideoEncParams *par = (AVVideoEncParams*)sd->data;
+            const int stride = frame->linesize[0];
+
+            if (par->nb_blocks) {
+                for (int block_idx = 0; block_idx < par->nb_blocks; block_idx++) {
+                    AVVideoBlockParams *b = av_video_enc_params_block(par, block_idx);
+                    uint8_t *buf = frame->data[0] + b->src_y * stride;
+
+                    draw_block_rectangle(buf, b->src_x, b->src_y, b->w, b->h, stride, 100);
+                }
+            }
+        }
+    }
+
     if (s->mv || s->mv_type) {
         AVFrameSideData *sd = av_frame_get_side_data(frame, AV_FRAME_DATA_MOTION_VECTORS);
         if (sd) {
@@ -303,11 +325,10 @@ static const AVFilterPad codecview_inputs[] = {
     {
         .name           = "default",
         .type           = AVMEDIA_TYPE_VIDEO,
+        .flags          = AVFILTERPAD_FLAG_NEEDS_WRITABLE,
         .filter_frame   = filter_frame,
         .config_props   = config_input,
-        .needs_writable = 1,
     },
-    { NULL }
 };
 
 static const AVFilterPad codecview_outputs[] = {
@@ -315,16 +336,17 @@ static const AVFilterPad codecview_outputs[] = {
         .name = "default",
         .type = AVMEDIA_TYPE_VIDEO,
     },
-    { NULL }
 };
 
-AVFilter ff_vf_codecview = {
+const AVFilter ff_vf_codecview = {
     .name          = "codecview",
     .description   = NULL_IF_CONFIG_SMALL("Visualize information about some codecs."),
     .priv_size     = sizeof(CodecViewContext),
-    .query_formats = query_formats,
-    .inputs        = codecview_inputs,
-    .outputs       = codecview_outputs,
+    FILTER_INPUTS(codecview_inputs),
+    FILTER_OUTPUTS(codecview_outputs),
+    // TODO: we can probably add way more pixel formats without any other
+    // changes; anything with 8-bit luma in first plane should be working
+    FILTER_SINGLE_PIXFMT(AV_PIX_FMT_YUV420P),
     .priv_class    = &codecview_class,
     .flags         = AVFILTER_FLAG_SUPPORT_TIMELINE_GENERIC,
 };
