@@ -416,7 +416,7 @@ static int dump_curves(const char *fname, uint16_t *graph[NB_COMP + 1],
     AVBPrint buf;
     const double scale = 1. / (lut_size - 1);
     static const char * const colors[] = { "red", "green", "blue", "#404040", };
-    FILE *f = av_fopen_utf8(fname, "w");
+    FILE *f = avpriv_fopen_utf8(fname, "w");
 
     av_assert0(FF_ARRAY_ELEMS(colors) == NB_COMP + 1);
 
@@ -513,30 +513,6 @@ static av_cold int curves_init(AVFilterContext *ctx)
     return 0;
 }
 
-static int query_formats(AVFilterContext *ctx)
-{
-    static const enum AVPixelFormat pix_fmts[] = {
-        AV_PIX_FMT_RGB24,  AV_PIX_FMT_BGR24,
-        AV_PIX_FMT_RGBA,   AV_PIX_FMT_BGRA,
-        AV_PIX_FMT_ARGB,   AV_PIX_FMT_ABGR,
-        AV_PIX_FMT_0RGB,   AV_PIX_FMT_0BGR,
-        AV_PIX_FMT_RGB0,   AV_PIX_FMT_BGR0,
-        AV_PIX_FMT_RGB48,  AV_PIX_FMT_BGR48,
-        AV_PIX_FMT_RGBA64, AV_PIX_FMT_BGRA64,
-        AV_PIX_FMT_GBRP,   AV_PIX_FMT_GBRAP,
-        AV_PIX_FMT_GBRP9,
-        AV_PIX_FMT_GBRP10, AV_PIX_FMT_GBRAP10,
-        AV_PIX_FMT_GBRP12, AV_PIX_FMT_GBRAP12,
-        AV_PIX_FMT_GBRP14,
-        AV_PIX_FMT_GBRP16, AV_PIX_FMT_GBRAP16,
-        AV_PIX_FMT_NONE
-    };
-    AVFilterFormats *fmts_list = ff_make_format_list(pix_fmts);
-    if (!fmts_list)
-        return AVERROR(ENOMEM);
-    return ff_set_common_formats(ctx, fmts_list);
-}
-
 static int filter_slice_packed(AVFilterContext *ctx, void *arg, int jobnr, int nb_jobs)
 {
     int x, y;
@@ -669,7 +645,7 @@ static int config_input(AVFilterLink *inlink)
 
     for (i = 0; i < NB_COMP + 1; i++) {
         if (!curves->graph[i])
-            curves->graph[i] = av_mallocz_array(curves->lut_size, sizeof(*curves->graph[0]));
+            curves->graph[i] = av_calloc(curves->lut_size, sizeof(*curves->graph[0]));
         if (!curves->graph[i])
             return AVERROR(ENOMEM);
         ret = parse_points_str(ctx, comp_points + i, curves->comp_points_str[i], curves->lut_size);
@@ -742,7 +718,8 @@ static int filter_frame(AVFilterLink *inlink, AVFrame *in)
 
     td.in  = in;
     td.out = out;
-    ctx->internal->execute(ctx, curves->filter_slice, &td, NULL, FFMIN(outlink->h, ff_filter_get_nb_threads(ctx)));
+    ff_filter_execute(ctx, curves->filter_slice, &td, NULL,
+                      FFMIN(outlink->h, ff_filter_get_nb_threads(ctx)));
 
     if (out != in)
         av_frame_free(&in);
@@ -802,7 +779,6 @@ static const AVFilterPad curves_inputs[] = {
         .filter_frame = filter_frame,
         .config_props = config_input,
     },
-    { NULL }
 };
 
 static const AVFilterPad curves_outputs[] = {
@@ -810,18 +786,29 @@ static const AVFilterPad curves_outputs[] = {
         .name = "default",
         .type = AVMEDIA_TYPE_VIDEO,
     },
-    { NULL }
 };
 
-AVFilter ff_vf_curves = {
+const AVFilter ff_vf_curves = {
     .name          = "curves",
     .description   = NULL_IF_CONFIG_SMALL("Adjust components curves."),
     .priv_size     = sizeof(CurvesContext),
     .init          = curves_init,
     .uninit        = curves_uninit,
-    .query_formats = query_formats,
-    .inputs        = curves_inputs,
-    .outputs       = curves_outputs,
+    FILTER_INPUTS(curves_inputs),
+    FILTER_OUTPUTS(curves_outputs),
+    FILTER_PIXFMTS(AV_PIX_FMT_RGB24,  AV_PIX_FMT_BGR24,
+                   AV_PIX_FMT_RGBA,   AV_PIX_FMT_BGRA,
+                   AV_PIX_FMT_ARGB,   AV_PIX_FMT_ABGR,
+                   AV_PIX_FMT_0RGB,   AV_PIX_FMT_0BGR,
+                   AV_PIX_FMT_RGB0,   AV_PIX_FMT_BGR0,
+                   AV_PIX_FMT_RGB48,  AV_PIX_FMT_BGR48,
+                   AV_PIX_FMT_RGBA64, AV_PIX_FMT_BGRA64,
+                   AV_PIX_FMT_GBRP,   AV_PIX_FMT_GBRAP,
+                   AV_PIX_FMT_GBRP9,
+                   AV_PIX_FMT_GBRP10, AV_PIX_FMT_GBRAP10,
+                   AV_PIX_FMT_GBRP12, AV_PIX_FMT_GBRAP12,
+                   AV_PIX_FMT_GBRP14,
+                   AV_PIX_FMT_GBRP16, AV_PIX_FMT_GBRAP16),
     .priv_class    = &curves_class,
     .flags         = AVFILTER_FLAG_SUPPORT_TIMELINE_GENERIC | AVFILTER_FLAG_SLICE_THREADS,
     .process_command = process_command,
