@@ -48,26 +48,13 @@ static const AVOption weave_options[] = {
     { NULL }
 };
 
-AVFILTER_DEFINE_CLASS(weave);
+AVFILTER_DEFINE_CLASS_EXT(weave, "(double)weave", weave_options);
 
 static int query_formats(AVFilterContext *ctx)
 {
-    AVFilterFormats *formats = NULL;
-    int ret;
+    int reject_flags = AV_PIX_FMT_FLAG_PAL | AV_PIX_FMT_FLAG_HWACCEL;
 
-    for (int fmt = 0; av_pix_fmt_desc_get(fmt); fmt++) {
-        const AVPixFmtDescriptor *desc = av_pix_fmt_desc_get(fmt);
-
-        if (!(desc->flags & AV_PIX_FMT_FLAG_PAL) &&
-            !(desc->flags & AV_PIX_FMT_FLAG_HWACCEL)) {
-            if ((ret = ff_add_format(&formats, fmt)) < 0) {
-                ff_formats_unref(&formats);
-                return ret;
-            }
-        }
-    }
-
-    return ff_set_common_formats(ctx, formats);
+    return ff_set_common_formats(ctx, ff_formats_pixdesc_filter(0, reject_flags));
 }
 
 static int config_props_output(AVFilterLink *outlink)
@@ -101,15 +88,52 @@ static int config_props_output(AVFilterLink *outlink)
     return 0;
 }
 
+typedef struct ThreadData {
+    AVFrame *in, *out;
+} ThreadData;
+
+static int weave_slice(AVFilterContext *ctx, void *arg, int jobnr, int nb_jobs)
+{
+    AVFilterLink *inlink = ctx->inputs[0];
+    WeaveContext *s = ctx->priv;
+    ThreadData *td = arg;
+    AVFrame *in = td->in;
+    AVFrame *out = td->out;
+
+    const int weave = (s->double_weave && !(inlink->frame_count_out & 1));
+    const int field1 = weave ? s->first_field : (!s->first_field);
+    const int field2 = weave ? (!s->first_field) : s->first_field;
+
+    for (int i = 0; i < s->nb_planes; i++) {
+        const int height = s->planeheight[i];
+        const int start = (height * jobnr) / nb_jobs;
+        const int end = (height * (jobnr+1)) / nb_jobs;
+        const int compensation = 2*end > s->outheight[i];
+
+        av_image_copy_plane(out->data[i] + out->linesize[i] * field1 +
+                            out->linesize[i] * start * 2,
+                            out->linesize[i] * 2,
+                            in->data[i] + start * in->linesize[i],
+                            in->linesize[i],
+                            s->linesize[i], end - start - compensation * field1);
+        av_image_copy_plane(out->data[i] + out->linesize[i] * field2 +
+                            out->linesize[i] * start * 2,
+                            out->linesize[i] * 2,
+                            s->prev->data[i] + start * s->prev->linesize[i],
+                            s->prev->linesize[i],
+                            s->linesize[i], end - start - compensation * field2);
+    }
+
+    return 0;
+}
+
 static int filter_frame(AVFilterLink *inlink, AVFrame *in)
 {
     AVFilterContext *ctx = inlink->dst;
     WeaveContext *s = ctx->priv;
     AVFilterLink *outlink = ctx->outputs[0];
+    ThreadData td;
     AVFrame *out;
-    int i;
-    int weave;
-    int field1, field2;
 
     if (!s->prev) {
         s->prev = in;
@@ -124,20 +148,9 @@ static int filter_frame(AVFilterLink *inlink, AVFrame *in)
     }
     av_frame_copy_props(out, in);
 
-    weave = (s->double_weave && !(inlink->frame_count_out & 1));
-    field1 = weave ? s->first_field : (!s->first_field);
-    field2 = weave ? (!s->first_field) : s->first_field;
-    for (i = 0; i < s->nb_planes; i++) {
-        const int compensation = 2*s->planeheight[i] > s->outheight[i];
-        av_image_copy_plane(out->data[i] + out->linesize[i] * field1,
-                            out->linesize[i] * 2,
-                            in->data[i], in->linesize[i],
-                            s->linesize[i], s->planeheight[i] - compensation * field1);
-        av_image_copy_plane(out->data[i] + out->linesize[i] * field2,
-                            out->linesize[i] * 2,
-                            s->prev->data[i], s->prev->linesize[i],
-                            s->linesize[i], s->planeheight[i] - compensation * field2);
-    }
+    td.out = out, td.in = in;
+    ff_filter_execute(ctx, weave_slice, &td, NULL,
+                      FFMIN(s->planeheight[1], ff_filter_get_nb_threads(ctx)));
 
     out->pts = s->double_weave ? s->prev->pts : in->pts / 2;
     out->interlaced_frame = 1;
@@ -164,7 +177,6 @@ static const AVFilterPad weave_inputs[] = {
         .type             = AVMEDIA_TYPE_VIDEO,
         .filter_frame     = filter_frame,
     },
-    { NULL }
 };
 
 static const AVFilterPad weave_outputs[] = {
@@ -173,18 +185,18 @@ static const AVFilterPad weave_outputs[] = {
         .type          = AVMEDIA_TYPE_VIDEO,
         .config_props  = config_props_output,
     },
-    { NULL }
 };
 
-AVFilter ff_vf_weave = {
+const AVFilter ff_vf_weave = {
     .name          = "weave",
     .description   = NULL_IF_CONFIG_SMALL("Weave input video fields into frames."),
     .priv_size     = sizeof(WeaveContext),
     .priv_class    = &weave_class,
-    .query_formats = query_formats,
     .uninit        = uninit,
-    .inputs        = weave_inputs,
-    .outputs       = weave_outputs,
+    FILTER_INPUTS(weave_inputs),
+    FILTER_OUTPUTS(weave_outputs),
+    FILTER_QUERY_FUNC(query_formats),
+    .flags         = AVFILTER_FLAG_SLICE_THREADS,
 };
 
 static av_cold int init(AVFilterContext *ctx)
@@ -197,17 +209,15 @@ static av_cold int init(AVFilterContext *ctx)
     return 0;
 }
 
-#define doubleweave_options weave_options
-AVFILTER_DEFINE_CLASS(doubleweave);
-
-AVFilter ff_vf_doubleweave = {
+const AVFilter ff_vf_doubleweave = {
     .name          = "doubleweave",
     .description   = NULL_IF_CONFIG_SMALL("Weave input video fields into double number of frames."),
+    .priv_class    = &weave_class,
     .priv_size     = sizeof(WeaveContext),
-    .priv_class    = &doubleweave_class,
-    .query_formats = query_formats,
     .init          = init,
     .uninit        = uninit,
-    .inputs        = weave_inputs,
-    .outputs       = weave_outputs,
+    FILTER_INPUTS(weave_inputs),
+    FILTER_OUTPUTS(weave_outputs),
+    FILTER_QUERY_FUNC(query_formats),
+    .flags         = AVFILTER_FLAG_SLICE_THREADS,
 };
