@@ -23,19 +23,16 @@
 #include <string.h>
 #include <stdio.h>
 
-#include "libavutil/avassert.h"
 #include "libavutil/buffer.h"
+#include "libavutil/cpu.h"
 #include "libavutil/hwcontext.h"
 #include "libavutil/imgutils.h"
-#include "libavutil/mem.h"
 
 #include "avfilter.h"
+#include "framepool.h"
 #include "internal.h"
 #include "video.h"
 
-#define BUFFER_ALIGN 32
-
-
 AVFrame *ff_null_get_video_buffer(AVFilterLink *link, int w, int h)
 {
     return ff_get_video_buffer(link->dst->outputs[0], w, h);
@@ -52,7 +49,7 @@ AVFrame *ff_default_get_video_buffer2(AVFilterLink *link, int w, int h, int alig
     if (link->hw_frames_ctx &&
         ((AVHWFramesContext*)link->hw_frames_ctx->data)->format == link->format) {
         int ret;
-        AVFrame *frame = av_frame_alloc();
+        frame = av_frame_alloc();
 
         if (!frame)
             return NULL;
@@ -66,7 +63,7 @@ AVFrame *ff_default_get_video_buffer2(AVFilterLink *link, int w, int h, int alig
 
     if (!link->frame_pool) {
         link->frame_pool = ff_frame_pool_video_init(av_buffer_allocz, w, h,
-                                                    link->format, BUFFER_ALIGN);
+                                                    link->format, align);
         if (!link->frame_pool)
             return NULL;
     } else {
@@ -77,11 +74,11 @@ AVFrame *ff_default_get_video_buffer2(AVFilterLink *link, int w, int h, int alig
         }
 
         if (pool_width != w || pool_height != h ||
-            pool_format != link->format || pool_align != BUFFER_ALIGN) {
+            pool_format != link->format || pool_align != align) {
 
             ff_frame_pool_uninit((FFFramePool **)&link->frame_pool);
             link->frame_pool = ff_frame_pool_video_init(av_buffer_allocz, w, h,
-                                                        link->format, BUFFER_ALIGN);
+                                                        link->format, align);
             if (!link->frame_pool)
                 return NULL;
         }
@@ -107,8 +104,8 @@ AVFrame *ff_get_video_buffer(AVFilterLink *link, int w, int h)
 
     FF_TPRINTF_START(NULL, get_video_buffer); ff_tlog_link(NULL, link, 0);
 
-    if (link->dstpad->get_video_buffer)
-        ret = link->dstpad->get_video_buffer(link, w, h);
+    if (link->dstpad->get_buffer.video)
+        ret = link->dstpad->get_buffer.video(link, w, h);
 
     if (!ret)
         ret = ff_default_get_video_buffer(link, w, h);
