@@ -19,6 +19,8 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
+#include "config_components.h"
+
 #include "libavutil/avassert.h"
 #include "libavutil/crc.h"
 #include "libavcodec/ac3_parser.h"
@@ -102,8 +104,7 @@ static int ac3_probe(const AVProbeData *p)
     return ac3_eac3_probe(p, AV_CODEC_ID_AC3);
 }
 
-FF_RAW_DEMUXER_CLASS(ac3)
-AVInputFormat ff_ac3_demuxer = {
+const AVInputFormat ff_ac3_demuxer = {
     .name           = "ac3",
     .long_name      = NULL_IF_CONFIG_SMALL("raw AC-3"),
     .read_probe     = ac3_probe,
@@ -113,7 +114,7 @@ AVInputFormat ff_ac3_demuxer = {
     .extensions = "ac3",
     .raw_codec_id   = AV_CODEC_ID_AC3,
     .priv_data_size = sizeof(FFRawDemuxerContext),
-    .priv_class     = &ac3_demuxer_class,
+    .priv_class     = &ff_raw_demuxer_class,
 };
 #endif
 
@@ -123,8 +124,7 @@ static int eac3_probe(const AVProbeData *p)
     return ac3_eac3_probe(p, AV_CODEC_ID_EAC3);
 }
 
-FF_RAW_DEMUXER_CLASS(eac3)
-AVInputFormat ff_eac3_demuxer = {
+const AVInputFormat ff_eac3_demuxer = {
     .name           = "eac3",
     .long_name      = NULL_IF_CONFIG_SMALL("raw E-AC-3"),
     .read_probe     = eac3_probe,
@@ -134,6 +134,6 @@ AVInputFormat ff_eac3_demuxer = {
     .extensions     = "eac3",
     .raw_codec_id   = AV_CODEC_ID_EAC3,
     .priv_data_size = sizeof(FFRawDemuxerContext),
-    .priv_class     = &eac3_demuxer_class,
+    .priv_class     = &ff_raw_demuxer_class,
 };
 #endif
