@@ -20,15 +20,24 @@
  */
 
 #include "config.h"
+#include "config_components.h"
 
 #include "libavutil/common.h"
-#include "libavutil/fifo.h"
 #include "libavutil/opt.h"
 #include "libavcodec/av1_parse.h"
+#include "libavcodec/bsf.h"
 #include "avformat.h"
 #include "avio_internal.h"
 #include "internal.h"
 
+typedef struct AV1DemuxContext {
+    const AVClass *class;
+    AVBSFContext *bsf;
+    AVRational framerate;
+    uint32_t temporal_unit_size;
+    uint32_t frame_unit_size;
+} AV1DemuxContext;
+
 //return < 0 if we need more data
 static int get_score(int type, int *seq)
 {
@@ -48,14 +57,16 @@ static int get_score(int type, int *seq)
     return 0;
 }
 
-static int read_header(AVFormatContext *s, const AVRational *framerate, AVBSFContext **bsf, void *logctx)
+static int av1_read_header(AVFormatContext *s)
 {
+    AV1DemuxContext *const c = s->priv_data;
     const AVBitStreamFilter *filter = av_bsf_get_by_name("av1_frame_merge");
     AVStream *st;
+    FFStream *sti;
     int ret;
 
     if (!filter) {
-        av_log(logctx, AV_LOG_ERROR, "av1_frame_merge bitstream filter "
+        av_log(s, AV_LOG_ERROR, "av1_frame_merge bitstream filter "
                "not found. This is a bug, please report it.\n");
         return AVERROR_BUG;
     }
@@ -63,43 +74,55 @@ static int read_header(AVFormatContext *s, const AVRational *framerate, AVBSFCon
     st = avformat_new_stream(s, NULL);
     if (!st)
         return AVERROR(ENOMEM);
+    sti = ffstream(st);
 
     st->codecpar->codec_type = AVMEDIA_TYPE_VIDEO;
     st->codecpar->codec_id = AV_CODEC_ID_AV1;
-    st->need_parsing = AVSTREAM_PARSE_HEADERS;
+    sti->need_parsing = AVSTREAM_PARSE_HEADERS;
 
-    st->internal->avctx->framerate = *framerate;
+    sti->avctx->framerate = c->framerate;
     // taken from rawvideo demuxers
     avpriv_set_pts_info(st, 64, 1, 1200000);
 
-    ret = av_bsf_alloc(filter, bsf);
+    ret = av_bsf_alloc(filter, &c->bsf);
     if (ret < 0)
         return ret;
 
-    ret = avcodec_parameters_copy((*bsf)->par_in, st->codecpar);
-    if (ret < 0) {
-        av_bsf_free(bsf);
-        return ret;
-    }
-
-    ret = av_bsf_init(*bsf);
+    ret = avcodec_parameters_copy(c->bsf->par_in, st->codecpar);
     if (ret < 0)
-        av_bsf_free(bsf);
+        return ret;
 
-    return ret;
+    ret = av_bsf_init(c->bsf);
+    if (ret < 0)
+        return ret;
 
+    return 0;
+}
+
+static int av1_read_close(AVFormatContext *s)
+{
+    AV1DemuxContext *const c = s->priv_data;
+
+    av_bsf_free(&c->bsf);
+    return 0;
 }
 
 #define DEC AV_OPT_FLAG_DECODING_PARAM
+#define OFFSET(x) offsetof(AV1DemuxContext, x)
+static const AVOption av1_options[] = {
+    { "framerate", "", OFFSET(framerate), AV_OPT_TYPE_VIDEO_RATE, {.str = "25"}, 0, INT_MAX, DEC},
+    { NULL },
+};
+#undef OFFSET
+
+static const AVClass av1_demuxer_class = {
+    .class_name = "AV1 Annex B/low overhead OBU demuxer",
+    .item_name  = av_default_item_name,
+    .option     = av1_options,
+    .version    = LIBAVUTIL_VERSION_INT,
+};
 
 #if CONFIG_AV1_DEMUXER
-typedef struct AnnexBContext {
-    const AVClass *class;
-    AVBSFContext *bsf;
-    uint32_t temporal_unit_size;
-    uint32_t frame_unit_size;
-    AVRational framerate;
-} AnnexBContext;
 
 static int leb(AVIOContext *pb, uint32_t *len) {
     int more, i = 0;
@@ -137,32 +160,33 @@ static int read_obu(const uint8_t *buf, int size, int64_t *obu_size, int *type)
 
 static int annexb_probe(const AVProbeData *p)
 {
-    AVIOContext pb;
+    FFIOContext ctx;
+    AVIOContext *const pb = &ctx.pub;
     int64_t obu_size;
     uint32_t temporal_unit_size, frame_unit_size, obu_unit_size;
     int seq = 0;
     int ret, type, cnt = 0;
 
-    ffio_init_context(&pb, p->buf, p->buf_size, 0,
+    ffio_init_context(&ctx, p->buf, p->buf_size, 0,
                       NULL, NULL, NULL, NULL);
 
-    ret = leb(&pb, &temporal_unit_size);
+    ret = leb(pb, &temporal_unit_size);
     if (ret < 0)
         return 0;
     cnt += ret;
-    ret = leb(&pb, &frame_unit_size);
+    ret = leb(pb, &frame_unit_size);
     if (ret < 0 || ((int64_t)frame_unit_size + ret) > temporal_unit_size)
         return 0;
     cnt += ret;
-    ret = leb(&pb, &obu_unit_size);
+    ret = leb(pb, &obu_unit_size);
     if (ret < 0 || ((int64_t)obu_unit_size + ret) >= frame_unit_size)
         return 0;
     cnt += ret;
 
     frame_unit_size -= obu_unit_size + ret;
 
-    avio_skip(&pb, obu_unit_size);
-    if (pb.eof_reached || pb.error)
+    avio_skip(pb, obu_unit_size);
+    if (pb->eof_reached || pb->error)
         return 0;
 
     // Check that the first OBU is a Temporal Delimiter.
@@ -172,13 +196,13 @@ static int annexb_probe(const AVProbeData *p)
     cnt += obu_unit_size;
 
     do {
-        ret = leb(&pb, &obu_unit_size);
+        ret = leb(pb, &obu_unit_size);
         if (ret < 0 || ((int64_t)obu_unit_size + ret) > frame_unit_size)
             return 0;
         cnt += ret;
 
-        avio_skip(&pb, obu_unit_size);
-        if (pb.eof_reached || pb.error)
+        avio_skip(pb, obu_unit_size);
+        if (pb->eof_reached || pb->error)
             return 0;
 
         ret = read_obu(p->buf + cnt, FFMIN(p->buf_size - cnt, obu_unit_size), &obu_size, &type);
@@ -196,15 +220,9 @@ static int annexb_probe(const AVProbeData *p)
     return 0;
 }
 
-static int annexb_read_header(AVFormatContext *s)
-{
-    AnnexBContext *c = s->priv_data;
-    return read_header(s, &c->framerate, &c->bsf, c);
-}
-
 static int annexb_read_packet(AVFormatContext *s, AVPacket *pkt)
 {
-    AnnexBContext *c = s->priv_data;
+    AV1DemuxContext *const c = s->priv_data;
     uint32_t obu_unit_size;
     int ret, len;
 
@@ -259,50 +277,22 @@ end:
     return ret;
 }
 
-static int annexb_read_close(AVFormatContext *s)
-{
-    AnnexBContext *c = s->priv_data;
-
-    av_bsf_free(&c->bsf);
-    return 0;
-}
-
-#define OFFSET(x) offsetof(AnnexBContext, x)
-static const AVOption annexb_options[] = {
-    { "framerate", "", OFFSET(framerate), AV_OPT_TYPE_VIDEO_RATE, {.str = "25"}, 0, INT_MAX, DEC},
-    { NULL },
-};
-#undef OFFSET
-
-static const AVClass annexb_demuxer_class = {
-    .class_name = "AV1 Annex B demuxer",
-    .item_name  = av_default_item_name,
-    .option     = annexb_options,
-    .version    = LIBAVUTIL_VERSION_INT,
-};
-
-AVInputFormat ff_av1_demuxer = {
+const AVInputFormat ff_av1_demuxer = {
     .name           = "av1",
     .long_name      = NULL_IF_CONFIG_SMALL("AV1 Annex B"),
-    .priv_data_size = sizeof(AnnexBContext),
+    .priv_data_size = sizeof(AV1DemuxContext),
+    .flags_internal = FF_FMT_INIT_CLEANUP,
     .read_probe     = annexb_probe,
-    .read_header    = annexb_read_header,
+    .read_header    = av1_read_header,
     .read_packet    = annexb_read_packet,
-    .read_close     = annexb_read_close,
+    .read_close     = av1_read_close,
     .extensions     = "obu",
     .flags          = AVFMT_GENERIC_INDEX,
-    .priv_class     = &annexb_demuxer_class,
+    .priv_class     = &av1_demuxer_class,
 };
 #endif
 
 #if CONFIG_OBU_DEMUXER
-typedef struct ObuContext {
-    const AVClass *class;
-    AVBSFContext *bsf;
-    AVRational framerate;
-    AVFifoBuffer *fifo;
-} ObuContext;
-
 //For low overhead obu, we can't foresee the obu size before we parsed the header.
 //So, we can't use parse_obu_header here, since it will check size <= buf_size
 //see c27c7b49dc for more details
@@ -370,63 +360,51 @@ static int obu_probe(const AVProbeData *p)
     return 0;
 }
 
-static int obu_read_header(AVFormatContext *s)
-{
-    ObuContext *c = s->priv_data;
-    c->fifo = av_fifo_alloc(MAX_OBU_HEADER_SIZE);
-    if (!c->fifo)
-        return AVERROR(ENOMEM);
-    return read_header(s, &c->framerate, &c->bsf, c);
-}
-
 static int obu_get_packet(AVFormatContext *s, AVPacket *pkt)
 {
-    ObuContext *c = s->priv_data;
+    AV1DemuxContext *const c = s->priv_data;
     uint8_t header[MAX_OBU_HEADER_SIZE + AV_INPUT_BUFFER_PADDING_SIZE];
     int64_t obu_size;
-    int size = av_fifo_space(c->fifo);
+    int size;
     int ret, len, type;
 
-    av_fifo_generic_write(c->fifo, s->pb, size,
-                          (int (*)(void*, void*, int))avio_read);
-    size = av_fifo_size(c->fifo);
-    if (!size)
-        return 0;
-
-    av_fifo_generic_peek(c->fifo, header, size, NULL);
+    if ((ret = ffio_ensure_seekback(s->pb, MAX_OBU_HEADER_SIZE)) < 0)
+        return ret;
+    size = avio_read(s->pb, header, MAX_OBU_HEADER_SIZE);
+    if (size < 0)
+        return size;
 
+    memset(header + size, 0, AV_INPUT_BUFFER_PADDING_SIZE);
     len = read_obu_with_size(header, size, &obu_size, &type);
     if (len < 0) {
         av_log(c, AV_LOG_ERROR, "Failed to read obu\n");
         return len;
     }
+    avio_seek(s->pb, -size, SEEK_CUR);
 
-    ret = av_new_packet(pkt, len);
-    if (ret < 0) {
-        av_log(c, AV_LOG_ERROR, "Failed to allocate packet for obu\n");
-        return ret;
-    }
-    size = FFMIN(size, len);
-    av_fifo_generic_read(c->fifo, pkt->data, size, NULL);
-    len -= size;
-    if (len > 0) {
-        ret = avio_read(s->pb, pkt->data + size, len);
-        if (ret != len) {
-            av_log(c, AV_LOG_ERROR, "Failed to read %d frome file\n", len);
-            return ret < 0 ? ret : AVERROR_INVALIDDATA;
-        }
+    ret = av_get_packet(s->pb, pkt, len);
+    if (ret != len) {
+        av_log(c, AV_LOG_ERROR, "Failed to get packet for obu\n");
+        return ret < 0 ? ret : AVERROR_INVALIDDATA;
     }
     return 0;
 }
 
 static int obu_read_packet(AVFormatContext *s, AVPacket *pkt)
 {
-    ObuContext *c = s->priv_data;
+    AV1DemuxContext *const c = s->priv_data;
     int ret;
 
+    if (s->io_repositioned) {
+        av_bsf_flush(c->bsf);
+        s->io_repositioned = 0;
+    }
     while (1) {
         ret = obu_get_packet(s, pkt);
-        if (ret < 0)
+        /* In case of AVERROR_EOF we need to flush the BSF. Conveniently
+         * obu_get_packet() returns a blank pkt in this case which
+         * can be used to signal that the BSF should be flushed. */
+        if (ret < 0 && ret != AVERROR_EOF)
             return ret;
         ret = av_bsf_send_packet(c->bsf, pkt);
         if (ret < 0) {
@@ -445,39 +423,17 @@ static int obu_read_packet(AVFormatContext *s, AVPacket *pkt)
     return ret;
 }
 
-static int obu_read_close(AVFormatContext *s)
-{
-    ObuContext *c = s->priv_data;
-
-    av_fifo_freep(&c->fifo);
-    av_bsf_free(&c->bsf);
-    return 0;
-}
-
-#define OFFSET(x) offsetof(ObuContext, x)
-static const AVOption obu_options[] = {
-    { "framerate", "", OFFSET(framerate), AV_OPT_TYPE_VIDEO_RATE, {.str = "25"}, 0, INT_MAX, DEC},
-    { NULL },
-};
-#undef OFFSET
-
-static const AVClass obu_demuxer_class = {
-    .class_name = "AV1 low overhead OBU demuxer",
-    .item_name  = av_default_item_name,
-    .option     = obu_options,
-    .version    = LIBAVUTIL_VERSION_INT,
-};
-
-AVInputFormat ff_obu_demuxer = {
+const AVInputFormat ff_obu_demuxer = {
     .name           = "obu",
     .long_name      = NULL_IF_CONFIG_SMALL("AV1 low overhead OBU"),
-    .priv_data_size = sizeof(ObuContext),
+    .priv_data_size = sizeof(AV1DemuxContext),
+    .flags_internal = FF_FMT_INIT_CLEANUP,
     .read_probe     = obu_probe,
-    .read_header    = obu_read_header,
+    .read_header    = av1_read_header,
     .read_packet    = obu_read_packet,
-    .read_close     = obu_read_close,
+    .read_close     = av1_read_close,
     .extensions     = "obu",
-    .flags          = AVFMT_GENERIC_INDEX,
-    .priv_class     = &obu_demuxer_class,
+    .flags          = AVFMT_GENERIC_INDEX | AVFMT_NO_BYTE_SEEK,
+    .priv_class     = &av1_demuxer_class,
 };
 #endif
