@@ -31,15 +31,14 @@
 
 #include "libavutil/time_internal.h"
 #include "avformat.h"
-#include "internal.h"
 #include "libavcodec/dv_profile.h"
 #include "libavcodec/dv.h"
 #include "dv.h"
+#include "mux.h"
 #include "libavutil/avassert.h"
 #include "libavutil/fifo.h"
 #include "libavutil/mathematics.h"
 #include "libavutil/intreadwrite.h"
-#include "libavutil/opt.h"
 #include "libavutil/timecode.h"
 
 #define MAX_AUDIO_FRAME_SIZE 192000 // 1 second of 48khz 32-bit audio
@@ -49,7 +48,7 @@ struct DVMuxContext {
     const AVDVProfile*  sys;           /* current DV profile, e.g.: 525/60, 625/50 */
     int               n_ast;         /* number of stereo audio streams (up to 2) */
     AVStream         *ast[4];        /* stereo audio streams */
-    AVFifoBuffer     *audio_data[4]; /* FIFO for storing excessive amounts of PCM */
+    AVFifo           *audio_data[4]; /* FIFO for storing excessive amounts of PCM */
     int               frames;        /* current frame number */
     int64_t           start_time;    /* recording start time */
     int               has_audio;     /* frame under construction has audio */
@@ -201,8 +200,9 @@ static void dv_inject_audio(DVMuxContext *c, int channel, uint8_t* frame_ptr)
                 if (of*2 >= size)
                     continue;
 
-                frame_ptr[d]   = *av_fifo_peek2(c->audio_data[channel], of*2+1); // FIXME: maybe we have to admit
-                frame_ptr[d+1] = *av_fifo_peek2(c->audio_data[channel], of*2);   //        that DV is a big-endian PCM
+                // FIXME: maybe we have to admit that DV is a big-endian PCM
+                av_fifo_peek(c->audio_data[channel], frame_ptr + d, 2, of * 2);
+                FFSWAP(uint8_t, frame_ptr[d], frame_ptr[d + 1]);
             }
             frame_ptr += 16 * 80; /* 15 Video DIFs + 1 Audio DIF */
         }
@@ -245,7 +245,7 @@ static void dv_inject_metadata(DVMuxContext *c, uint8_t* frame)
 
 static int dv_assemble_frame(AVFormatContext *s,
                              DVMuxContext *c, AVStream* st,
-                             uint8_t* data, int data_size, uint8_t** frame)
+                             const uint8_t *data, int data_size, uint8_t **frame)
 {
     int i, reqasize;
 
@@ -254,8 +254,10 @@ static int dv_assemble_frame(AVFormatContext *s,
     switch (st->codecpar->codec_type) {
     case AVMEDIA_TYPE_VIDEO:
         /* FIXME: we have to have more sensible approach than this one */
-        if (c->has_video)
+        if (c->has_video) {
             av_log(s, AV_LOG_ERROR, "Can't process DV frame #%d. Insufficient audio data or severe sync problem.\n", c->frames);
+            return AVERROR(EINVAL);
+        }
         if (data_size != c->sys->frame_size) {
             av_log(s, AV_LOG_ERROR, "Unexpected frame size, %d != %d\n",
                    data_size, c->sys->frame_size);
@@ -269,14 +271,16 @@ static int dv_assemble_frame(AVFormatContext *s,
         for (i = 0; i < c->n_ast && st != c->ast[i]; i++);
 
           /* FIXME: we have to have more sensible approach than this one */
-        if (av_fifo_size(c->audio_data[i]) + data_size >= 100*MAX_AUDIO_FRAME_SIZE)
+        if (av_fifo_can_write(c->audio_data[i]) < data_size) {
             av_log(s, AV_LOG_ERROR, "Can't process DV frame #%d. Insufficient video data or severe sync problem.\n", c->frames);
-        av_fifo_generic_write(c->audio_data[i], data, data_size, NULL);
+            return AVERROR(EINVAL);
+        }
+        av_fifo_write(c->audio_data[i], data, data_size);
 
         reqasize = 4 * dv_audio_frame_size(c->sys, c->frames, st->codecpar->sample_rate);
 
         /* Let us see if we've got enough audio for one DV frame. */
-        c->has_audio |= ((reqasize <= av_fifo_size(c->audio_data[i])) << i);
+        c->has_audio |= ((reqasize <= av_fifo_can_read(c->audio_data[i])) << i);
 
         break;
     default:
@@ -290,8 +294,8 @@ static int dv_assemble_frame(AVFormatContext *s,
         for (i=0; i < c->n_ast; i++) {
             dv_inject_audio(c, i, *frame);
             reqasize = 4 * dv_audio_frame_size(c->sys, c->frames, c->ast[i]->codecpar->sample_rate);
-            av_fifo_drain(c->audio_data[i], reqasize);
-            c->has_audio |= ((reqasize <= av_fifo_size(c->audio_data[i])) << i);
+            av_fifo_drain2(c->audio_data[i], reqasize);
+            c->has_audio |= ((reqasize <= av_fifo_can_read(c->audio_data[i])) << i);
         }
 
         c->has_video = 0;
@@ -328,7 +332,7 @@ static DVMuxContext* dv_init_mux(AVFormatContext* s)
             if (c->n_ast > 1) return NULL;
             /* Some checks -- DV format is very picky about its incoming streams */
             if(st->codecpar->codec_id    != AV_CODEC_ID_PCM_S16LE ||
-               st->codecpar->channels    != 2)
+               st->codecpar->ch_layout.nb_channels    != 2)
                 goto bail_out;
             if (st->codecpar->sample_rate != 48000 &&
                 st->codecpar->sample_rate != 44100 &&
@@ -369,9 +373,11 @@ static DVMuxContext* dv_init_mux(AVFormatContext* s)
     ff_parse_creation_time_metadata(s, &c->start_time, 1);
 
     for (i=0; i < c->n_ast; i++) {
-        if (c->ast[i] && !(c->audio_data[i]=av_fifo_alloc_array(100, MAX_AUDIO_FRAME_SIZE))) {
+        if (!c->ast[i])
+           continue;
+        c->audio_data[i] = av_fifo_alloc2(100 * MAX_AUDIO_FRAME_SIZE, 1, 0);
+        if (!c->audio_data[i])
             goto bail_out;
-        }
     }
 
     return c;
@@ -433,10 +439,10 @@ static void dv_deinit(AVFormatContext *s)
     DVMuxContext *c = s->priv_data;
 
     for (int i = 0; i < c->n_ast; i++)
-        av_fifo_freep(&c->audio_data[i]);
+        av_fifo_freep2(&c->audio_data[i]);
 }
 
-AVOutputFormat ff_dv_muxer = {
+const AVOutputFormat ff_dv_muxer = {
     .name              = "dv",
     .long_name         = NULL_IF_CONFIG_SMALL("DV (Digital Video)"),
     .extensions        = "dv",
