@@ -19,6 +19,8 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
+#include <stdatomic.h>
+
 #include "libavutil/avassert.h"
 #include "libavutil/opt.h"
 #include "libavutil/time.h"
@@ -26,6 +28,7 @@
 #include "libavutil/threadmessage.h"
 #include "avformat.h"
 #include "internal.h"
+#include "mux.h"
 
 #define FIFO_DEFAULT_QUEUE_SIZE              60
 #define FIFO_DEFAULT_MAX_RECOVERY_ATTEMPTS   0
@@ -77,6 +80,9 @@ typedef struct FifoContext {
     /* Value > 0 signals queue overflow */
     volatile uint8_t overflow_flag;
 
+    atomic_int_least64_t queue_duration;
+    int64_t last_sent_dts;
+    int64_t timeshift;
 } FifoContext;
 
 typedef struct FifoThreadContext {
@@ -98,9 +104,12 @@ typedef struct FifoThreadContext {
      * so finalization by calling write_trailer and ff_io_close must be done
      * before exiting / reinitialization of underlying muxer */
     uint8_t header_written;
+
+    int64_t last_received_dts;
 } FifoThreadContext;
 
 typedef enum FifoMessageType {
+    FIFO_NOOP,
     FIFO_WRITE_HEADER,
     FIFO_WRITE_PACKET,
     FIFO_FLUSH_OUTPUT
@@ -121,7 +130,7 @@ static int fifo_thread_write_header(FifoThreadContext *ctx)
 
     ret = av_dict_copy(&format_options, fifo->format_options, 0);
     if (ret < 0)
-        return ret;
+        goto end;
 
     ret = ff_format_output_open(avf2, avf->url, &format_options);
     if (ret < 0) {
@@ -131,7 +140,7 @@ static int fifo_thread_write_header(FifoThreadContext *ctx)
     }
 
     for (i = 0;i < avf2->nb_streams; i++)
-        avf2->streams[i]->cur_dts = 0;
+        ffstream(avf2->streams[i])->cur_dts = 0;
 
     ret = avformat_write_header(avf2, &format_options);
     if (!ret)
@@ -159,6 +168,15 @@ static int fifo_thread_flush_output(FifoThreadContext *ctx)
     return av_write_frame(avf2, NULL);
 }
 
+static int64_t next_duration(AVFormatContext *avf, AVPacket *pkt, int64_t *last_dts)
+{
+    AVStream *st = avf->streams[pkt->stream_index];
+    int64_t dts = av_rescale_q(pkt->dts, st->time_base, AV_TIME_BASE_Q);
+    int64_t duration = (*last_dts == AV_NOPTS_VALUE ? 0 : dts - *last_dts);
+    *last_dts = dts;
+    return duration;
+}
+
 static int fifo_thread_write_packet(FifoThreadContext *ctx, AVPacket *pkt)
 {
     AVFormatContext *avf = ctx->avf;
@@ -166,6 +184,10 @@ static int fifo_thread_write_packet(FifoThreadContext *ctx, AVPacket *pkt)
     AVFormatContext *avf2 = fifo->avf;
     AVRational src_tb, dst_tb;
     int ret, s_idx;
+    int64_t orig_pts, orig_dts, orig_duration;
+
+    if (fifo->timeshift && pkt->dts != AV_NOPTS_VALUE)
+        atomic_fetch_sub_explicit(&fifo->queue_duration, next_duration(avf, pkt, &ctx->last_received_dts), memory_order_relaxed);
 
     if (ctx->drop_until_keyframe) {
         if (pkt->flags & AV_PKT_FLAG_KEY) {
@@ -178,14 +200,23 @@ static int fifo_thread_write_packet(FifoThreadContext *ctx, AVPacket *pkt)
         }
     }
 
+    orig_pts = pkt->pts;
+    orig_dts = pkt->dts;
+    orig_duration = pkt->duration;
     s_idx = pkt->stream_index;
     src_tb = avf->streams[s_idx]->time_base;
     dst_tb = avf2->streams[s_idx]->time_base;
     av_packet_rescale_ts(pkt, src_tb, dst_tb);
 
     ret = av_write_frame(avf2, pkt);
-    if (ret >= 0)
+    if (ret >= 0) {
         av_packet_unref(pkt);
+    } else {
+        // avoid scaling twice
+        pkt->pts = orig_pts;
+        pkt->dts = orig_dts;
+        pkt->duration = orig_duration;
+    }
     return ret;
 }
 
@@ -209,6 +240,9 @@ static int fifo_thread_dispatch_message(FifoThreadContext *ctx, FifoMessage *msg
 {
     int ret = AVERROR(EINVAL);
 
+    if (msg->type == FIFO_NOOP)
+        return 0;
+
     if (!ctx->header_written) {
         ret = fifo_thread_write_header(ctx);
         if (ret < 0)
@@ -390,12 +424,13 @@ static void *fifo_consumer_thread(void *data)
     AVFormatContext *avf = data;
     FifoContext *fifo = avf->priv_data;
     AVThreadMessageQueue *queue = fifo->queue;
-    FifoMessage msg = {FIFO_WRITE_HEADER, {0}};
+    FifoMessage msg = {fifo->timeshift ? FIFO_NOOP : FIFO_WRITE_HEADER, {0}};
     int ret;
 
     FifoThreadContext fifo_thread_ctx;
     memset(&fifo_thread_ctx, 0, sizeof(FifoThreadContext));
     fifo_thread_ctx.avf = avf;
+    fifo_thread_ctx.last_received_dts = AV_NOPTS_VALUE;
 
     while (1) {
         uint8_t just_flushed = 0;
@@ -429,6 +464,10 @@ static void *fifo_consumer_thread(void *data)
         if (just_flushed)
             av_log(avf, AV_LOG_INFO, "FIFO queue flushed\n");
 
+        if (fifo->timeshift)
+            while (atomic_load_explicit(&fifo->queue_duration, memory_order_relaxed) < fifo->timeshift)
+                av_usleep(10000);
+
         ret = av_thread_message_queue_recv(queue, &msg, 0);
         if (ret < 0) {
             av_thread_message_queue_set_err_send(queue, ret);
@@ -441,7 +480,7 @@ static void *fifo_consumer_thread(void *data)
     return NULL;
 }
 
-static int fifo_mux_init(AVFormatContext *avf, ff_const59 AVOutputFormat *oformat,
+static int fifo_mux_init(AVFormatContext *avf, const AVOutputFormat *oformat,
                          const char *filename)
 {
     FifoContext *fifo = avf->priv_data;
@@ -461,6 +500,7 @@ static int fifo_mux_init(AVFormatContext *avf, ff_const59 AVOutputFormat *oforma
         return ret;
     avf2->opaque = avf->opaque;
     avf2->io_close = avf->io_close;
+    avf2->io_close2 = avf->io_close2;
     avf2->io_open = avf->io_open;
     avf2->flags = avf->flags;
 
@@ -480,7 +520,7 @@ static int fifo_mux_init(AVFormatContext *avf, ff_const59 AVOutputFormat *oforma
 static int fifo_init(AVFormatContext *avf)
 {
     FifoContext *fifo = avf->priv_data;
-    ff_const59 AVOutputFormat *oformat;
+    const AVOutputFormat *oformat;
     int ret = 0;
 
     if (fifo->recovery_wait_streamtime && !fifo->drop_pkts_on_overflow) {
@@ -488,6 +528,8 @@ static int fifo_init(AVFormatContext *avf)
                " only when drop_pkts_on_overflow is also turned on\n");
         return AVERROR(EINVAL);
     }
+    atomic_init(&fifo->queue_duration, 0);
+    fifo->last_sent_dts = AV_NOPTS_VALUE;
 
     oformat = av_guess_format(fifo->format, avf->url, NULL);
     if (!oformat) {
@@ -563,6 +605,9 @@ static int fifo_write_packet(AVFormatContext *avf, AVPacket *pkt)
         goto fail;
     }
 
+    if (fifo->timeshift && pkt && pkt->dts != AV_NOPTS_VALUE)
+        atomic_fetch_add_explicit(&fifo->queue_duration, next_duration(avf, pkt, &fifo->last_sent_dts), memory_order_relaxed);
+
     return ret;
 fail:
     if (pkt)
@@ -576,6 +621,27 @@ static int fifo_write_trailer(AVFormatContext *avf)
     int ret;
 
     av_thread_message_queue_set_err_recv(fifo->queue, AVERROR_EOF);
+    if (fifo->timeshift) {
+        int64_t now = av_gettime_relative();
+        int64_t elapsed = 0;
+        FifoMessage msg = {FIFO_NOOP};
+        do {
+            int64_t delay = av_gettime_relative() - now;
+            if (delay < 0) { // Discontinuity?
+                delay = 10000;
+                now = av_gettime_relative();
+            } else {
+                now += delay;
+            }
+            atomic_fetch_add_explicit(&fifo->queue_duration, delay, memory_order_relaxed);
+            elapsed += delay;
+            if (elapsed > fifo->timeshift)
+                break;
+            av_usleep(10000);
+            ret = av_thread_message_queue_send(fifo->queue, &msg, AV_THREAD_MESSAGE_NONBLOCK);
+        } while (ret >= 0 || ret == AVERROR(EAGAIN));
+        atomic_store(&fifo->queue_duration, INT64_MAX);
+    }
 
     ret = pthread_join(fifo->writer_thread, NULL);
     if (ret < 0) {
@@ -630,6 +696,9 @@ static const AVOption options[] = {
         {"recover_any_error", "Attempt recovery regardless of type of the error", OFFSET(recover_any_error),
          AV_OPT_TYPE_BOOL, {.i64 = 0}, 0, 1, AV_OPT_FLAG_ENCODING_PARAM},
 
+        {"timeshift", "Delay fifo output", OFFSET(timeshift),
+         AV_OPT_TYPE_DURATION, {.i64 = 0}, 0, INT64_MAX, AV_OPT_FLAG_ENCODING_PARAM},
+
         {NULL},
 };
 
@@ -640,7 +709,7 @@ static const AVClass fifo_muxer_class = {
     .version    = LIBAVUTIL_VERSION_INT,
 };
 
-AVOutputFormat ff_fifo_muxer = {
+const AVOutputFormat ff_fifo_muxer = {
     .name           = "fifo",
     .long_name      = NULL_IF_CONFIG_SMALL("FIFO queue pseudo-muxer"),
     .priv_data_size = sizeof(FifoContext),
