@@ -23,6 +23,11 @@
 
 #include "libavutil/adler32.h"
 #include "libavutil/avstring.h"
+
+#include "libavcodec/codec_id.h"
+#include "libavcodec/codec_par.h"
+#include "libavcodec/packet.h"
+
 #include "avformat.h"
 #include "internal.h"
 
@@ -52,23 +57,11 @@ static int framecrc_write_packet(struct AVFormatContext *s, AVPacket *pkt)
     if (pkt->flags != AV_PKT_FLAG_KEY)
         av_strlcatf(buf, sizeof(buf), ", F=0x%0X", pkt->flags);
     if (pkt->side_data_elems) {
-        int i, j;
         av_strlcatf(buf, sizeof(buf), ", S=%d", pkt->side_data_elems);
 
-        for (i=0; i<pkt->side_data_elems; i++) {
-            uint32_t side_data_crc = 0;
-            if (HAVE_BIGENDIAN && AV_PKT_DATA_PALETTE == pkt->side_data[i].type) {
-                for (j=0; j<pkt->side_data[i].size; j++) {
-                    side_data_crc = av_adler32_update(side_data_crc,
-                                                      pkt->side_data[i].data + (j^3),
-                                                      1);
-                }
-            } else {
-                side_data_crc = av_adler32_update(0,
-                                                  pkt->side_data[i].data,
-                                                  pkt->side_data[i].size);
-            }
-            av_strlcatf(buf, sizeof(buf), ", %8d, 0x%08"PRIx32, pkt->side_data[i].size, side_data_crc);
+        for (int i = 0; i < pkt->side_data_elems; i++) {
+            av_strlcatf(buf, sizeof(buf), ", %8"SIZE_SPECIFIER,
+                        pkt->side_data[i].size);
         }
     }
     av_strlcatf(buf, sizeof(buf), "\n");
@@ -76,7 +69,7 @@ static int framecrc_write_packet(struct AVFormatContext *s, AVPacket *pkt)
     return 0;
 }
 
-AVOutputFormat ff_framecrc_muxer = {
+const AVOutputFormat ff_framecrc_muxer = {
     .name              = "framecrc",
     .long_name         = NULL_IF_CONFIG_SMALL("framecrc testing"),
     .audio_codec       = AV_CODEC_ID_PCM_S16LE,
