@@ -31,6 +31,7 @@
 #endif
 
 #include "libavutil/avstring.h"
+#include "libavutil/channel_layout.h"
 #include "libavutil/opt.h"
 #include "avformat.h"
 #include "internal.h"
@@ -39,11 +40,10 @@ typedef struct OpenMPTContext {
     const AVClass *class;
     openmpt_module *module;
 
-    int channels;
     double duration;
     /* options */
     int sample_rate;
-    int64_t layout;
+    AVChannelLayout ch_layout;
     int subsong;
 } OpenMPTContext;
 
@@ -52,7 +52,7 @@ typedef struct OpenMPTContext {
 #define D AV_OPT_FLAG_DECODING_PARAM
 static const AVOption options[] = {
     { "sample_rate", "set sample rate",    OFFSET(sample_rate), AV_OPT_TYPE_INT,            { .i64 = 48000 },               1000, INT_MAX,   A | D },
-    { "layout",      "set channel layout", OFFSET(layout),      AV_OPT_TYPE_CHANNEL_LAYOUT, { .i64 = AV_CH_LAYOUT_STEREO }, 0,    INT64_MAX, A | D },
+    { "layout",      "set channel layout", OFFSET(ch_layout),   AV_OPT_TYPE_CHLAYOUT,       { .str = "stereo" },            0,    0,         A | D },
     { "subsong",     "set subsong",        OFFSET(subsong),     AV_OPT_TYPE_INT,            { .i64 = -2 },                  -2,   INT_MAX,   A | D, "subsong"},
     { "all",         "all",                0,                   AV_OPT_TYPE_CONST,          { .i64 = -1},                   0,    0,         A | D, "subsong" },
     { "auto",        "auto",               0,                   AV_OPT_TYPE_CONST,          { .i64 = -2},                   0,    0,         A | D, "subsong" },
@@ -119,10 +119,7 @@ static int read_header_openmpt(AVFormatContext *s)
             return AVERROR_INVALIDDATA;
 #endif
 
-    openmpt->channels = av_get_channel_layout_nb_channels(openmpt->layout);
-
     if (openmpt->subsong >= openmpt_module_get_num_subsongs(openmpt->module)) {
-        openmpt_module_destroy(openmpt->module);
         av_log(s, AV_LOG_ERROR, "Invalid subsong index: %d\n", openmpt->subsong);
         return AVERROR(EINVAL);
     }
@@ -133,7 +130,6 @@ static int read_header_openmpt(AVFormatContext *s)
         }
         ret = openmpt_module_select_subsong(openmpt->module, openmpt->subsong);
         if (!ret){
-            openmpt_module_destroy(openmpt->module);
             av_log(s, AV_LOG_ERROR, "Could not select requested subsong: %d", openmpt->subsong);
             return AVERROR(EINVAL);
         }
@@ -148,18 +144,18 @@ static int read_header_openmpt(AVFormatContext *s)
     add_meta(s, "date",    openmpt_module_get_metadata(openmpt->module, "date"));
 
     st = avformat_new_stream(s, NULL);
-    if (!st) {
-        openmpt_module_destroy(openmpt->module);
-        openmpt->module = NULL;
+    if (!st)
         return AVERROR(ENOMEM);
-    }
     avpriv_set_pts_info(st, 64, 1, AV_TIME_BASE);
-    st->duration = llrint(openmpt->duration*AV_TIME_BASE);
+    if (openmpt->duration >= 0 && openmpt->duration < ((double)INT64_MAX + 1) / AV_TIME_BASE)
+        st->duration = llrint(openmpt->duration*AV_TIME_BASE);
 
     st->codecpar->codec_type  = AVMEDIA_TYPE_AUDIO;
     st->codecpar->codec_id    = AV_NE(AV_CODEC_ID_PCM_F32BE, AV_CODEC_ID_PCM_F32LE);
-    st->codecpar->channels    = openmpt->channels;
     st->codecpar->sample_rate = openmpt->sample_rate;
+    ret = av_channel_layout_copy(&st->codecpar->ch_layout, &openmpt->ch_layout);
+    if (ret < 0)
+        return ret;
 
     return 0;
 }
@@ -169,13 +165,15 @@ static int read_header_openmpt(AVFormatContext *s)
 static int read_packet_openmpt(AVFormatContext *s, AVPacket *pkt)
 {
     OpenMPTContext *openmpt = s->priv_data;
-    int n_samples = AUDIO_PKT_SIZE / (openmpt->channels ? openmpt->channels*4 : 4);
+    int n_samples = AUDIO_PKT_SIZE / (openmpt->ch_layout.nb_channels ? openmpt->ch_layout.nb_channels*4 : 4);
     int ret;
 
     if ((ret = av_new_packet(pkt, AUDIO_PKT_SIZE)) < 0)
         return ret;
 
-    switch (openmpt->channels) {
+    double pos = openmpt_module_get_position_seconds(openmpt->module);
+
+    switch (openmpt->ch_layout.nb_channels) {
     case 1:
         ret = openmpt_module_read_float_mono(openmpt->module, openmpt->sample_rate,
                                              n_samples, (float *)pkt->data);
@@ -189,7 +187,7 @@ static int read_packet_openmpt(AVFormatContext *s, AVPacket *pkt)
                                                          n_samples, (float *)pkt->data);
         break;
     default:
-        av_log(s, AV_LOG_ERROR, "Unsupported number of channels: %d", openmpt->channels);
+        av_log(s, AV_LOG_ERROR, "Unsupported number of channels: %d", openmpt->ch_layout.nb_channels);
         return AVERROR(EINVAL);
     }
 
@@ -198,7 +196,10 @@ static int read_packet_openmpt(AVFormatContext *s, AVPacket *pkt)
         return AVERROR_EOF;
     }
 
-    pkt->size = ret * (openmpt->channels * 4);
+    pkt->size = ret * (openmpt->ch_layout.nb_channels * 4);
+
+    if (pos >= 0 && pos < ((double)INT64_MAX + 1) / AV_TIME_BASE)
+        pkt->pts = llrint(pos * AV_TIME_BASE);
 
     return 0;
 }
@@ -206,14 +207,18 @@ static int read_packet_openmpt(AVFormatContext *s, AVPacket *pkt)
 static int read_close_openmpt(AVFormatContext *s)
 {
     OpenMPTContext *openmpt = s->priv_data;
-    openmpt_module_destroy(openmpt->module);
-    openmpt->module = NULL;
+    if (openmpt->module) {
+        openmpt_module_destroy(openmpt->module);
+        openmpt->module = NULL;
+    }
     return 0;
 }
 
 static int read_seek_openmpt(AVFormatContext *s, int stream_idx, int64_t ts, int flags)
 {
     OpenMPTContext *openmpt = s->priv_data;
+    if (ts < 0)
+        ts = 0;
     openmpt_module_set_position_seconds(openmpt->module, (double)ts/AV_TIME_BASE);
     return 0;
 }
@@ -281,10 +286,11 @@ static const AVClass class_openmpt = {
     .version    = LIBAVUTIL_VERSION_INT,
 };
 
-AVInputFormat ff_libopenmpt_demuxer = {
+const AVInputFormat ff_libopenmpt_demuxer = {
     .name           = "libopenmpt",
     .long_name      = NULL_IF_CONFIG_SMALL("Tracker formats (libopenmpt)"),
     .priv_data_size = sizeof(OpenMPTContext),
+    .flags_internal = FF_FMT_INIT_CLEANUP,
     .read_probe     = read_probe_openmpt,
     .read_header    = read_header_openmpt,
     .read_packet    = read_packet_openmpt,
