@@ -77,7 +77,9 @@ static int64_t count_ts(const char *p)
 static int64_t read_ts(const char *p, int64_t *start)
 {
     int64_t offset = 0;
-    uint64_t mm, ss, cs;
+    uint32_t mm;
+    double ss;
+    char prefix[3];
 
     while(p[offset] == ' ' || p[offset] == '\t') {
         offset++;
@@ -85,14 +87,14 @@ static int64_t read_ts(const char *p, int64_t *start)
     if(p[offset] != '[') {
         return 0;
     }
-    if(sscanf(p, "[-%"SCNu64":%"SCNu64".%"SCNu64"]", &mm, &ss, &cs) == 3) {
-        /* Just in case negative pts, players may drop it but we won't. */
-        *start = -(int64_t) (mm*60000 + ss*1000 + cs*10);
-    } else if(sscanf(p, "[%"SCNu64":%"SCNu64".%"SCNu64"]", &mm, &ss, &cs) == 3) {
-        *start = mm*60000 + ss*1000 + cs*10;
-    } else {
+    int ret = sscanf(p, "%2[[-]%"SCNu32":%lf]", prefix, &mm, &ss);
+    if (ret != 3 || prefix[0] != '[' || ss < 0 || ss > 60) {
         return 0;
     }
+    *start = (mm * 60 + ss) * AV_TIME_BASE;
+    if (prefix[1] == '-') {
+        *start = - *start;
+    }
     do {
         offset++;
     } while(p[offset] && p[offset-1] != ']');
@@ -163,7 +165,7 @@ static int lrc_read_header(AVFormatContext *s)
     if(!st) {
         return AVERROR(ENOMEM);
     }
-    avpriv_set_pts_info(st, 64, 1, 1000);
+    avpriv_set_pts_info(st, 64, 1, AV_TIME_BASE);
     lrc->ts_offset = 0;
     st->codecpar->codec_type = AVMEDIA_TYPE_SUBTITLE;
     st->codecpar->codec_id   = AV_CODEC_ID_TEXT;
@@ -204,10 +206,8 @@ static int lrc_read_header(AVFormatContext *s)
                 ts_stroffset += ts_stroffset_incr;
                 sub = ff_subtitles_queue_insert(&lrc->q, line.str + ts_strlength,
                                                 line.len - ts_strlength, 0);
-                if(!sub) {
-                    ff_subtitles_queue_clean(&lrc->q);
+                if (!sub)
                     return AVERROR(ENOMEM);
-                }
                 sub->pos = pos;
                 sub->pts = ts_start - lrc->ts_offset;
                 sub->duration = -1;
@@ -220,34 +220,14 @@ static int lrc_read_header(AVFormatContext *s)
     return 0;
 }
 
-static int lrc_read_packet(AVFormatContext *s, AVPacket *pkt)
-{
-    LRCContext *lrc = s->priv_data;
-    return ff_subtitles_queue_read_packet(&lrc->q, pkt);
-}
-
-static int lrc_read_seek(AVFormatContext *s, int stream_index,
-                         int64_t min_ts, int64_t ts, int64_t max_ts, int flags)
-{
-    LRCContext *lrc = s->priv_data;
-    return ff_subtitles_queue_seek(&lrc->q, s, stream_index,
-                                   min_ts, ts, max_ts, flags);
-}
-
-static int lrc_read_close(AVFormatContext *s)
-{
-    LRCContext *lrc = s->priv_data;
-    ff_subtitles_queue_clean(&lrc->q);
-    return 0;
-}
-
-AVInputFormat ff_lrc_demuxer = {
+const AVInputFormat ff_lrc_demuxer = {
     .name           = "lrc",
     .long_name      = NULL_IF_CONFIG_SMALL("LRC lyrics"),
     .priv_data_size = sizeof (LRCContext),
+    .flags_internal = FF_FMT_INIT_CLEANUP,
     .read_probe     = lrc_probe,
     .read_header    = lrc_read_header,
-    .read_packet    = lrc_read_packet,
-    .read_close     = lrc_read_close,
-    .read_seek2     = lrc_read_seek
+    .read_packet    = ff_subtitles_read_packet,
+    .read_close     = ff_subtitles_read_close,
+    .read_seek2     = ff_subtitles_read_seek
 };
