@@ -20,11 +20,13 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
+#include "config_components.h"
+
 #include "libavutil/intreadwrite.h"
 
 #include "avformat.h"
 #include "rawenc.h"
-#include "internal.h"
+#include "mux.h"
 
 int ff_raw_write_packet(AVFormatContext *s, AVPacket *pkt)
 {
@@ -57,7 +59,7 @@ static int force_one_stream(AVFormatContext *s)
 /* Note: Do not forget to add new entries to the Makefile as well. */
 
 #if CONFIG_AC3_MUXER
-AVOutputFormat ff_ac3_muxer = {
+const AVOutputFormat ff_ac3_muxer = {
     .name              = "ac3",
     .long_name         = NULL_IF_CONFIG_SMALL("raw AC-3"),
     .mime_type         = "audio/x-ac3",
@@ -79,7 +81,7 @@ static int adx_write_trailer(AVFormatContext *s)
 
     if (pb->seekable & AVIO_SEEKABLE_NORMAL) {
         int64_t file_size = avio_tell(pb);
-        uint64_t sample_count = (file_size - 36) / par->channels / 18 * 32;
+        uint64_t sample_count = (file_size - 36) / par->ch_layout.nb_channels / 18 * 32;
         if (sample_count <= UINT32_MAX) {
             avio_seek(pb, 12, SEEK_SET);
             avio_wb32(pb, sample_count);
@@ -90,7 +92,7 @@ static int adx_write_trailer(AVFormatContext *s)
     return 0;
 }
 
-AVOutputFormat ff_adx_muxer = {
+const AVOutputFormat ff_adx_muxer = {
     .name              = "adx",
     .long_name         = NULL_IF_CONFIG_SMALL("CRI ADX"),
     .extensions        = "adx",
@@ -104,7 +106,7 @@ AVOutputFormat ff_adx_muxer = {
 #endif
 
 #if CONFIG_APTX_MUXER
-AVOutputFormat ff_aptx_muxer = {
+const AVOutputFormat ff_aptx_muxer = {
     .name              = "aptx",
     .long_name         = NULL_IF_CONFIG_SMALL("raw aptX (Audio Processing Technology for Bluetooth)"),
     .extensions        = "aptx",
@@ -117,7 +119,7 @@ AVOutputFormat ff_aptx_muxer = {
 #endif
 
 #if CONFIG_APTX_HD_MUXER
-AVOutputFormat ff_aptx_hd_muxer = {
+const AVOutputFormat ff_aptx_hd_muxer = {
     .name              = "aptx_hd",
     .long_name         = NULL_IF_CONFIG_SMALL("raw aptX HD (Audio Processing Technology for Bluetooth)"),
     .extensions        = "aptxhd",
@@ -130,7 +132,7 @@ AVOutputFormat ff_aptx_hd_muxer = {
 #endif
 
 #if CONFIG_AVS2_MUXER
-AVOutputFormat ff_avs2_muxer = {
+const AVOutputFormat ff_avs2_muxer = {
     .name              = "avs2",
     .long_name         = NULL_IF_CONFIG_SMALL("raw AVS2-P2/IEEE1857.4 video"),
     .extensions        = "avs,avs2",
@@ -142,8 +144,22 @@ AVOutputFormat ff_avs2_muxer = {
 };
 #endif
 
+#if CONFIG_AVS3_MUXER
+const AVOutputFormat ff_avs3_muxer = {
+    .name              = "avs3",
+    .long_name         = NULL_IF_CONFIG_SMALL("AVS3-P2/IEEE1857.10"),
+    .extensions        = "avs3",
+    .audio_codec       = AV_CODEC_ID_NONE,
+    .video_codec       = AV_CODEC_ID_AVS3,
+    .init              = force_one_stream,
+    .write_packet      = ff_raw_write_packet,
+    .flags             = AVFMT_NOTIMESTAMPS,
+};
+#endif
+
+
 #if CONFIG_CAVSVIDEO_MUXER
-AVOutputFormat ff_cavsvideo_muxer = {
+const AVOutputFormat ff_cavsvideo_muxer = {
     .name              = "cavsvideo",
     .long_name         = NULL_IF_CONFIG_SMALL("raw Chinese AVS (Audio Video Standard) video"),
     .extensions        = "cavs",
@@ -156,7 +172,7 @@ AVOutputFormat ff_cavsvideo_muxer = {
 #endif
 
 #if CONFIG_CODEC2RAW_MUXER
-AVOutputFormat ff_codec2raw_muxer = {
+const AVOutputFormat ff_codec2raw_muxer = {
     .name              = "codec2raw",
     .long_name         = NULL_IF_CONFIG_SMALL("raw codec2 muxer"),
     .audio_codec       = AV_CODEC_ID_CODEC2,
@@ -169,7 +185,7 @@ AVOutputFormat ff_codec2raw_muxer = {
 
 
 #if CONFIG_DATA_MUXER
-AVOutputFormat ff_data_muxer = {
+const AVOutputFormat ff_data_muxer = {
     .name              = "data",
     .long_name         = NULL_IF_CONFIG_SMALL("raw data"),
     .init              = force_one_stream,
@@ -178,8 +194,21 @@ AVOutputFormat ff_data_muxer = {
 };
 #endif
 
+#if CONFIG_DFPWM_MUXER
+const AVOutputFormat ff_dfpwm_muxer = {
+    .name              = "dfpwm",
+    .long_name         = NULL_IF_CONFIG_SMALL("raw DFPWM1a"),
+    .extensions        = "dfpwm",
+    .audio_codec       = AV_CODEC_ID_DFPWM,
+    .video_codec       = AV_CODEC_ID_NONE,
+    .init              = force_one_stream,
+    .write_packet      = ff_raw_write_packet,
+    .flags             = AVFMT_NOTIMESTAMPS,
+};
+#endif
+
 #if CONFIG_DIRAC_MUXER
-AVOutputFormat ff_dirac_muxer = {
+const AVOutputFormat ff_dirac_muxer = {
     .name              = "dirac",
     .long_name         = NULL_IF_CONFIG_SMALL("raw Dirac"),
     .extensions        = "drc,vc2",
@@ -192,7 +221,7 @@ AVOutputFormat ff_dirac_muxer = {
 #endif
 
 #if CONFIG_DNXHD_MUXER
-AVOutputFormat ff_dnxhd_muxer = {
+const AVOutputFormat ff_dnxhd_muxer = {
     .name              = "dnxhd",
     .long_name         = NULL_IF_CONFIG_SMALL("raw DNxHD (SMPTE VC-3)"),
     .extensions        = "dnxhd,dnxhr",
@@ -205,7 +234,7 @@ AVOutputFormat ff_dnxhd_muxer = {
 #endif
 
 #if CONFIG_DTS_MUXER
-AVOutputFormat ff_dts_muxer = {
+const AVOutputFormat ff_dts_muxer = {
     .name              = "dts",
     .long_name         = NULL_IF_CONFIG_SMALL("raw DTS"),
     .mime_type         = "audio/x-dca",
@@ -219,7 +248,7 @@ AVOutputFormat ff_dts_muxer = {
 #endif
 
 #if CONFIG_EAC3_MUXER
-AVOutputFormat ff_eac3_muxer = {
+const AVOutputFormat ff_eac3_muxer = {
     .name              = "eac3",
     .long_name         = NULL_IF_CONFIG_SMALL("raw E-AC-3"),
     .mime_type         = "audio/x-eac3",
@@ -233,7 +262,7 @@ AVOutputFormat ff_eac3_muxer = {
 #endif
 
 #if CONFIG_G722_MUXER
-AVOutputFormat ff_g722_muxer = {
+const AVOutputFormat ff_g722_muxer = {
     .name              = "g722",
     .long_name         = NULL_IF_CONFIG_SMALL("raw G.722"),
     .mime_type         = "audio/G722",
@@ -247,7 +276,7 @@ AVOutputFormat ff_g722_muxer = {
 #endif
 
 #if CONFIG_G723_1_MUXER
-AVOutputFormat ff_g723_1_muxer = {
+const AVOutputFormat ff_g723_1_muxer = {
     .name              = "g723_1",
     .long_name         = NULL_IF_CONFIG_SMALL("raw G.723.1"),
     .mime_type         = "audio/g723",
@@ -261,7 +290,7 @@ AVOutputFormat ff_g723_1_muxer = {
 #endif
 
 #if CONFIG_G726_MUXER
-AVOutputFormat ff_g726_muxer = {
+const AVOutputFormat ff_g726_muxer = {
     .name              = "g726",
     .long_name         = NULL_IF_CONFIG_SMALL("raw big-endian G.726 (\"left-justified\")"),
     .audio_codec       = AV_CODEC_ID_ADPCM_G726,
@@ -273,7 +302,7 @@ AVOutputFormat ff_g726_muxer = {
 #endif
 
 #if CONFIG_G726LE_MUXER
-AVOutputFormat ff_g726le_muxer = {
+const AVOutputFormat ff_g726le_muxer = {
     .name              = "g726le",
     .long_name         = NULL_IF_CONFIG_SMALL("raw little-endian G.726 (\"right-justified\")"),
     .audio_codec       = AV_CODEC_ID_ADPCM_G726LE,
@@ -285,7 +314,7 @@ AVOutputFormat ff_g726le_muxer = {
 #endif
 
 #if CONFIG_GSM_MUXER
-AVOutputFormat ff_gsm_muxer = {
+const AVOutputFormat ff_gsm_muxer = {
     .name              = "gsm",
     .long_name         = NULL_IF_CONFIG_SMALL("raw GSM"),
     .mime_type         = "audio/x-gsm",
@@ -299,7 +328,7 @@ AVOutputFormat ff_gsm_muxer = {
 #endif
 
 #if CONFIG_H261_MUXER
-AVOutputFormat ff_h261_muxer = {
+const AVOutputFormat ff_h261_muxer = {
     .name              = "h261",
     .long_name         = NULL_IF_CONFIG_SMALL("raw H.261"),
     .mime_type         = "video/x-h261",
@@ -313,7 +342,7 @@ AVOutputFormat ff_h261_muxer = {
 #endif
 
 #if CONFIG_H263_MUXER
-AVOutputFormat ff_h263_muxer = {
+const AVOutputFormat ff_h263_muxer = {
     .name              = "h263",
     .long_name         = NULL_IF_CONFIG_SMALL("raw H.263"),
     .mime_type         = "video/x-h263",
@@ -327,16 +356,16 @@ AVOutputFormat ff_h263_muxer = {
 #endif
 
 #if CONFIG_H264_MUXER
-static int h264_check_bitstream(struct AVFormatContext *s, const AVPacket *pkt)
+static int h264_check_bitstream(AVFormatContext *s, AVStream *st,
+                                const AVPacket *pkt)
 {
-    AVStream *st = s->streams[0];
     if (pkt->size >= 5 && AV_RB32(pkt->data) != 0x0000001 &&
                           AV_RB24(pkt->data) != 0x000001)
         return ff_stream_add_bitstream_filter(st, "h264_mp4toannexb", NULL);
     return 1;
 }
 
-AVOutputFormat ff_h264_muxer = {
+const AVOutputFormat ff_h264_muxer = {
     .name              = "h264",
     .long_name         = NULL_IF_CONFIG_SMALL("raw H.264 video"),
     .extensions        = "h264,264",
@@ -350,16 +379,16 @@ AVOutputFormat ff_h264_muxer = {
 #endif
 
 #if CONFIG_HEVC_MUXER
-static int hevc_check_bitstream(struct AVFormatContext *s, const AVPacket *pkt)
+static int hevc_check_bitstream(AVFormatContext *s, AVStream *st,
+                                const AVPacket *pkt)
 {
-    AVStream *st = s->streams[0];
     if (pkt->size >= 5 && AV_RB32(pkt->data) != 0x0000001 &&
                           AV_RB24(pkt->data) != 0x000001)
         return ff_stream_add_bitstream_filter(st, "hevc_mp4toannexb", NULL);
     return 1;
 }
 
-AVOutputFormat ff_hevc_muxer = {
+const AVOutputFormat ff_hevc_muxer = {
     .name              = "hevc",
     .long_name         = NULL_IF_CONFIG_SMALL("raw HEVC video"),
     .extensions        = "hevc,h265,265",
@@ -373,7 +402,7 @@ AVOutputFormat ff_hevc_muxer = {
 #endif
 
 #if CONFIG_M4V_MUXER
-AVOutputFormat ff_m4v_muxer = {
+const AVOutputFormat ff_m4v_muxer = {
     .name              = "m4v",
     .long_name         = NULL_IF_CONFIG_SMALL("raw MPEG-4 video"),
     .extensions        = "m4v",
@@ -386,7 +415,7 @@ AVOutputFormat ff_m4v_muxer = {
 #endif
 
 #if CONFIG_MJPEG_MUXER
-AVOutputFormat ff_mjpeg_muxer = {
+const AVOutputFormat ff_mjpeg_muxer = {
     .name              = "mjpeg",
     .long_name         = NULL_IF_CONFIG_SMALL("raw MJPEG video"),
     .mime_type         = "video/x-mjpeg",
@@ -399,21 +428,8 @@ AVOutputFormat ff_mjpeg_muxer = {
 };
 #endif
 
-#if CONFIG_SINGLEJPEG_MUXER
-AVOutputFormat ff_singlejpeg_muxer = {
-    .name              = "singlejpeg",
-    .long_name         = NULL_IF_CONFIG_SMALL("JPEG single image"),
-    .mime_type         = "image/jpeg",
-    .audio_codec       = AV_CODEC_ID_NONE,
-    .video_codec       = AV_CODEC_ID_MJPEG,
-    .init              = force_one_stream,
-    .write_packet      = ff_raw_write_packet,
-    .flags             = AVFMT_NOTIMESTAMPS,
-};
-#endif
-
 #if CONFIG_MLP_MUXER
-AVOutputFormat ff_mlp_muxer = {
+const AVOutputFormat ff_mlp_muxer = {
     .name              = "mlp",
     .long_name         = NULL_IF_CONFIG_SMALL("raw MLP"),
     .extensions        = "mlp",
@@ -426,20 +442,21 @@ AVOutputFormat ff_mlp_muxer = {
 #endif
 
 #if CONFIG_MP2_MUXER
-AVOutputFormat ff_mp2_muxer = {
+const AVOutputFormat ff_mp2_muxer = {
     .name              = "mp2",
     .long_name         = NULL_IF_CONFIG_SMALL("MP2 (MPEG audio layer 2)"),
     .mime_type         = "audio/mpeg",
     .extensions        = "mp2,m2a,mpa",
     .audio_codec       = AV_CODEC_ID_MP2,
     .video_codec       = AV_CODEC_ID_NONE,
+    .init              = force_one_stream,
     .write_packet      = ff_raw_write_packet,
     .flags             = AVFMT_NOTIMESTAMPS,
 };
 #endif
 
 #if CONFIG_MPEG1VIDEO_MUXER
-AVOutputFormat ff_mpeg1video_muxer = {
+const AVOutputFormat ff_mpeg1video_muxer = {
     .name              = "mpeg1video",
     .long_name         = NULL_IF_CONFIG_SMALL("raw MPEG-1 video"),
     .mime_type         = "video/mpeg",
@@ -453,7 +470,7 @@ AVOutputFormat ff_mpeg1video_muxer = {
 #endif
 
 #if CONFIG_MPEG2VIDEO_MUXER
-AVOutputFormat ff_mpeg2video_muxer = {
+const AVOutputFormat ff_mpeg2video_muxer = {
     .name              = "mpeg2video",
     .long_name         = NULL_IF_CONFIG_SMALL("raw MPEG-2 video"),
     .extensions        = "m2v",
@@ -465,8 +482,28 @@ AVOutputFormat ff_mpeg2video_muxer = {
 };
 #endif
 
+#if CONFIG_OBU_MUXER
+static int obu_check_bitstream(AVFormatContext *s, AVStream *st,
+                               const AVPacket *pkt)
+{
+    return ff_stream_add_bitstream_filter(st, "av1_metadata", "td=insert");
+}
+
+const AVOutputFormat ff_obu_muxer = {
+    .name              = "obu",
+    .long_name         = NULL_IF_CONFIG_SMALL("AV1 low overhead OBU"),
+    .extensions        = "obu",
+    .audio_codec       = AV_CODEC_ID_NONE,
+    .video_codec       = AV_CODEC_ID_AV1,
+    .init              = force_one_stream,
+    .write_packet      = ff_raw_write_packet,
+    .check_bitstream   = obu_check_bitstream,
+    .flags             = AVFMT_NOTIMESTAMPS,
+};
+#endif
+
 #if CONFIG_RAWVIDEO_MUXER
-AVOutputFormat ff_rawvideo_muxer = {
+const AVOutputFormat ff_rawvideo_muxer = {
     .name              = "rawvideo",
     .long_name         = NULL_IF_CONFIG_SMALL("raw video"),
     .extensions        = "yuv,rgb",
@@ -478,7 +515,7 @@ AVOutputFormat ff_rawvideo_muxer = {
 #endif
 
 #if CONFIG_SBC_MUXER
-AVOutputFormat ff_sbc_muxer = {
+const AVOutputFormat ff_sbc_muxer = {
     .name              = "sbc",
     .long_name         = NULL_IF_CONFIG_SMALL("raw SBC"),
     .mime_type         = "audio/x-sbc",
@@ -491,7 +528,7 @@ AVOutputFormat ff_sbc_muxer = {
 #endif
 
 #if CONFIG_TRUEHD_MUXER
-AVOutputFormat ff_truehd_muxer = {
+const AVOutputFormat ff_truehd_muxer = {
     .name              = "truehd",
     .long_name         = NULL_IF_CONFIG_SMALL("raw TrueHD"),
     .extensions        = "thd",
@@ -504,7 +541,7 @@ AVOutputFormat ff_truehd_muxer = {
 #endif
 
 #if CONFIG_VC1_MUXER
-AVOutputFormat ff_vc1_muxer = {
+const AVOutputFormat ff_vc1_muxer = {
     .name              = "vc1",
     .long_name         = NULL_IF_CONFIG_SMALL("raw VC-1 video"),
     .extensions        = "vc1",
