@@ -27,7 +27,6 @@
 #include <stdlib.h>
 #include <inttypes.h>
 
-#include "libavutil/cpu.h"
 #include "libavutil/bswap.h"
 #include "config.h"
 #include "rgb2rgb.h"
@@ -146,8 +145,8 @@ const int *sws_getCoefficients(int colorspace)
             dst_type av_unused *r, *g, *b;                                  \
             const uint8_t *py_1 = src[0] +  y       * srcStride[0];         \
             const uint8_t *py_2 = py_1   +            srcStride[0];         \
-            const uint8_t *pu   = src[1] + (y >> 1) * srcStride[1];         \
-            const uint8_t *pv   = src[2] + (y >> 1) * srcStride[2];         \
+            const uint8_t av_unused *pu = src[1] + (y >> 1) * srcStride[1]; \
+            const uint8_t av_unused *pv = src[2] + (y >> 1) * srcStride[2]; \
             const uint8_t av_unused *pa_1, *pa_2;                           \
             unsigned int h_size = c->dstW >> 3;                             \
             if (alpha) {                                                    \
@@ -680,10 +679,11 @@ SwsFunc ff_yuv2rgb_get_func_ptr(SwsContext *c)
 {
     SwsFunc t = NULL;
 
-    if (ARCH_PPC)
-        t = ff_yuv2rgb_init_ppc(c);
-    if (ARCH_X86)
-        t = ff_yuv2rgb_init_x86(c);
+#if ARCH_PPC
+    t = ff_yuv2rgb_init_ppc(c);
+#elif ARCH_X86
+    t = ff_yuv2rgb_init_x86(c);
+#endif
 
     if (t)
         return t;
@@ -784,6 +784,8 @@ av_cold int ff_yuv2rgb_c_init_tables(SwsContext *c, const int inv_table[4],
                       c->dstFormat == AV_PIX_FMT_RGB555LE  ||
                       c->dstFormat == AV_PIX_FMT_RGB444BE  ||
                       c->dstFormat == AV_PIX_FMT_RGB444LE  ||
+                      c->dstFormat == AV_PIX_FMT_X2RGB10BE ||
+                      c->dstFormat == AV_PIX_FMT_X2RGB10LE ||
                       c->dstFormat == AV_PIX_FMT_RGB8      ||
                       c->dstFormat == AV_PIX_FMT_RGB4      ||
                       c->dstFormat == AV_PIX_FMT_RGB4_BYTE ||
@@ -794,7 +796,8 @@ av_cold int ff_yuv2rgb_c_init_tables(SwsContext *c, const int inv_table[4],
                         c->dstFormat == AV_PIX_FMT_NE(BGR565LE, BGR565BE) ||
                         c->dstFormat == AV_PIX_FMT_NE(BGR555LE, BGR555BE) ||
                         c->dstFormat == AV_PIX_FMT_NE(BGR444LE, BGR444BE) ||
-                        c->dstFormat == AV_PIX_FMT_NE(X2RGB10LE, X2RGB10BE);
+                        c->dstFormat == AV_PIX_FMT_NE(X2RGB10LE, X2RGB10BE) ||
+                        c->dstFormat == AV_PIX_FMT_NE(X2BGR10LE, X2BGR10BE);
     const int bpp = c->dstFormatBpp;
     uint8_t *y_table;
     uint16_t *y_table16;
@@ -826,7 +829,7 @@ av_cold int ff_yuv2rgb_c_init_tables(SwsContext *c, const int inv_table[4],
     cbu  = (cbu * contrast * saturation) >> 32;
     cgu  = (cgu * contrast * saturation) >> 32;
     cgv  = (cgv * contrast * saturation) >> 32;
-    oy  -= 256 * brightness;
+    oy  -= 256LL * brightness;
 
     c->uOffset = 0x0400040004000400LL;
     c->vOffset = 0x0400040004000400LL;
@@ -967,9 +970,9 @@ av_cold int ff_yuv2rgb_c_init_tables(SwsContext *c, const int inv_table[4],
         fill_gv_table(c->table_gV, 1, cgv);
         break;
     case 30:
-        rbase = 20;
+        rbase = isRgb ? 20 : 0;
         gbase = 10;
-        bbase = 0;
+        bbase = isRgb ? 0 : 20;
         needAlpha = CONFIG_SWSCALE_ALPHA && isALPHA(c->srcFormat);
         if (!needAlpha)
             abase = 30;
@@ -977,7 +980,7 @@ av_cold int ff_yuv2rgb_c_init_tables(SwsContext *c, const int inv_table[4],
         y_table32   = c->yuvTable;
         yb = -(384 << 16) - YUVRGB_TABLE_LUMA_HEADROOM*cy - oy;
         for (i = 0; i < table_plane_size; i++) {
-            unsigned yval = av_clip_uint8((yb + 0x8000) >> 16);
+            unsigned yval = av_clip_uintp2((yb + 0x8000) >> 14, 10);
             y_table32[i]= (yval << rbase) + (needAlpha ? 0 : (255u << abase));
             y_table32[i + table_plane_size] = yval << gbase;
             y_table32[i + 2 * table_plane_size] = yval << bbase;
