@@ -20,7 +20,7 @@
 
 #include <string.h>
 #include "checkasm.h"
-#include "libavfilter/blend.h"
+#include "libavfilter/vf_blend_init.h"
 #include "libavutil/common.h"
 #include "libavutil/internal.h"
 #include "libavutil/intreadwrite.h"
@@ -71,7 +71,7 @@
         w = WIDTH / depth;                                                                 \
                                                                                            \
         for (i = 0; i < BUF_UNITS - 1; i++) {                                              \
-            int src_offset = i * SIZE_PER_UNIT + i; /* Test various alignments */          \
+            int src_offset = i * SIZE_PER_UNIT + (BUF_UNITS - 1 - i) * depth; /* Test various alignments */  \
             int dst_offset = i * SIZE_PER_UNIT; /* dst must be aligned */                  \
             randomize_buffers();                                                           \
             call_ref(top1 + src_offset, w, bot1 + src_offset, w,                           \
@@ -99,7 +99,7 @@ void checkasm_check_blend(void)
 
 #define check_and_report(name, val, depth)        \
     param.mode = val;                             \
-    ff_blend_init(&param, depth - 1);             \
+    ff_blend_init(&param, depth * 8);             \
     if (check_func(param.blend, #name))           \
         check_blend_func(depth);
 
